/* $Cambridge: hermes/src/prayer/prayer/cmd_abook_search.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static int abook_search_field_form_to_enum(char *key)
{
    int field = ABOOK_SEARCH_FIELD_NONE;

    if (!strcmp(key, "alias"))
        field = ABOOK_SEARCH_FIELD_ALIAS;
    else if (!strcmp(key, "name"))
        field = ABOOK_SEARCH_FIELD_NAME;
    else if (!strcmp(key, "comment"))
        field = ABOOK_SEARCH_FIELD_COMMENT;
    else if (!strcmp(key, "email"))
        field = ABOOK_SEARCH_FIELD_EMAIL;

    return (field);
}

static int abook_search_type_form_to_enum(char *op)
{
    int type = ABOOK_SEARCH_TYPE_NONE;

    if (!strcmp(op, "is"))
        type = ABOOK_SEARCH_TYPE_IS;
    else if (!strcmp(op, "begins"))
        type = ABOOK_SEARCH_TYPE_BEGINS;
    else if (!strcmp(op, "ends"))
        type = ABOOK_SEARCH_TYPE_ENDS;
    else if (!strcmp(op, "contains"))
        type = ABOOK_SEARCH_TYPE_CONTAINS;

    return (type);
}

static void generate_hdr(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Addressbook Search");
    html_banner_toolbar(session, b, "abook_search", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Addressbook Search</h2>" CRLF);

}

static void
generate_form(struct session *session, struct abook_search_list *asl)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct abook_search_elt *ase;
    unsigned long i;

    html_form_start(session, b, "POST", "abook_search");
    bprintf(b, "Search for entries where " CRLF);
    bprintf(b, "<select name=\"cond\" size=1>" CRLF);

    if (asl->match_all) {
        bprintf(b, "<option value=\"any\">any" CRLF);
        bprintf(b, "<option value=\"all\" selected>all" CRLF);
    } else {
        bprintf(b, "<option value=\"any\" selected>any" CRLF);
        bprintf(b, "<option value=\"all\">all" CRLF);
    }
    bprintf(b, "</select>" CRLF);
    bprintf(b, "of the following conditions hold" CRLF);
    bprintf(b, "<table>" CRLF);

    for (i = 0, ase = asl->first; ase; ase = ase->next, i++) {
        bprintf(b, "<tr><td>" CRLF);

        bprintf(b, "<select name=\"key%lu\" size=1>" CRLF, i);

        bprintf(b, "<option value=\"alias\"%s>Alias" CRLF,
                (ase->field ==
                 ABOOK_SEARCH_FIELD_ALIAS) ? " selected" : "");

        bprintf(b, "<option value=\"name\"%s>Name" CRLF,
                ((ase->field == ABOOK_SEARCH_FIELD_NONE) ||
                 (ase->field ==
                  ABOOK_SEARCH_FIELD_NAME)) ? " selected" : "");

        bprintf(b, "<option value=\"comment\"%s>Comment" CRLF,
                (ase->field ==
                 ABOOK_SEARCH_FIELD_COMMENT) ? " selected" : "");

        bprintf(b, "<option value=\"email\"%s>Address" CRLF,
                (ase->field ==
                 ABOOK_SEARCH_FIELD_EMAIL) ? " selected" : "");

        bprintf(b, "</select>" CRLF);
        bprintf(b, "</td><td>" CRLF);

        bprintf(b, "<select name=\"op%lu\" size=1>" CRLF, i);
        bprintf(b, "<option value=\"is\"%s>is" CRLF,
                (ase->type == ABOOK_SEARCH_TYPE_IS) ? " selected" : "");

        bprintf(b, "<option value=\"begins\"%s>begins" CRLF,
                (ase->type ==
                 ABOOK_SEARCH_TYPE_BEGINS) ? " selected" : "");

        bprintf(b, "<option value=\"ends\"%s>ends" CRLF,
                (ase->type == ABOOK_SEARCH_TYPE_ENDS) ? " selected" : "");

        bprintf(b, "<option value=\"contains\"%s>contains" CRLF,
                ((ase->type == ABOOK_SEARCH_TYPE_NONE) ||
                 (ase->type ==
                  ABOOK_SEARCH_TYPE_CONTAINS)) ? " selected" : "");

        bprintf(b, "</select>" CRLF);
        bprintf(b, "</td><td>" CRLF);
        bprintf(b, "<input name=\"val%lu\" value=\"%s\" size=\"30\">" CRLF,
                i, ase->value);
        bprintf(b, "</td></tr>" CRLF);
    }
    bputs(b, "</table>" CRLF);

    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_search", "Search");
    html_form_table_submit(b, "sub_add_cond", "Add Condition");
    bputs(b, "</tr></table>" CRLF);

    html_form_end(b);
}

static void
generate_list(struct session *session, struct abook_search_list *asl)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    struct options *options = session->options;
    struct abook *abook = options->abook;
    struct abook_entry *abe;
    unsigned long count;

    abook_search_init(abook);

    count = 0;
    while (abook_search_find_next(abook, asl))
        count++;

    if (count == 0) {
        bputs(b, "No entries match those search criteria");
        return;
    }

    if (count != 1)
        bprintf(b, "%lu matches for those search criteria:", count);
    else
        bputs(b, "1 match for those search criteria:");

    bputs(b, "<table border=1><tr>" CRLF);
    bputs(b, "<th align=\"left\">Alias</th>" CRLF);
    bputs(b, "<th align=\"left\">Name</th>" CRLF);
    bputs(b, "<th align=\"left\">Comment</th>" CRLF);
    bputs(b, "<th align=\"left\">Address</th>" CRLF);
    bputs(b, "<th>&nbsp;</th>" CRLF);

    bprintf(b, "</tr>" CRLF);

    abook_search_init(abook);

    while ((abe = abook_search_find_next(abook, asl))) {
        bprintf(b, "<tr>" CRLF);
        html_session_bprintf(session, b, "<td><a href=\"", "\">",
                             ("abook_update?alias=%s&amp;name=%s"
                              "&amp;comment=%s&amp;email=%s"),
                             string_url_encode(pool, abe->alias),
                             string_url_encode(pool, abe->name),
                             string_url_encode(pool, abe->comment),
                             string_url_encode(pool, abe->email));
        html_quote_string(b, abe->alias);
        bputs(b, "</a></td>" CRLF);

        if (abe->name && abe->name[0])
            bprintf(b, "<td>%s</td>" CRLF, abe->name);
        else
            bputs(b, "<td>&nbsp;</td>" CRLF);

        if (abe->comment && abe->comment[0])
            bprintf(b, "<td>%s</td>" CRLF, abe->comment);
        else
            bputs(b, "<td>&nbsp;</td>" CRLF);

        if (abe->email && abe->email[0])
            bprintf(b, "<td>%s</td>" CRLF, abe->email);
        else
            bputs(b, "<td>&nbsp;</td>" CRLF);

        if (session->draft->have_draft) {
            /* Add links */
            bputs(b, "<td>");
            html_session_bprintf(session, b, "<a href=\"",
                                 "\">To</a>" CRLF,
                                 "add_address/abook_search/To/%s/%s",
                                 string_canon_encode(pool, abe->name),
                                 string_canon_encode(pool, abe->email));

            html_session_bprintf(session, b, "<a href=\"",
                                 "\">Cc</a>" CRLF,
                                 "add_address/abook_search/Cc/%s/%s",
                                 string_canon_encode(pool, abe->name),
                                 string_canon_encode(pool, abe->email));

            html_session_bprintf(session, b, "<a href=\"",
                                 "\">Bcc</a>" CRLF,
                                 "add_address/abook_search/Bcc/%s/%s",
                                 string_canon_encode(pool, abe->name),
                                 string_canon_encode(pool, abe->email));
            bputs(b, "</td>");
        } else
            html_session_bprintf(session, b, "<td><a href=\"",
                                 "\">Compose fresh message</a></td>" CRLF,
                                 "abook_compose/%s/%s",
                                 string_canon_encode(pool, abe->name),
                                 string_canon_encode(pool, abe->email));

        bputs(b, "</tr>" CRLF);
    }
    bputs(b, "</table>" CRLF);
}

void cmd_abook_search(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    unsigned long i;
    char *key, *op, *value, *s, *t;
    int field, type;
    struct abook_search_list *asl;

    if (request->method == POST) {
        request_decode_post(request);
        h = request->form;
    }

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "abook_list");
        return;
    }

    if ((s = assoc_lookup(h, "cond")) && !strcmp(s, "all"))
        asl = abook_search_list_create(T);
    else
        asl = abook_search_list_create(NIL);

    for (i = 0; 1; i++) {
        t = string_itoa(request->pool, i);

        s = pool_strcat(request->pool, "key", t);
        if ((key = assoc_lookup(h, s)) == NIL)
            break;

        s = pool_strcat(request->pool, "op", t);
        if ((op = assoc_lookup(h, s)) == NIL)
            break;

        s = pool_strcat(request->pool, "val", t);
        if ((value = assoc_lookup(h, s)) == NIL)
            break;

        field = abook_search_field_form_to_enum(key);
        type = abook_search_type_form_to_enum(op);

        if ((field == ABOOK_SEARCH_FIELD_NONE)
            || (type == ABOOK_SEARCH_TYPE_NONE))
            break;

        abook_search_list_add(asl, field, type, value);
    }

    /* Add new (empty) filter to end of list if empty or additional requested */
    if (!asl->first || assoc_lookup(h, "sub_add_cond"))
        abook_search_list_add(asl,
                              ABOOK_SEARCH_FIELD_NAME,
                              ABOOK_SEARCH_TYPE_CONTAINS, "");

    generate_hdr(session);

    generate_form(session, asl);
    if (request->method == POST)
        generate_list(session, asl);    /* Result of previous POST opertion */

    html_end(b);
    response_html(request, 200);        /* Success */

    abook_search_list_free(asl);
}
