/* $Cambridge: hermes/src/prayer/prayer/cmd_abook_update.c,v 1.3 2003/10/23 15:14:17 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session, struct assoc *h)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Addressbook Entry Update");
    html_banner_toolbar(session, b, "abook_update", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Addressbook Entry Update</h2>" CRLF);

    /* Form for addressbook entry */
    html_form_start(session, b, "POST", "abook_update");
    bprintf(b, "<table>" CRLF);

    /* Original Alias name */
    bprintf(b, "<tr>" CRLF);
    if ((assoc_lookup(h, "original")))
        html_form_table_hidden(b, "original", assoc_lookup(h, "original"));
    else
        html_form_table_hidden(b, "original", assoc_lookup(h, "alias"));
    bprintf(b, "</tr>" CRLF);

    /* Alias name */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Alias", "alias", assoc_lookup(h, "alias"),
                          20);
    bprintf(b, "</tr>" CRLF);

    /* Name */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Name", "name", assoc_lookup(h, "name"), 43);
    bprintf(b, "</tr>" CRLF);

    /* Comment */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Comment", "comment",
                          assoc_lookup(h, "comment"), 43);
    bprintf(b, "</tr>" CRLF);

    /* Email addresses */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Address(es)", "email",
                          assoc_lookup(h, "email"), 43);
    bprintf(b, "</tr>" CRLF);
    bprintf(b, "</table>" CRLF);

    /* Submit options for form */
    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_delete", "Delete entry");
    html_form_table_submit(b, "sub_update", "Add/Update entry");
    bputs(b, "</tr></table>" CRLF);


    html_form_end(b);
    html_end(b);

    response_html(request, 200);        /* Success */
}

void cmd_abook_update(struct session *session)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct assoc *h = NIL;
    char *s;

    if (request->method != POST) {
        if (request->get_suffix) {
            request_decode_get(request);
            h = request->form;
        }
        generate_form(session, h);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "abook_list");
        return;
    }

    if (assoc_lookup(h, "sub_delete") && (s = assoc_lookup(h, "alias"))) {
        abook_remove(options->abook, s);
        session->options->save = T;
        session_message(session, "Removed address book entry: \"%s\"", s);
    }

    if (assoc_lookup(h, "sub_update")) {
        char *original = assoc_lookup(h, "original");
        char *alias = assoc_lookup(h, "alias");
        char *name = assoc_lookup(h, "name");
        char *fcc = pool_strdup(request->pool, "");     /* Not yet */
        char *comment = assoc_lookup(h, "comment");
        char *email = assoc_lookup(h, "email");

        if (alias)
            alias = string_trim_whitespace(alias);
        else
            alias = "";

        if (!alias[0]) {
            session_message(session, "No alias provided");
            generate_form(session, h);
            return;
        }

        if (strchr(alias, ' ') || strchr(alias, '\t')) {
            session_message(session, "Alias cannot contain spaces");
            generate_form(session, h);
            return;
        }

        if (!(name && comment && email)) {
            session_message(session, "Missing form entries");
            generate_form(session, h);
            return;
        }

        name = string_trim_whitespace(name);
        fcc = string_trim_whitespace(fcc);
        comment = string_trim_whitespace(comment);
        email = string_trim_whitespace(email);

        if (original && (strcmp(original, alias) != 0) &&
            abook_lookup(options->abook, alias)) {
            session_message(session,
                            "An addressbook entry named \"%s\" already exists",
                            alias);
            generate_form(session, h);
            return;
        }

        if (email[0] == '\0') {
            session_message(session, "No email address supplied");
            generate_form(session, h);
            return;
        }

        if (!addr_check_valid(request->pool, email)) {
            session_message(session, "%s", ml_errmsg());
            generate_form(session, h);
            return;
        }

        /* Have to add unchecked entry to addressbook to spot all possible loops */
        abook_replace(options->abook, alias, name, fcc, comment, email);

        if (!abook_check_loop(request->pool, options->abook, alias, T)) {
            session_message(session,
                            "Addressbook loop involving alias \"%s\"",
                            assoc_lookup(h, "alias"));
            generate_form(session, h);
            return;
        }

        session_message(session, "Updated address book entry: \"%s\"",
                        alias);
        session->options->save = T;
    }

    session_redirect(session, request, "abook_list");
}
