/* $Cambridge: hermes/src/prayer/prayer/cmd_attachments.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct draft *draft = session->draft;
    struct buffer *b = request->write_buffer;
    unsigned long count;
    struct attlist *a;
    char *s;

    /* Title bar */
    html_start(session, b, "MIME Attachments");
    html_banner_toolbar(session, b, "attachments", 0);

    /* Body */
    if (draft->attlist) {
        bprintf(b, "<h1 align=\"center\">MIME Attachments</h1>" CRLF);
        bprintf(b, "<table>" CRLF);

        for (count = 1, a = draft->attlist; a; a = a->next, count++) {
            s = pool_strcat(request->pool, "detach/",
                            string_itoa(request->pool, count));
            bprintf(b,
                    ("<tr><td align=\"right\">%lu.</td><td>%s (%lu bytes) of "
                     "type %s</td>" CRLF), count, a->name, a->size,
                    a->type);
            html_textlink_session_table(session, b, s, "Detach");
            bprintf(b, "</tr>" CRLF);
        }
        bprintf(b, "</table>" CRLF);
        bprintf(b, "<hr>" CRLF);
    }

    bprintf(b, "<h3>Attach file:</h3>" CRLF "<br>" CRLF);

    html_session_bputs(session, b,
                       "<form method=\"POST\" action=\"",
                       "\" enctype=\"multipart/form-data\">" CRLF,
                       "attachments");
    bputs(b, "File <input align=\"middle\" type=\"file\" ");
    bputs(b, "name=\"file\" size=\"40\">" CRLF);
    bputs(b, "<input type=\"submit\" value=\"Attach File\">");
    bputs(b, "</form>" CRLF);

    html_form_start(session, b, "POST", "attachments/cancel");
    html_form_submit(b, "sub_cancel", "Back to Compose Screen");
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void process_form(struct session *session)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct draft *draft = session->draft;
    struct assoc *hdrs = assoc_create(request->pool, 16, T);
    char *start, *end;
    char *type, *disp, *encoding, *name, *s;
    unsigned long len;

    if (!request_decode_post_multipart(request, hdrs, &start, &end))
        return;

    if (end > start)
        len = (char *) end - (char *) start;
    else
        len = 0;

    if ((config->draft_att_single_max > 0) &&
        (len > config->draft_att_single_max)) {
        session_message(session,
                        "Attachment too large: %lu bytes exceeds limit %lu",
                        len, config->draft_att_single_max);
        session_log(session,
                    "Attachment too large: %lu bytes exceeds limit %lu",
                    len, config->draft_att_single_max);
        return;
    }

    if ((config->draft_att_total_max > 0) &&
        ((len + draft_att_size(draft)) > config->draft_att_total_max)) {
        session_message(session,
                        "Total size of attachments exceeds limit: %lu bytes",
                        config->draft_att_total_max);
        session_log(session,
                    "Total size of attachments exceeds limit: %lu bytes",
                    config->draft_att_total_max);
        return;
    }


    if (!(type = assoc_lookup(hdrs, "content-type")))
        type = "application/octet-stream";

    /* Opera attaches filename to Content-Type header e.g:
     *   Content-Type: application/octet-stream; name="<whatever>"
     */
    if ((s = strchr(type, ';')))
        *s = '\0';

    if (!(encoding = assoc_lookup(hdrs, "content-transfer-encoding")))
        encoding = "";

    name = "";

    if ((disp = assoc_lookup(hdrs, "content-disposition"))) {
        for (s = disp; *s; s++) {
            if (!strncasecmp(s, "filename=\"", strlen("filename=\""))) {
                name = s = s + strlen("filename=\"");
                while (*s && (*s != '"'))
                    s++;
                *s = '\0';
                name = string_trim_whitespace(name);
                break;
            } else if (!strncasecmp(s, "filename=", strlen("filename="))) {
                name = s + strlen("filename=");
                name = string_trim_whitespace(name);
                break;
            }
        }
    }

    /* Isolate last component of name */
    if ((s = strrchr(name, '/')))       /* Unix names */
        name = s + 1;
    else if ((s = strrchr(name, '\\'))) /* Windows names */
        name = s + 1;

    /* Mac Netscape sends names with URL encoding. Sigh! */
    if (name && name[0])
        name = string_url_decode(name);

    draft_add_attachment(draft, name, type, encoding, NIL, 0, start, len);

    session_message(session, "Added attachment \"%s\"", name);
    session_log(session,
                "[cmd_attachments] Added attachment \"%s\" (%lu bytes)",
                name, len);
}

void cmd_attachments(struct session *session)
{
    struct request *request = session->request;

    if ((request->argc >= 2) && (!strcmp(request->argv[1], "cancel"))) {
        session_redirect(session, request, "compose");
        return;
    }

    if (request->method == POST)
        process_form(session);

    generate_form(session);
}
