/* $Cambridge: hermes/src/prayer/prayer/cmd_block.c,v 1.3 2003/05/21 10:10:55 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    struct list_item *item;
    unsigned long count;

    /* Title bar */
    html_start(session, b, "Manual Address Blacklist");
    html_banner_toolbar(session, b, "block", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Manual Address Blacklist</h2>" CRLF);

    if (list_length(account->block_list) == 0) {
        bputs(b, "No block actions defined at the current time" CRLF);
    } else {
        bputs(b, "<ol>" CRLF);

        count = 0;
        for (item = account->block_list->head; item; item = item->next) {
            struct filter *filter = (struct filter *) item;

            if (filter->type != FILTER_BLOCK) {
                bputs(b, "<li>Unexpected filter type</li>" CRLF);
                count++;
                continue;
            }

            bputs(b, "<li><strong>BLOCK:</strong>" CRLF);
            bputs(b, "Blocking all messages from \"");
            html_quote_string(b, filter->local_part);
            bputs(b, " at ");
            html_quote_string(b, filter->domain);
            bputs(b, "\"." CRLF);
            html_session_bprintf(session, b, "<a href=\"",
                                 "\">Remove this block.</a>",
                                 "block/remove/%lu", count);
            bputs(b, "</li>" CRLF);
            count++;
        }
        bputs(b, "</ol>" CRLF);
    }

    /* Form body */
    html_form_start(session, b, "POST", "block");
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    bputs(b, "<td>Block:</td>" CRLF);
    html_form_table_field(b, NIL, "param", NIL, 40);
    html_form_table_submit(b, "sub_add", "Block Address");
    bputs(b, "</tr></table>" CRLF);

    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr></table>" CRLF);

    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void
generate_error(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    char *msg = account_fetch_message(account);
    struct buffer *b = request->write_buffer;

    /* Title Bar */
    html_start(session, b, "Automatic Mail Blocking");
    html_banner_toolbar(session, b, "block", 0);

    /* Title line */
    bputs(b,
          "<h2 align=\"center\">Automatic Mail Blocking</h2>"
          CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg)
        html_quote_string(b, msg);
    else
        bputs(b, "Unable to check mail processing status");
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "block");

    /* Cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel",
                           "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);
}

void
cmd_block(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    unsigned long offset;
    struct assoc *h = NIL;
    struct filter *filter;
    char *param;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool)) {
            generate_error(session);
            return;
        }

        if ((request->argc >= 3) && !strcmp(request->argv[1], "remove")) {
            offset = atoi(request->argv[2]);
            account_block_remove(session->account, offset);
            account_mail_update(session->account, request->pool);
            session_message(session, "Removed block number: %lu", offset + 1);
        }
        generate_form(session);
        return;
    }

    filter = filter_alloc();
    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "manage");
        return;
    }

    param = assoc_lookup(h, "param");

    if (param && param[0])
        param = string_trim_whitespace(param);

    if (!(param && param[0])) {
        session_message(session, "Missing or empty parameter field");
        generate_form(session);
        return;
    }

    /* Param is address or address wildcard */
    if (!filter_test_addr(filter, session, param)) {
        session_message(session, "Invalid filter pattern");
        generate_form(session);
        return;
    }

    /* Block is special: doesn't require target folder */
    filter_set_type(filter, FILTER_BLOCK);
    filter_set_addr(filter, session, param);
    account_block_add(session->account, filter);
    account_mail_update(session->account, request->pool);
    session_message(session, "Added BLOCK filter");
    generate_form(session);
}
