/* $Cambridge: hermes/src/prayer/prayer/cmd_compose.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void cmd_compose_generate_form(struct session *session)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct draft *draft = session->draft;
    struct prefs *prefs = session->options->prefs;
    struct buffer *b = request->write_buffer;
    char *atts = pool_printf(request->pool, "Attachments (%lu)",
                             draft_att_count(draft));

    /* Title Bar */
    html_start(session, b, "Draft message");
    html_form_start(session, b, "POST", "compose");
    html_banner_toolbar(session, b, "compose", HTML_BANNER_CALLBACK_ALL);

    /* Start of header section */
    bprintf(b, "<br>" CRLF);
    bprintf(b, "<table cellspacing=0 cellpadding=0>" CRLF);

    /* To header */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "To:", "hdr_To", draft->to, 70);
    html_form_table_submit(b, "sub_abook_To", "Expand");
    bprintf(b, "</tr>" CRLF);

    /* Cc header */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Cc:", "hdr_Cc", draft->cc, 70);
    html_form_table_submit(b, "sub_abook_Cc", "Expand");
    bprintf(b, "</tr>" CRLF);

    if (draft->rich_headers) {
        /* Bcc header */
        bprintf(b, "<tr>" CRLF);
        html_form_table_field(b, "Bcc:", "hdr_Bcc", draft->bcc, 70);
        html_form_table_submit(b, "sub_abook_Bcc", "Expand");
        bprintf(b, "</tr>" CRLF);

        /* Fcc header */
        bprintf(b, "<tr>" CRLF);
        html_form_table_field(b, "Fcc:", "hdr_Fcc", draft->fcc, 70);
        bprintf(b, "</tr>" CRLF);

        /* Reply-To header */
        bprintf(b, "<tr>" CRLF);
        html_form_table_field(b, "Reply-To:", "hdr_Reply_To",
                              draft->reply_to, 70);
        bprintf(b, "</tr>" CRLF);
    }

    /* Subject header */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Subject:", "hdr_Subject", draft->subject,
                          70);

    /* Rich header toggle */
    if (draft->rich_headers)
        html_form_table_submit(b, "sub_no_rich_headers",
                               "Hide Rich Headers");
    else
        html_form_table_submit(b, "sub_rich_headers", "Rich Headers");

    bprintf(b, "</tr>" CRLF);
    bprintf(b, "</table>" CRLF);

    /* End of header section */

    /* Start of submit options strip */
    bputs(b, "<br>" CRLF);
    bputs(b, "<table cellspacing=0 cellpadding=0>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_large", "Large");
    bputs(b, "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>");
    html_form_table_submit(b, "sub_clear_headers", "Clear Headers");
    html_form_table_submit(b, "sub_clear_body", "Clear Body");
    html_form_table_submit(b, "sub_undo", "Undo");
    bputs(b, "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
    html_form_table_submit(b, "sub_import", "Import");
    html_form_table_submit(b, "sub_attachments", atts);
    bputs(b, "</tr></table>" CRLF);

    /* The main text area */
    html_form_textarea(session, b, "body", draft->body,
                       prefs->small_cols, prefs->small_rows);

    /* Banner line containing submit options */
    bputs(b, "<table cellspacing=0 cellpadding=0><tr>" CRLF);

    if (config->ispell_path)
        html_form_table_submit(b, "sub_spell", "Check Spelling");

    if (prefs->line_wrap_advanced) {
        html_form_table_submit(b, "sub_line_wrap", "Line Wrap");
        html_form_table_checkbox(b, "Line Wrap on Send",
                                 "line_wrap", draft->line_wrap);
    }

    bputs(b, "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_postpone", "Postpone");
    html_form_table_submit(b, "sub_send", "Send");
    html_form_table_checkbox(b, "Save copy", "copy_outgoing",
                             draft->save_copy);

    bputs(b, "</tr></table>" CRLF);

    /* End of form */
    html_form_end(b);
    html_end(b);

    /* Send out HTML */
    response_html(request, 200);
}

/* ====================================================================== */

static void cmd_compose_generate_large_form(struct session *session)
{
    struct request *request = session->request;
    struct prefs *prefs = session->options->prefs;
    struct draft *draft = session->draft;
    struct buffer *b = request->write_buffer;

    html_start(session, b, "Draft message");

    /* Banner line containing submit options */
    html_form_start(session, b, "POST", "compose");

    html_session_message(session, b);
    bputs(b, "<table cellspacing=0 cellpadding=0>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_small", "Small");
    bputs(b, "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>");
    html_form_table_submit(b, "sub_help", "Help");
    html_form_table_submit(b, "sub_logout", "Logout");
    bputs(b, "</tr></table>" CRLF);

    /* End of submit options line */
    html_form_textarea(session, b, "body", draft->body,
                       prefs->large_cols, prefs->large_rows);

    /* More submit options */
    bprintf(b, "<table cellspacing=0 cellpadding=0>" CRLF);
    bprintf(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_spell", "Check Spelling");

    if (prefs->line_wrap_advanced) {
        html_form_table_submit(b, "sub_line_wrap", "Line Wrap");
        html_form_table_checkbox(b, "Line Wrap on Send",
                                 "line_wrap", draft->line_wrap);
    }

    bputs(b, "<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>");
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_postpone", "Postpone");
    html_form_table_submit(b, "sub_send", "Send");
    html_form_table_checkbox(b, "Save copy", "copy_outgoing",
                             draft->save_copy);
    bputs(b, "</tr></table>" CRLF);
    /* End of form */
    html_form_end(b);
    html_end(b);

    /* Send out HTML */
    response_html(request, 200);
}

/* ====================================================================== */

static BOOL cmd_compose_generate_postponed_list(struct session *session)
{
    struct config *config = session->config;
    struct prefs *prefs = session->options->prefs;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    MAILSTREAM *stream = session->draft_stream;
    unsigned long msgno;

    /* Title bar */
    html_start(session, b, "Select Postponed message");
    html_banner_toolbar(session, b, "compose/postponed", 0);

    /* Title line */
    bputs(b, "<h1 align=\"center\">List of Postponed messages</h1>");

    bputs(b, "<table border=1>" CRLF);

    bputs(b, "<tr>");
    bputs(b, "<td>Msgno</td>");
    bputs(b, "<td>Date</td>");
    bputs(b, "<td>First To/Cc/Bcc</td>");
    bputs(b, "<td align=\"right\">Size</td>");
    bputs(b, "<td nowrap>Subject</td>");
    bputs(b, "</tr>" CRLF);

    for (msgno = 1; msgno <= stream->nmsgs; msgno++) {
        MESSAGECACHE *elt;
        ENVELOPE *env;
        ADDRESS *addr;
        char *string;

        if (!((elt = ml_elt(session, stream, msgno)) &&
              (env = ml_fetch_structure(session, stream, msgno, NIL, 0))))
            return (NIL);

        /* Msg No */
        bprintf(b, "<tr><td>%lu.</td>" CRLF, msgno);

        /* Message Date */
        if (prefs->use_full_date)
            bprintf(b, "<td nowrap>%s</td>" CRLF,
                    string_date_to_string_long(elt->month, elt->day,
                                               elt->year + BASEYEAR));
        else
            bprintf(b, "<td nowrap>%s</td>" CRLF,
                    string_date_to_string_short(elt->month, elt->day));

        /* To/Cc/Bcc address (use first non-empty address) */
        if (!((addr = env->to) && addr->mailbox && addr->host))
            if (!((addr = env->cc) && addr->mailbox && addr->host))
                addr = env->bcc;

        if (addr && addr->personal) {
            string = addr->personal;
        } else if (addr && addr->mailbox && addr->host)
            string =
                pool_printf(request->pool, "%s@%s", addr->mailbox,
                            addr->host);
        else
            string = "(unknown)";

        string = (char *)
            rfc1522_decode(pool_alloc(request->pool, strlen(string)),
                           strlen(string), string, NIL);

        string =
            string_prune(request->pool, string, config->list_addr_maxlen);

        bputs(b, "<td>" CRLF);
        html_quote_string(b, string);
        bputs(b, "</td>" CRLF);

        /* Message size */
        if (elt->rfc822_size > 1024)
            bprintf(b, "  <td align=\"right\">%luK</td>" CRLF,
                    elt->rfc822_size / 1024);
        else
            bprintf(b, "  <td align=\"right\">&lt;1K</td>" CRLF);

        /* Message subject */
        html_session_bprintf(session, b, "<td nowrap><a href=\"", "\">",
                             "compose?postponed=%lu", msgno);

        if (env->subject && env->subject[0]) {
            string = env->subject;
            string = (char *)
                rfc1522_decode(pool_alloc(request->pool, strlen(string)),
                               strlen(string), string, NIL);

            string = string_prune(request->pool, string,
                                  config->list_subject_maxlen);
        } else
            string = "(No subject)";


        html_quote_string(b, string);
        bprintf(b, "</a></td></tr>" CRLF);
    }

    bputs(b, "</table>" CRLF);
    bputs(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "compose");
    html_form_submit(b, "postponed_cancel", "Cancel");
    html_form_submit(b, "postponed_fresh", "Compose a fresh message");
    html_form_end(b);
    html_end(b);

    /* Send out the response */
    response_html(request, 200);

    return (T);
}

/* ====================================================================== */

static void cmd_compose_generate_role_list(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct options *options = session->options;
    struct list_item *li;

    /* Title bar */
    html_start(session, b, "Select Role");
    html_banner_toolbar(session, b, "compose/role_list", 0);

    /* Title line */
    bputs(b, "<h1 align=\"center\">Roles</h1>");

    bputs(b, "Roles:" CRLF "<ol>" CRLF);

    html_session_bputs(session, b, "  <li><a href=\"",
                       "\">Default</a>" CRLF, "compose1?role=default");

    for (li = options->role_list->head; li; li = li->next) {
        struct role *role = (struct role *) li;

        html_session_bprintf(session, b,
                             "  <li><a href=\"", "\">", "compose1?role=%s",
                             string_url_encode(request->pool, role->name));
        html_quote_string(b, role->name);
        bputs(b, "</a>" CRLF);
    }

    bputs(b, "</ol>" CRLF);

    html_form_start(session, b, "POST", "compose/cancel");
    html_form_submit(b, "sub_cancel", "Cancel");
    html_form_end(b);

    html_end(b);
    /* Send out the response */
    response_html(request, 200);
}

/* ====================================================================== */

/* Check draft message
 * Returns: T   if draft exists or created by end of function
 *          NIL if redirect or page issued in order to generate draft 
 */

static BOOL cmd_compose_check_draft(struct session *session)
{
    struct request *request = session->request;
    struct draft *draft = session->draft;
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct assoc *h = NIL;
    char *s;
    unsigned long value;

    if (draft->have_draft)
        return (T);

    if (request->method == GET) {
        request_decode_get(request);
        h = request->form;

        if (assoc_lookup(h, "postponed_cancel")) {
            session_redirect(session, request,
                             session->compose_parent_cmd);
            return (NIL);
        }

        if (assoc_lookup(h, "postponed_list")) {
            struct pool *pool = request->pool;
            char *name;

            name = string_url_encode(pool, session->draft_foldername);
            session_redirect(session, request,
                             pool_strcat(pool, "change/", name));
            return (NIL);
        }
    }

    if (!h && session->have_drafts && draft_check(session)) {
        if (!cmd_compose_generate_postponed_list(session))
            session_redirect(session, request, "restart");
        return (NIL);
    }

    if (h && !assoc_lookup(h, "postponed_fresh") &&
        (s = assoc_lookup(h, "postponed")) && ((value = atoi(s)) > 0)) {

        /* Conceivable that session has gone to sleep on postponed list page */
        if (!draft_check(session)) {
            session_redirect(session, request, "restart");
            return (NIL);
        }

        draft_role_set(draft, NIL);     /* Set up default role */
        draft_init(draft, prefs->use_sent_mail);

        /* Reload draft message */
        if (!draft_restore_postponed(draft, request->pool,
                                     session->draft_stream, value)) {
            session_redirect(session, request, "restart");
            return (NIL);
        }

        /* Remove from draft folder */
        if (!draft_delete(session, value)) {
            session_redirect(session, request, "restart");
            return (NIL);
        }

        if (draft_att_count(draft) > 1)
            session_message(session,
                            "Restored postponed draft, including %lu attachments",
                            draft_att_count(draft));
        else if (draft_att_count(draft) == 1)
            session_message(session,
                            "Restored postponed draft, including 1 attachment");
        else
            session_message(session,
                            "Restored postponed draft, no attachments");
    } else if (list_length(options->role_list) > 0L) {
        /* Fresh new message and need to select role */
        session_message(session,
                        "Composing fresh message: please select a role");
        cmd_compose_generate_role_list(session);
        return (NIL);
    } else {
        /* Set up default role */
        draft_role_set(draft, NIL);
        draft_init(draft, prefs->use_sent_mail);
        session_message(session, "Composing fresh message");
    }
    session->compose_large = NIL;
    draft_init_rich_headers(draft);
    return (T);
}

/* ====================================================================== */

void cmd_compose(struct session *session)
{
    struct request *request = session->request;
    struct draft *draft = session->draft;
    struct draft *draft0 = session->draft0;
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct assoc *h = NIL;
    char *hdr;

    if ((request->argc >= 2) && (!strcmp(request->argv[1], "cancel"))) {
        session_log(session, "[cmd_compose] Cancel");
        session->compose_large = NIL;
        session->abook_parent_cmd = "list";
        /* Kill the draft */
        draft_clear_atts(draft);
        draft_free(draft);
        draft_free(draft0);
        session_redirect(session, request, session->compose_parent_cmd);
        return;
    }

    if (!cmd_compose_check_draft(session))
        return;

    /* If we get this far then draft exists with correct role */

    if (request->method != POST) {
        if (session->compose_large)
            cmd_compose_generate_large_form(session);
        else
            cmd_compose_generate_form(session);
        return;
    }

    request_decode_post(request);
    h = request->form;

    /* Undo is special: rewind to previous version */
    if (assoc_lookup(h, "sub_undo")) {
        struct draft *tmp;

        if (!draft0->have_draft)
            draft_init(draft0, prefs->use_sent_mail);

        /* Update existing draft so that undo can be toggled */
        if (session->compose_large)
            draft_update_body(draft, h);
        else
            draft_update(draft, h);

        /* Then switch draft and draft0 */
        tmp = session->draft;
        session->draft = session->draft0;
        session->draft0 = tmp;
        cmd_compose_generate_form(session);
        return;
    }

    /* Create backup copy of existing draft */
    draft_free(draft0);
    draft_copy(draft0, draft);

    /* Address book lookups on To:/Cc:/Bcc: */
    /* Need to do this before draft structure is updated */
    if ((assoc_lookup(h, "sub_abook_To"))
        && (hdr = assoc_lookup(h, "hdr_To"))) {
        hdr = string_trim_whitespace(hdr);
        if (hdr[0]) {
            if ((hdr =
                 abook_substitute(session, request->pool, options->abook,
                                  hdr)))
                assoc_update(h, "hdr_To", hdr, T);
            session_message(session, "Header To Lookup Complete");
            session_log(session,
                        "[cmd_compose] Header To Lookup Complete");
        } else {
            session_message(session, "Header To Empty");
            session_log(session, "[cmd_compose] Header To Empty");
        }
    }

    if ((assoc_lookup(h, "sub_abook_Cc"))
        && (hdr = assoc_lookup(h, "hdr_Cc"))) {
        hdr = string_trim_whitespace(hdr);
        if (hdr[0]) {
            if ((hdr =
                 abook_substitute(session, request->pool, options->abook,
                                  hdr)))
                assoc_update(h, "hdr_Cc", hdr, T);
            session_message(session, "Header Cc Lookup Complete");
            session_log(session,
                        "[cmd_compose] Header Cc Lookup Complete");
        } else {
            session_message(session, "Header Cc Empty");
            session_log(session, "[cmd_compose] Header Cc Empty");
        }
    }

    if ((assoc_lookup(h, "sub_abook_Bcc"))
        && (hdr = assoc_lookup(h, "hdr_Bcc"))) {
        hdr = string_trim_whitespace(hdr);
        if (hdr[0]) {
            if ((hdr =
                 abook_substitute(session, request->pool, options->abook,
                                  hdr)))
                assoc_update(h, "hdr_Bcc", hdr, T);
            session_message(session, "Header Bcc Lookup Complete");
            session_log(session,
                        "[cmd_compose] Header Bcc Lookup Complete");
        } else {
            session_message(session, "Header Bcc Empty");
            session_log(session, "[cmd_compose] Header Bcc Empty");
        }
    }

    /* Create new draft from updated assoc table (i.e: contents of form) */
    if (session->compose_large)
        draft_update_body(draft, h);
    else
        draft_update(draft, h);

    /* Process buttons which move us to other screens */
    /* Three most common operations first */

    if (assoc_lookup(h, "sub_cancel")) {
        session_log(session, "[cmd_compose] Cancel");
        session->compose_large = NIL;
        /* Kill the draft */
        draft_clear_atts(draft);
        draft_free(draft);
        draft_free(draft0);
        session_message(session, "Draft message cancelled");
        session_redirect(session, request, session->compose_parent_cmd);
        return;
    }

    if (assoc_lookup(h, "sub_postpone")) {
        session_log(session, "[cmd_compose] Postpone");
        /* Save, then kill the draft */
        if (draft_write(draft)) {
            session->abook_parent_cmd = "list";
            draft_clear_atts(draft);
            draft_free(draft);
            draft_free(draft0);
            session_message(session, "Draft message postponed");
            session_redirect(session, request,
                             session->compose_parent_cmd);
            return;
        } else {
            session_message(session, "Failed to write postponed message");
            session_log(session,
                        "Failed to write postponed message: %s",
                        ml_errmsg());
        }
    }

    if (assoc_lookup(h, "sub_send")) {
        session_log(session, "[cmd_compose] Sending message");
        session->compose_large = NIL;
        session_redirect(session, request, "send");
        return;
    }

    /* Toolbar actions moving us to other screens */
    if (assoc_lookup(h, "sub_list")) {
        session_redirect(session, request, "list");
        return;
    }

    if (assoc_lookup(h, "sub_display")) {
        session_redirect(session, request, "display");
        return;
    }

    if (assoc_lookup(h, "sub_abook_list")) {
        session->abook_parent_cmd = "compose";
        session_redirect(session, request, "abook_list");
        return;
    }

    if (assoc_lookup(h, "sub_folders")) {
        session_redirect(session, request, "folders");
        return;
    }

    if (assoc_lookup(h, "sub_manage")) {
        session_redirect(session, request, "manage");
        return;
    }

    if (assoc_lookup(h, "sub_folder_dialogue")) {
        char *name = assoc_lookup(request->form, "folder");

        if (name && name[0]) {
            session_redirect(session, request,
                             pool_strcat(request->pool, "change/", name));
            return;
        } else
            session_message(session, "Invalid mail folder");
    }

    if (assoc_lookup(h, "sub_help")) {
        session_log(session, "[cmd_compose] Help");
        if (session->compose_large)
            session_redirect(session, request, "help/compose/large");
        else
            session_redirect(session, request, "help/compose/normal");
        return;
    }

    if (assoc_lookup(h, "sub_logout")) {
        session_log(session, "[cmd_compose] Logout");
        session_redirect(session, request, "logout/compose");
        return;
    }

    /* Subsiduary screens */
    if (assoc_lookup(h, "sub_import")) {
        session_redirect(session, request, "include");
        return;
    }

    if (assoc_lookup(h, "sub_attachments")) {
        session_redirect(session, request, "attachments");
        return;
    }

    if (assoc_lookup(h, "sub_spell")) {
        session_log(session, "[cmd_compose] Spell Check");
        session_redirect(session, request, "spell");
        return;
    }

    /* Subsiduary actions which leave us on compose screen */

    /* Enable/Disable rich headers */
    if (assoc_lookup(h, "sub_rich_headers")) {
        session_log(session, "[cmd_compose] Rich headers enabled");
        draft_rich_headers(draft, T);
    }

    if (assoc_lookup(h, "sub_no_rich_headers")) {
        session_log(session, "[cmd_compose] Rich headers disabled");
        draft_rich_headers(draft, NIL);
    }

    if (assoc_lookup(h, "sub_large")) {
        session_log(session, "[cmd_compose] Large compose window");
        session->compose_large = T;
    }

    if (assoc_lookup(h, "sub_small")) {
        session_log(session, "[cmd_compose] Small compose window");
        session->compose_large = NIL;
    }

    if (assoc_lookup(h, "sub_clear_headers")) {
        session_log(session, "[cmd_compose] Clear headers");
        draft_clear_hdrs(draft);
    }

    if (assoc_lookup(h, "sub_clear_body")) {
        session_log(session, "[cmd_compose] Clear body");
        draft_clear_body(draft);
    }

    if (assoc_lookup(h, "sub_line_wrap")) {
        session_log(session, "[cmd_compose] Line wrap body");
        draft_line_wrap_body(draft);
    }

    if (session->compose_large)
        cmd_compose_generate_large_form(session);
    else
        cmd_compose_generate_form(session);
}
