/* $Cambridge: hermes/src/prayer/prayer/cmd_copy.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_inbox_line(struct session *session,
                    struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"10%\">" CRLF);
    html_session_bputs(session, b, "<a href=\"", "\">", "copy_msg/INBOX");
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td width=\"40%\">");
    html_session_bputs(session, b, "<a href=\"", "\">INBOX</a>" CRLF,
                       "copy_msg/INBOX");
    bputs(b, "</td></tr>" CRLF);
}

static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(session->pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    if (session->cwd && session->cwd[0]) {
        bputs(b, "<td>" CRLF);
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "copy?cwd=%s", string_canon_path_encode(pool,
                                                                     "",
                                                                     parent));

        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a></td></tr>" CRLF);
    } else {
        bputs(b, "<td>");
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</td></tr>" CRLF);
    }
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_session_bprintf(session, b,
                         "<a href=\"", "\">" CRLF, "copy?cwd=%s", name);
    html_icon(session, b, "right.gif", "[directory]");
    bprintf(b, "</a></td>" CRLF);
    html_session_bprintf(session, b,
                         "<td><a href=\"", "\">" CRLF, "copy?cwd=%s",
                         name);
    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);

    bputs(b, "</tr>" CRLF);
}

static void
generate_favourite_folder_line(struct session *session,
                               struct pool *pool,
                               struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, "", folder);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_session_bprintf(session, b,
                         "<a href=\"", "\">" CRLF, "copy_msg/%s", name);

    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    html_session_bprintf(session, b,
                         "<td><a href=\"", "\">" CRLF, "copy_msg/%s",
                         name);
    html_quote_string(b, folder);
    bprintf(b, "</a></td>" CRLF);

    bputs(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, session->cwd, folder);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_session_bprintf(session, b,
                         "<a href=\"", "\">" CRLF, "copy_msg/%s", name);

    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    html_session_bprintf(session, b,
                         "<td><a href=\"", "\">" CRLF, "copy_msg/%s",
                         name);
    html_quote_string(b, folder);
    bprintf(b, "</a></td>" CRLF);

    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */


static void
html_folderlist(struct session *session,
                struct pool *pool,
                struct buffer *b, struct dirlist *dl, unsigned long count)
{
    struct config_theme *theme = session->theme_main;
    struct favourite_list *fl = session->options->favourite_list;
    struct list_item *li;
    BOOL shaded = NIL;

    /* Title Bar */
    if (count != 1) {
        html_start(session, b,
                   pool_printf(pool, "Copy %lu Messages", count));
        html_banner_toolbar(session, b, "copy", 0);

        /* Title line */
        bprintf(b,
                "<h2 align=\"center\">Copy %lu Messages to folder...</h2>"
                CRLF, count);
    } else {
        html_start(session, b, "Copy message");
        html_banner_toolbar(session, b, "copy", 0);

        /* Title line */
        bprintf(b,
                "<h2 align=\"center\">Copy message to folder...</h2>"
                CRLF);
    }


    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "copy");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>");
    bputs(b, "<table><tr><td>" CRLF);
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    bputs(b, "</td></tr></table>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, session->copy_parent_cmd,
                                "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/copy", "Refresh");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">");

    /* Inbox item */
    if (session->cwd && session->cwd[0]) {
        generate_inbox_line(session, pool, b, shaded);
        shaded = (shaded) ? NIL : T;
    }

    if (list_length(fl->list) > 0L) {
        /* Replace td with something which stops silly spacing */
        bputs(b, "<tr><td>Favourite mail folders:</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        for (li = fl->list->head; li; li = li->next) {
            generate_favourite_folder_line(session, pool, b, li->name,
                                           shaded);
            shaded = (shaded) ? NIL : T;
        }

        bputs(b, "<tr><td>Normal folder listing:</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);
    }

    shaded = NIL;

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "create/copy");
    bputs(b, "<table width=\"100%\">");

    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>" CRLF);
    bputs(b, "<table><tr><td>" CRLF);
    bprintf(b, "Create" CRLF);
    bprintf(b, "<select name=\"type\" size=1>" CRLF);
    bprintf(b, "<option value=\"mailbox\" selected>mailbox</option>" CRLF);
    bprintf(b, "<option value=\"directory\">directory</option>" CRLF);
    bprintf(b, "</select>" CRLF);
    bprintf(b, "with name" CRLF);
    bprintf(b, "<input name=\"name\">" CRLF);
    bprintf(b,
            "<input type=\"submit\" name=\"create\" value=\"Create\">"
            CRLF);
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, session->copy_parent_cmd,
                                "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/copy", "Refresh");
    bputs(b, "</tr></table>" CRLF);
    bputs(b, "</td></tr></table>" CRLF);
    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

void cmd_copy(struct session *session)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    MAILSTREAM *stream = session->stream;
    struct assoc *h = NIL;
    unsigned long msgno, msgno_uid;
    unsigned long count = 1;
    char *s;

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != NIL))
            string_strdup(&session->dir_filter, s);
    }

    if ((request->argc > 1) && !strcmp(request->argv[1], "aggregate")) {
        if (prefs->use_mark_persist)
            count = msgmap_marked_count(session->zm);
        else
            count = msgmap_tmp_marked_count(session->zm);

        if (count > 1)
            session_message(session, "Copying %lu marked messages", count);
        else if (count == 1)
            session_message(session, "Copying single marked message");
        else {
            session_message(session, "No marked messages to copy");
            session_redirect(session, request, "list");
            return;
        }

        session->aggregate = T;
    } else if (request->argc > 2) {
        msgno = atoi(request->argv[1]);
        msgno_uid = atoi(request->argv[2]);

        if (!(msgno = stream_check_uid(session, stream, msgno, msgno_uid))) {
            session_redirect(session, request, "restart");
            return;
        }

        session->current = msgno;
        session->aggregate = NIL;
        session_message(session, "Copying message %lu", msgno);
    } else if (session->aggregate)
        count = msgmap_marked_count(session->zm);
    else
        count = 1;

    html_folderlist(session, pool, b,
                    dircache_fetch(session, session->cwd), count);

    response_html(request, 200);
}
