/* $Cambridge: hermes/src/prayer/prayer/cmd_dictionary.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session, struct buffer *b)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct dictionary *dictionary = options->dictionary;
    struct list_item *li;
    unsigned long i, len, max, perline, count;

    /* Title Bar */
    html_start(session, b, "Personal Dictionary Maintenance");
    html_banner_toolbar(session, b, "dictionary", 0);

    /* Title Line */
    bprintf(b,
            "<h2 align=\"center\">Personal Dictionary Maintenance</h2>"
            CRLF);

    html_form_start(session, b, "POST", "dictionary");

    /* Find maximum length of list item */
    count = 0;
    max = 1;
    for (li = dictionary->list->head; li; li = li->next) {
        if ((len = strlen(li->name)) > max)
            max = len;
        count++;
    }

    if ((max > 0) && (max < 38))
        perline = 80 / (max + 5);
    else
        perline = 1;

    if (count > 0) {
        bputs(b,
              "Personal Dictionary (click links to remove entries):" CRLF);
        bputs(b, "<table width=\"100%\"><tr>" CRLF);

        for (i = 0, li = dictionary->list->head; li; li = li->next, i++) {
            html_session_bprintf(session, b, "<td><a href=\"", "\">",
                                 "dictionary?remove=%s",
                                 string_url_encode(request->pool,
                                                   li->name));
            html_quote_string(b, li->name);
            bputs(b, "</a></td>" CRLF);

            /* Start a new line? */
            if ((i % perline) == (perline - 1))
                bputs(b, "</tr><tr>" CRLF);
        }
        bputs(b, "</tr></table>" CRLF);
    } else
        bputs(b, "Personal Dictionary (currently empty)" CRLF);


    bputs(b, "<p>Add or remove specific words from the list:</p>" CRLF);
    bputs(b, "<table><tr>");
    html_form_table_field(b, NIL, "add", "", 32);
    html_form_table_submit(b, "sub_add", "Add");
    bputs(b, "</tr><tr>" CRLF);
    html_form_table_field(b, NIL, "remove", "", 32);
    html_form_table_submit(b, "sub_remove", "Remove");
    bputs(b, "</tr><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr></table>");

    html_form_end(b);
    html_end(b);
}

void cmd_dictionary(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    struct options *options = session->options;
    struct dictionary *dictionary = options->dictionary;
    char *s, *line, *word;

    if (request->method != POST) {
        request_decode_get(request);
        h = request->form;
        if ((s = assoc_lookup(h, "remove")) && s[0]) {
            dictionary_remove(dictionary, s);
            options->save = T;
        }

        generate_form(session, b);
        response_html(request, 200);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "manage");
        return;
    }

    /* Add or remove words from dictionary */

    if (assoc_lookup(h, "sub_add")) {
        line = assoc_lookup(h, "add");
        while ((word = string_get_token(&line))) {
            if (word[0]) {
                dictionary_add(dictionary, word);
                options->save = T;
            }
        }
    } else if (assoc_lookup(h, "sub_remove")) {
        line = assoc_lookup(h, "remove");
        while ((word = string_get_token(&line))) {
            if (word[0]) {
                dictionary_remove(dictionary, word);
                options->save = T;
            }
        }
    }

    generate_form(session, b);
    response_html(request, 200);
}
