/* $Cambridge: hermes/src/prayer/prayer/cmd_display.c,v 1.5 2004/10/07 15:12:54 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void buffer_puts_mime_quote_html(struct buffer *b, char *s)
{
    unsigned long len = strlen(s) + 20;
    char *buffer = pool_alloc(b->pool, len);    /* Decoded form smaller */
    char *d;

    d = (char *) rfc1522_decode((unsigned char *) buffer, len, s, NIL);

    html_quote_string(b, d);
}

/* ====================================================================== */

static BOOL simple_string_compare(char *s1, char *s2)
{
    if ((s1 == NIL) && (s2 == NIL))
        return T;

    if ((s1 == NIL) || (s2 == NIL))
        return NIL;

    return (!strcmp(s1, s2));
}

/* Compare two ADDRESS structures, see if identical
 *   Returns: T   => definitely identical, can use shortcuts.
 *            NIL => Not clear whether identical. Don't try to use shortcuts.
 */

static BOOL simple_address_compare(ADDRESS * addr1, ADDRESS * addr2)
{
    /* Get rid of two difficult cases quickly */

    if ((addr1 == NIL) || (addr2 == NIL))
        return NIL;

    if ((addr1->next != NIL) || (addr2->next != NIL))
        return NIL;

    if (simple_string_compare(addr1->personal, addr2->personal) &&
        simple_string_compare(addr1->adl, addr2->adl) &&
        simple_string_compare(addr1->mailbox, addr2->mailbox) &&
        simple_string_compare(addr1->host, addr2->host) &&
        simple_string_compare(addr1->error, addr2->error))
        return T;

    return NIL;
}

/* ====================================================================== */

static void
cmd_display_write_addr(struct session *session, struct buffer *b,
                       ADDRESS * addr)
{
    struct pool *pool = b->pool;
    struct abook *abook = session->options->abook;
    ADDRESS *a;
    char *email, *alias;
    struct abook_entry *abe;

    for (a = addr; a; a = a->next) {
        if (a->mailbox && a->host) {
            email = pool_strcat3(pool, a->mailbox, "@", a->host);
            abe = abook_find_email(abook, email);
            alias = (abe && abe->alias) ? abe->alias : "";

            if (a->personal && a->personal[0]) {
                unsigned long len = strlen(a->personal) + 20;
                char *buffer = pool_alloc(pool, len);   /* Decoded form smaller */
                char *d = (char *) rfc1522_decode((unsigned char *) buffer,
                                                  len, a->personal, NIL);

                html_session_bprintf(session, b, "<a href=\"", "\">",
                                     "abook_take?alias=%s&amp;name=%s&amp;email=%s",
                                     string_url_encode(pool, alias),
                                     string_url_encode(pool, d),
                                     string_url_encode(pool, email));
                buffer_puts_mime_quote_html(b, a->personal);
                bputs(b, " &lt;");
                html_quote_string(b, a->mailbox);
                bputs(b, "@");
                html_quote_string(b, a->host);
                bputs(b, "&gt;");
                bputs(b, "</a>" CRLF);
            } else {
                html_session_bprintf(session, b, "<a href=\"", "\">",
                                     "abook_take?alias=%s&amp;email=%s",
                                     string_url_encode(pool, alias),
                                     email);
                html_quote_string(b, a->mailbox);
                bputs(b, "@");
                html_quote_string(b, a->host);
                bputs(b, "</a>" CRLF);
            }
        } else {
            /* Degenerate case: original behaviour. Is this correct? */
            buffer_puts_mime_quote_html(b, addr_text(b->pool, a));
        }

        if (a->next)
            bputs(b, ", ");
    }
}

/* ====================================================================== */

static BOOL
cmd_display_toolbar(struct session *session, MAILSTREAM * stream,
                    struct buffer *b, unsigned long msgno)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct config_theme *theme = session->theme_main;
    struct msgmap *zm = session->zm;
    MESSAGECACHE *elt;
    unsigned long zm_offset;
    BOOL use_icons = session->user_agent->use_icons;

    zm_offset = msgmap_find(zm, msgno);

    if (!(elt = ml_elt(session, stream, msgno)))
        return (NIL);

    /* Table line */
    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\"><table><tr>" CRLF);

    /* Previous and next message buttons */
    if (zm_offset > 1)
        html_button_msg_table(session, b, "display",
                              msgmap_value(zm, zm_offset - 1),
                              "left.gif", "Previous");
    else
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");

    if (!use_icons)
        bputs(b, "<td>|</td>");
    if (zm_offset < msgmap_size(zm))

        html_button_msg_table(session, b, "display",
                              msgmap_value(zm, zm_offset + 1),
                              "right.gif", "Next");
    else
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");

    /* Save message */
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_msg_table(session, b, "copy", msgno, "Copy");

    /* (Un)delete message */
    bputs(b, "<td>|</td>" CRLF);
    if (elt->deleted)
        html_textlink_msg_table(session, b, "disp_undelete", msgno,
                                "Undelete");
    else
        html_textlink_msg_table(session, b, "disp_delete", msgno,
                                "Delete");

    if (prefs->use_mark_persist) {
        /* (Un)mark message */
        bputs(b, "<td>|</td>" CRLF);
        if (msgmap_has_mark(zm, msgno))
            html_textlink_msg_table(session, b, "disp_unmark", msgno,
                                    "Unmark");
        else
            html_textlink_msg_table(session, b, "disp_mark", msgno,
                                    "Mark");
    }

    /* Reply to message */
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_msg_table(session, b, "reply", msgno, "Reply");
    /* Forward message */
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_msg_table(session, b, "forward", msgno, "Forward");

    if (session->stream == session->draft_stream) {
        /* Resume message */
        bputs(b, "<td>|</td>" CRLF);
        html_textlink_msg_table(session, b, "resume", msgno, "Resume");
    }

    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"center\">");
    bprintf(b, "<strong>Message: %lu out of %lu</strong>", msgno,
            zm->nmsgs);
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"34%\"align=\"right\"><table><tr>" CRLF);
    /* Show full headers */
    if (session->full_hdrs)
        html_textlink_msg_table(session, b, "hdrs", msgno, "Hide Hdrs");
    else
        html_textlink_msg_table(session, b, "hdrs", msgno, "Show Hdrs");
    bputs(b, "<td>|</td>" CRLF);
    bputs(b, "<td>");
    bprintf(b,
            "<a href=\"%s/NOSEQ/download/%lu/%lu\">Download Message</a>",
            session->url_prefix_bsession, msgno, ml_uid(session, stream,
                                                        msgno));
    bputs(b, "</td>");
    bputs(b, "</tr></table>" CRLF);
    bputs(b, "</td></tr></table>" CRLF);
    return (T);
}

/* ====================================================================== */

static BOOL
cmd_display_hdr(struct session *session,
                struct buffer *b, unsigned long msgno)
{
    struct config_theme *theme = session->theme_main;
    MAILSTREAM *stream = session->stream;
    MESSAGECACHE *elt;
    ENVELOPE *env;

    if (!((elt = ml_elt(session, stream, msgno)) &&
          (env = ml_fetch_structure(session, stream, msgno, NIL, 0))))
        return (NIL);

    if (session->full_hdrs) {
        char *hdr;
        unsigned long len;
        bprintf(b, "<pre>" CRLF);
        if (!
            (hdr =
             ml_fetch_header(session, stream, msgno, NIL, NIL, &len, 0)))
            return (NIL);
        html_quote_linewrap(b, hdr, 80, T);
        bprintf(b, "</pre>" CRLF);
    } else {
        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<table width=\"100%%\" bgcolor=\"%s\">" CRLF,
                    theme->bgcolor_banner);
        else
            bputs(b, "<table width=\"100%\">" CRLF);

        bputs(b, "<tr><td><table>" CRLF);

        if (env->reply_to
            && !simple_address_compare(env->reply_to, env->from)) {
            bputs(b, "<tr><td><strong>Reply-To:</strong></td>");
            bputs(b, "<td align=left>");
            buffer_puts_mime_quote_html(b,
                                        addr_text(b->pool, env->reply_to));
            bputs(b, "</td></tr>" CRLF);
        }

        bputs(b, "<tr><td><strong>From:</strong></td>");
        bputs(b, "<td align=left>");
        cmd_display_write_addr(session, b, env->from);
        bputs(b, "</td></tr>" CRLF);

        if (env->to) {
            bputs(b, "<tr><td><strong>To:</strong></td>");
            bputs(b, "<td align=left>");
            cmd_display_write_addr(session, b, env->to);
            bputs(b, "</td></tr>" CRLF);
        }

        if (env->cc) {
            bputs(b, "<tr><td><strong>Cc:</strong></td>");
            bputs(b, "<td align=left>");
            cmd_display_write_addr(session, b, env->cc);
            bputs(b, "</td></tr>" CRLF);
        }

        bprintf(b, "<tr><td><strong>Date:</strong></td>");
        bprintf(b, "<td>");
        if (env->date)
            html_quote_string(b, env->date);
        else
            html_quote_string(b, "(Unknown date)");
        bprintf(b, "</td></tr>" CRLF);

        bputs(b, "<tr><td><strong>Subject:</strong></td><td>" CRLF);
        if (env->subject)
            buffer_puts_mime_quote_html(b, env->subject);
        else
            html_quote_string(b, "(No subject)");
        bputs(b, "</td></tr>" CRLF);
        bputs(b, "</table></td></tr></table>" CRLF);
    }

    return (T);
}

/* ====================================================================== */

/* show_structure():
 *   Recursively render message MIME structure as collection of
 *   nested <ol> with links to body parts.
 *
 *  session:
 *     pool: Scratch pool
 *   stream: Active message stream
 *    msgno: Message that we are rendering
 *   msguid: Message UID
 *     body: Current body part that we are rendering
 *  section: IMAP session number prefix for this body part
 *        i: Offset count for multipart messages. ($section$i gives offset).
 *             (section and i could be combined, this approach requires
 *              fewer temporary copies of strings).
 *
 * Returns:
 *   NIL if some kind of access error while rendering.
 */

static BOOL
show_structure(struct session *session,
               struct pool *pool,
               struct buffer *b,
               MAILSTREAM * stream,
               unsigned long msgno,
               unsigned long msguid, BODY * body, char *section, long i)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    PART *part;
    PARAMETER *parameter;
    char *type, *name, *url_type, *url_name;

    switch (body->type) {
    case TYPEMULTIPART:
        if (section)
            section = pool_printf(pool, "%s%lu.", section, i);
        else
            section = "";

        if (section[0]) {
            bputs(b, "<li> Multipart:" CRLF);
            bputs(b, "<ol>" CRLF);
        }

        for (i = 1, part = body->nested.part; part != NIL;
             part = part->next, i++) {
            if (!show_structure(session, pool, b, stream, msgno, msguid,
                                &part->body, section, i))
                return (NIL);
        }
        if (section[0])
            bputs(b, "</ol>" CRLF);
        break;
    case TYPEMESSAGE:
        bputs(b, "<li>" CRLF);
        if (section && section[0])
            html_session_bprintf(session, b, "<a href=\"", "\">",
                                 "display/%lu/%lu/%s%lu",
                                 msgno, msguid, section, i);

        else
            html_session_bprintf(session, b, "<a href=\"", "\">",
                                 "display/%lu/%lu/%lu", msgno, msguid, i);

        name = NIL;
        for (parameter = body->parameter; parameter;
             parameter = parameter->next) {
            if (!strcasecmp(parameter->attribute, "NAME"))
                name = parameter->value;
        }
        if (!name && body->description)
            name = body->description;

        if (name) {
            html_quote_string(b, name);
            bputs(b, " (");
        } else
            bputs(b, "(");

        type =
            pool_strcat3(pool, body_types[body->type], "/", body->subtype);
        lcase(type);

        html_quote_string(b, type);

        if (body->size.bytes >= 2048)
            bprintf(b, "), %lu Kbytes</a>" CRLF, body->size.bytes / 1024);
        else if (body->size.bytes != 1)
            bprintf(b, "), %lu bytes</a>" CRLF, body->size.bytes);
        else
            bprintf(b, "), 1 byte</a>" CRLF);

        /* Download link for sub-message */
        type = "application/octet-stream";
        url_type = string_url_encode(pool, type);

        if (name) {
            char *url_name = string_url_encode(pool, name);

            bprintf(b,
                    "<a href=\"%s/NOSEQ/rawdisplay/%lu/%lu/%s%lu/%s/%s\">",
                    session->url_prefix_bsession, msgno, msguid,
                    ((section) ? section : ""), i, url_type, url_name);
        } else
            bprintf(b,
                    "<a href=\"%s/NOSEQ/rawdisplay/%lu/%lu/%s%lu/%s/msg-%lu\">",
                    session->url_prefix_bsession, msgno, msguid,
                    ((section) ? section : ""), i, url_type, msguid);

        bputs(b, "&nbsp;&nbsp;");
        bputs(b, "Download this section</a>" CRLF);

        bputs(b, "</li>" CRLF);

        if (section)
            section = pool_printf(pool, "%s%lu.", section, i);
        else
            section = "1.";

        if (!strcmp(body->subtype, "RFC822")
            && (body = body->nested.msg->body)) {
            if (body->type == TYPEMULTIPART) {
                bprintf(b, "<ol>" CRLF);
                for (i = 1, part = body->nested.part; part != NIL;
                     part = part->next, i++) {
                    if (!show_structure(session, pool, b, stream,
                                        msgno, msguid, &part->body,
                                        section, i))
                        return (NIL);
                }
                bprintf(b, "</ol>" CRLF);
            } else {
                /* Nested singlepart message */
                bprintf(b, "<ol>" CRLF);
                if (!show_structure(session, pool, b, stream,
                                    msgno, msguid, body, section, 1))
                    return (NIL);
                bprintf(b, "</ol>" CRLF);
            }
        }
        break;
    case TYPETEXT:
        type =
            pool_strcat3(pool, body_types[body->type], "/", body->subtype);
        lcase(type);
        bputs(b, "<li>" CRLF);

        html_session_bprintf(session, b, "<a href=\"", "\">(",
                             "display/%lu/%lu/%s%lu",
                             msgno, msguid, (section) ? section : "", i);

        html_quote_string(b, type);
        if (body->size.lines != 1)
            bprintf(b, "), %lu lines</a>" CRLF, body->size.lines);
        else
            bprintf(b, "), 1 line</a>" CRLF);

        if (body->type != TYPEOTHER) {
            type = pool_strcat3(pool, body_types[body->type], "/",
                                body->subtype);
            lcase(type);
        } else
            type = "application/octet-stream";
        
        name = NIL;
        for (parameter = body->parameter; parameter;
             parameter = parameter->next) {
            if (!strcasecmp(parameter->attribute, "NAME"))
                name = parameter->value;
        }
        if (!name && body->description)
            name = body->description;
        
        if (!name)
            name = "";

        url_type = string_url_encode(pool, type);
        url_name = string_url_encode(pool, name);

        bputs(b, "&nbsp;&nbsp;");
        bprintf(b, "<a href=\"%s/NOSEQ/rawdisplay/%lu/%lu/%s%lu/%s/%s\">",
                session->url_prefix_bsession, msgno, msguid,
                ((section) ? section : ""), i, url_type, url_name);
        bputs(b, "Download this text</a>" CRLF);
        break;
    default:
        if (body->type != TYPEOTHER) {
            type =
                pool_strcat3(pool, body_types[body->type], "/",
                             body->subtype);
            lcase(type);
        } else
            type = "application/octet-stream";

        name = NIL;
        for (parameter = body->parameter; parameter;
             parameter = parameter->next) {
            if (!strcasecmp(parameter->attribute, "NAME"))
                name = parameter->value;
        }
        if (!name && body->description)
            name = body->description;

        if (!name)
            name = "";

        url_type = string_url_encode(pool, type);
        url_name = string_url_encode(pool, name);

        bputs(b, "<li>" CRLF);
        if (prefs->use_icons) {
            bprintf(b,
                    "<a href=\"%s/NOSEQ/rawdisplay/%lu/%lu/%s%lu/%s/%s\">",
                    session->url_prefix_bsession, msgno, msguid,
                    ((section) ? section : ""), i, url_type, url_name);

            html_icon(session, b, "paperclip-small.gif", "[paperclip]");
            bputs(b, "</a>" CRLF);
        }

        bprintf(b, "<a href=\"%s/NOSEQ/rawdisplay/%lu/%lu/%s%lu/%s/%s\">",
                session->url_prefix_bsession, msgno, msguid,
                ((section) ? section : ""), i, url_type, url_name);

        html_quote_string(b, name);
        bputs(b, " (");
        html_quote_string(b, type);

        if (body->size.bytes > 1024)
            bprintf(b, "), %lu K</a>" CRLF, body->size.bytes / 1024);
        else if (body->size.bytes != 1)
            bprintf(b, "), %lu bytes</a>" CRLF, body->size.bytes);
        else
            bprintf(b, "), 1 byte</a>" CRLF);
        bputs(b, "</li>" CRLF);

        break;
    }
    return (T);
}

/* ====================================================================== */

/* Small utility routine stolen from PINE */

static STRINGLIST *new_strlst(char **l)
{
    STRINGLIST *sl = mail_newstringlist();

    sl->text.data = (unsigned char *) (*l);
    sl->text.size = strlen(*l);
    sl->next = (*++l) ? new_strlst(l) : NULL;
    return (sl);
}

/* ====================================================================== */

void cmd_display(struct session *session)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct request *request = session->request;
    struct msgmap *zm = session->zm;
    struct pool *pool = request->pool;
    unsigned long msgno, msguid;
    char *section;
    char *init_msg, *decode_msg;
    unsigned long len;
    struct buffer *b = request->write_buffer;
    MAILSTREAM *stream = session->stream;
    BODY *body = NIL;
    char *type;
    BOOL show_html = NIL;

    if (request->method == POST) {
        char *cmd, *folder;

        request_decode_post(request);

        if (assoc_lookup(request->form, "sub_folder_dialogue") &&
            (folder = assoc_lookup(request->form, "folder")) && folder[0])
        {
            if ((cmd = assoc_lookup(request->form, "command")) &&
                !strcmp(cmd, "copy")) {
                session->aggregate = T;
                session_redirect(session, request,
                                 pool_strcat(request->pool, "copy_msg/",
                                             folder));
            } else
                session_redirect(session, request,
                                 pool_strcat(request->pool, "change/",
                                             folder));
            return;
        }
    }

    if (!msgmap_update(zm)) {
        session_message(session, "Failed to update msgmap");
        session_redirect(session, request, "restart");
        return;
    }

    if (msgmap_size(zm) == 0) {
        session_message(session, "Folder is empty");
        session_redirect(session, request, "list");
        return;
    }

    /* Save options if anything changed */
    if (options->save)
        session_streams_save_options(session);

    /* Record last command for compose/cancel and friends */
    session->compose_parent_cmd = "display";

    /* Record last command for save/cancel */
    session->copy_parent_cmd = "display";

    section = NIL;
    if (request->argc >= 4)
        section = request->argv[3];

    if (request->argc >= 3) {
        msgno = atoi(request->argv[1]);
        msguid = atoi(request->argv[2]);

        if (!(msgno = stream_check_uid(session, stream, msgno, msguid))) {
            session_redirect(session, request, "list");
            return;
        }
    } else {
        msgno = session->last_displayed;

        if ((msgno == 0) || (msgno > msgmap_size(zm)))
            msgno = msgmap_value(zm, msgmap_size(zm));

        msguid = ml_uid(session, stream, msgno);
    }

    session->current = msgno;
    session->last_displayed = msgno;

    /* Fetch message structure */
    if (!ml_fetch_structure(session, stream, msgno, &body, NIL)) {
        session_redirect(session, request, "restart");
        return;
    }

    /* Toolbar */
    html_start(session, b, "Message Display");
    html_banner_toolbar(session, b, "display", HTML_BANNER_CALLBACK_SAVE);

    /* Start message */
    cmd_display_toolbar(session, stream, b, msgno);

    /* Display message header */
    if (!cmd_display_hdr(session, b, msgno)) {
        session_redirect(session, request, "restart");
        return;
    }

    /* Print out MIME bodystructure if multipart message */
    if ((body->type == TYPEMULTIPART) || (body->type == TYPEMESSAGE)) {
        bprintf(b, "<hr>" CRLF);
        html_session_bprintf(session, b, "MIME structure of <a href=\"",
                             "\">this message</a>, including any attachments:"
                             CRLF, "display/%lu/%lu", msgno, msguid);

        bputs(b, "<ol>" CRLF);
        if (!show_structure(session, pool, b, stream,
                            msgno, msguid, body, NIL, 1)) {
            session_redirect(session, request, "restart");
            return;
        }
        bputs(b, "</ol>" CRLF);
        bputs(b, "<hr>" CRLF);
    }

    if (section)
        bprintf(b, "Section %s:<br>" CRLF, section);
    else
        section = "1";

    /* Get body structure for specific message part that we are looking at */
    if ((body = ml_body(session, stream, msgno, section)) == NIL) {
        session_redirect(session, request, "restart");
        return;
    }

    if (body->type == TYPEMESSAGE) {
        char *text;
        static char *short_hdrs[] =
            { "from", "to", "cc", "date", "subject", NIL };
        static STRINGLIST *hdrslist_cache = NIL;
        STRINGLIST *hdrslist;

        if (!hdrslist_cache)
            hdrslist_cache = new_strlst(short_hdrs);

        hdrslist = (session->full_hdrs) ? NIL : hdrslist_cache;

        bprintf(b, "<pre>" CRLF);
        if (!
            ((text =
              ml_fetch_header(session, stream, msgno, section, hdrslist,
                              &len, 0))
             && (init_msg =
                 ml_fetch_body(session, stream, msgno, section, &len,
                               0)))) {
            session_redirect(session, request, "restart");
            return;
        }
        html_quote_string(b, text);

        /* Don't try to decode the message contents: substructure links do this */
        decode_msg = init_msg;

        html_quote_string(b, decode_msg);
        bprintf(b, "</pre>" CRLF);

    } else if (body->type == TYPETEXT) {
        type =
            pool_strcat3(pool, body_types[body->type], "/", body->subtype);
        lcase(type);

        /* Should check text part! */

        if (!
            (init_msg =
             ml_fetchbody(session, stream, msgno, section, &len))) {
            session_redirect(session, request, "restart");
            return;
        }

        /* Strip off encoding */
        switch (body->encoding) {
        case ENCBASE64:
            if (!
                (decode_msg =
                 (char *) rfc822_base64((unsigned char *) init_msg,
                                        body->size.bytes, &len))) {
                /* Decode failed */
                decode_msg = init_msg;
                len = body->size.bytes;
            }
            break;
        case ENCQUOTEDPRINTABLE:
            if (!
                (decode_msg =
                 (char *) rfc822_qprint((unsigned char *) init_msg,
                                        body->size.bytes, &len))) {
                /* Decode failed */
                decode_msg = init_msg;
                len = body->size.bytes;
            }
            break;
        case ENC7BIT:
        case ENC8BIT:
        case ENCBINARY:
        case ENCOTHER:
        default:
            decode_msg = init_msg;
            len = body->size.bytes;
        }

        if ((prefs->html_inline) && !strcmp(type, "text/html"))
            show_html = T;
        else if (prefs->html_inline_auto) {
            char *s = decode_msg;

            while ((*s == ' ') || (*s == '\t') || (*s == '\015')
                   || (*s == '\012'))
                s++;

            if (!strncasecmp(s, "<html>", strlen("<html>")))
                show_html = T;
            else if (!strncasecmp
                     (s, "<!doctype html ", strlen("<!doctype html ")))
                show_html = T;
            else
                show_html = NIL;
        } else
            show_html = NIL;

        if (show_html) {
            if (decode_msg == init_msg)
                decode_msg = strdup(init_msg);
            html_secure(session, b, decode_msg);
        } else {
            bprintf(b, "<pre>" CRLF);
            wrap_line_html(session, b, decode_msg, 80);
            bprintf(b, "</pre>" CRLF);
        }

        if (decode_msg != init_msg)
            fs_give((void **) &decode_msg);
    } else {
        bputs(b, "<pre>" CRLF);
        bputs(b, "Not a mail message. Download link follows:" CRLF);
        bputs(b, "</pre>" CRLF);

        if (!show_structure(session, pool, b, stream,
                            msgno, msguid, body, NIL, 1L)) {
            session_redirect(session, request, "restart");
            return;
        }
        bputs(b, "<pre>" CRLF);
        bputs(b, "" CRLF);
        bputs(b, "</pre>" CRLF);
    }

    cmd_display_toolbar(session, stream, b, msgno);

    if (prefs->use_tail_banner)
        html_banner_toolbar(session, b, "display",
                            HTML_BANNER_SUPPRESS_STATUS);

    html_end(b);

    /* Send out the response */
    response_html(request, 200);
}
