/* $Cambridge: hermes/src/prayer/prayer/cmd_favourites.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_inbox_line(struct session *session,
                    struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    struct options *options = session->options;
    struct favourite_list *fl = options->favourite_list;
    char *color;

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"10%\">" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "<td width=\"40%\">INBOX</td>" CRLF);

    bputs(b, "<td nowrap width=\"25%\">");
    if (list_length(fl->list) > 0L) {
        if (fl->preferred && (!strcasecmp(fl->preferred, "inbox")))
            bputs(b, "Current Preferred");
        else
            html_textlink_session(session, b, "preferred/INBOX",
                                  "Make Preferred");
    }
    bputs(b, "</td>" CRLF);
    bputs(b, "<td width=\"25%\">&nbsp;</td>");
    bputs(b, "</tr>" CRLF);
}


static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>");
    if (session->cwd && session->cwd[0]) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "favourites?cwd=%s",
                             string_canon_path_encode(pool, "", parent));
        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a>" CRLF);
    } else
        html_icon(session, b, "left-inactive.gif", "[back]");

    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "favourites?cwd=%s", name);

    html_icon(session, b, "right.gif", "[directory]");
    bputs(b, "</a></td><td>" CRLF);
    html_session_bprintf(session, b, "<a href=\"", "\">" CRLF,
                         "favourites?cwd=%s", name);

    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_favourite_folder_line(struct session *session,
                               struct pool *pool,
                               struct buffer *b, char *folder, BOOL shaded)
{
    char *name = string_canon_path_encode(pool, "", folder);
    struct options *options = session->options;
    struct favourite_list *fl = options->favourite_list;
    struct config_theme *theme = session->theme_main;
    char *color;

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td>" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td nowrap>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "unsubscribe", name,
                                "Remove from Favourites");

    if (fl->preferred && !strcmp(fl->preferred, folder))
        bputs(b, "<td nowrap>Current Preferred</td>");
    else
        html_textlink_mailbox_table(session, b, "preferred", name,
                                    "Make Preferred");

    bprintf(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color;
    char *name = string_canon_path_encode(pool, session->cwd, folder);

    color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</td>" CRLF);

    bprintf(b, "<td nowrap>" CRLF);
    html_quote_string(b, folder);
    bprintf(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "subscribe", name,
                                "Add to Favourites");

    bputs(b, "<td>&nbsp;</td>");
    bputs(b, "</tr>" CRLF);
}

/* ====================================================================== */

static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b, struct dirlist *dl)
{
    struct config_theme *theme = session->theme_main;
    struct favourite_list *fl = session->options->favourite_list;
    struct list_item *li;
    BOOL shaded = NIL;

    /* Title Bar */
    html_start(session, b, "Favourite Mail Folders");
    html_banner_toolbar(session, b, "favourites", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Favourite mail folders</h2>" CRLF);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "favourites");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>");
    bputs(b, "<table><tr><td>" CRLF);
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    bputs(b, "</td></tr></table>" CRLF);
    bputs(b, "</td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr>" CRLF);
    html_textlink_session_table(session, b, "folders", "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/favourites",
                                "Refresh");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    /* Inbox item */
    if (session->cwd && session->cwd[0]) {
        generate_inbox_line(session, pool, b, shaded);
        shaded = (shaded) ? NIL : T;
    }

    if (list_length(fl->list) > 0L) {
        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");
        bputs(b, "<td>Favourite mail folders:</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        shaded = NIL;
        for (li = fl->list->head; li; li = li->next) {
            generate_favourite_folder_line(session, pool, b, li->name,
                                           shaded);
            shaded = (shaded) ? NIL : T;
        }

        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");
        bputs(b, "<td>Folder list:</td>");
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        shaded = (shaded) ? NIL : T;
    }

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, "folders", "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/favourites",
                                "Refresh");
    bputs(b, "</tr></table></td></tr></table>" CRLF);

    html_end(b);
}

/* ====================================================================== */

void cmd_favourites(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != NIL))
            string_strdup(&session->dir_filter, s);

        if (request->argc > 1) {
            char *redirect = NIL;

            if (assoc_lookup(h, "sub_subscribe"))
                redirect =
                    pool_strcat(request->pool, "subscribe/",
                                request->argv[1]);
            else if (assoc_lookup(h, "sub_unsubscribe"))
                redirect =
                    pool_strcat(request->pool, "unsubscribe/",
                                request->argv[1]);
            else if (assoc_lookup(h, "sub_preferred"))
                redirect =
                    pool_strcat(request->pool, "preferred/",
                                request->argv[1]);

            if (redirect) {
                session_redirect(session, request, redirect);
                return;
            }
        }

    }

    html_folderlist(session, request->pool, b,
                    dircache_fetch(session, session->cwd));

    response_html(request, 200);
}
