/* $Cambridge: hermes/src/prayer/prayer/cmd_filter_select.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(session->pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"10%\">");
    if (session->cwd && session->cwd[0]) {
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "filter_select?cwd=%s",
                             string_canon_path_encode(pool, "", parent));
        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a>" CRLF);
    } else {
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</a>" CRLF);
    }
    bputs(b, "</td>");
    bputs(b, "<td width=\"40%\">&nbsp;</td>");
    bputs(b, "<td width=\"25%\">&nbsp;</td>");
    bputs(b, "<td width=\"25%\">&nbsp;</td>");
    bputs(b, "</tr>");
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "filter_select?cwd=%s", name);

    html_icon(session, b, "right.gif", "[directory]");
    bprintf(b, "</a></td>" CRLF);
    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "filter_select?cwd=%s", name);

    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);

    bputs(b, "</tr>" CRLF);
}

static void
generate_favourite_folder_line(struct session *session,
                               struct pool *pool,
                               struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, "", folder);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "filter_mbox/copy", name,
                                "Select (save copy to inbox)");

    html_textlink_mailbox_table(session, b, "filter_mbox/nocopy", name,
                                "Select (no copy)");

    bputs(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, folder);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "filter_mbox/copy", name,
                                "Select (save copy to inbox)");

    html_textlink_mailbox_table(session, b, "filter_mbox/nocopy", name,
                                "Select (no copy)");

    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */


static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b, struct dirlist *dl)
{
    struct config_theme *theme = session->theme_main;
    struct favourite_list *fl = session->options->favourite_list;
    struct list_item *li;
    BOOL shaded = NIL;

    /* Title Bar */
    html_start(session, b, "Filter Select Mailbox");
    html_banner_toolbar(session, b, "filter_select", 0);

    /* Title line */
    bprintf(b,
            "<h2 align=\"center\">Select Target Mail Folder...</h2>" CRLF);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "filter_select");

    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "sub_apply", "Apply");
    html_form_submit(b, "sub_cancel", "Cancel");
    html_form_end(b);

    bputs(b, "<table width=\"100%\">" CRLF);

    if (list_length(fl->list) > 0L) {
        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");
        bputs(b, "<td>Favourite mail folders:</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        shaded = NIL;
        for (li = fl->list->head; li; li = li->next) {
            generate_favourite_folder_line(session, pool, b, li->name,
                                           shaded);
            shaded = (shaded) ? NIL : T;
        }

        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");
        bputs(b, "<td>Normal folder listing:</td>");
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);
    }

    shaded = NIL;

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "create/filter_select");

    bputs(b, "Create" CRLF);
    bputs(b, "<select name=\"type\" size=1>" CRLF);
    bputs(b, "<option value=\"mailbox\" selected>mailbox</option>" CRLF);
    bputs(b, "<option value=\"directory\">directory</option>" CRLF);
    bputs(b, "</select>" CRLF);
    bputs(b, "with name" CRLF);
    bputs(b, "<input name=\"name\">" CRLF);
    bputs(b,
          "<input type=\"submit\" name=\"sub_create\" value=\"Create\">"
          CRLF);
    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

void cmd_filter_select(struct session *session)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if (assoc_lookup(h, "sub_cancel")) {
            session_redirect(session, request, "filter");
            return;
        }

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != 0))
            string_strdup(&session->dir_filter, s);
    }

    html_folderlist(session, pool, b,
                    dircache_fetch(session, session->cwd));

    response_html(request, 200);
}
