/* $Cambridge: hermes/src/prayer/prayer/cmd_folders.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_inbox_line(struct session *session,
                    struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td width=\"10%\">");
    html_session_bputs(session, b, "<a href=\"", "\">", "change/INBOX");
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td width=\"40%\">");
    html_session_bputs(session, b, "<a href=\"", "\">INBOX</a>",
                       "change/INBOX");
    bputs(b, "</td>");

    bputs(b, "<td width=\"25%\">&nbsp;</td>" CRLF);
    bputs(b, "<td width=\"25%\">&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}


static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    if (session->cwd && session->cwd[0]) {
        html_session_bprintf(session, b, "<td><a href=\"", "\">",
                             "folders?cwd=%s",
                             string_canon_path_encode(pool, "", parent));
        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a></td>" CRLF);
    } else {
        bputs(b, "<tr><td>");
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</td>" CRLF);
    }
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "folders?cwd=%s", name);

    html_icon(session, b, "right.gif", "[directory]");
    bputs(b, "</a></td>" CRLF);

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "folders?cwd=%s", name);
    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);

    html_textlink_mailbox_table(session, b, "rename", name, "Rename");
    html_textlink_mailbox_table(session, b, "rm/directory", name,
                                "Delete");
    bputs(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, folder);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "change/%s", name);

    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    html_session_bprintf(session, b, "<td nowrap><a href=\"", "\">" CRLF,
                         "change/%s", name);

    html_quote_string(b, folder);
    bprintf(b, "</a></td>" CRLF);

    if (strcmp(name, "INBOX") != 0) {
        html_textlink_mailbox_table(session, b, "rename", name, "Rename");
        html_textlink_mailbox_table(session, b, "rm/mailbox", name,
                                    "Delete");
    } else
        bputs(b, "<td></td><td></td>");

    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */

static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b, struct dirlist *dl)
{
    struct config_theme *theme = session->theme_main;
    BOOL shaded = NIL;

    html_start(session, b, "Mail Folders");

    html_banner_toolbar(session, b, "folders", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Mail Folders</h2>" CRLF);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "folders");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td><table><tr><td>" CRLF);
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr>" CRLF);
    html_textlink_session_table(session, b, "dir_check/folders",
                                "Refresh");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "transfer",
                                "Transfer Folders");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "favourites",
                                "Favourite Folders");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    /* Inbox item */
    if (session->cwd && session->cwd[0]) {
        generate_inbox_line(session, pool, b, shaded);
        shaded = (shaded) ? NIL : T;
    }

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "create/folders");
    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td><table><tr><td>" CRLF);
    bputs(b, "Create" CRLF);
    bputs(b, "<select name=\"type\" size=1>" CRLF);
    bputs(b, "<option value=\"mailbox\" selected>mailbox</option>" CRLF);
    bputs(b, "<option value=\"directory\">directory</option>" CRLF);
    bputs(b, "</select>" CRLF);
    bputs(b, "with name" CRLF);
    bputs(b, "<input name=\"name\">" CRLF);
    bputs(b,
          "<input type=\"submit\" name=\"create\" value=\"Create\">" CRLF);
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/folders",
                                "Refresh");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "transfer",
                                "Transfer Folders");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "favourites",
                                "Favourite Folders");
    bputs(b, "</tr></table></td></tr></table>" CRLF);

    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

void cmd_folders(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    /* Record last command for compose/cancel and friends */
    session->compose_parent_cmd = "folders";

    /* Save options if anything changed */
    if (session->options->save)
        session_streams_save_options(session);

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != 0))
            string_strdup(&session->dir_filter, s);
    }

    html_folderlist(session, request->pool, b,
                    dircache_fetch(session, session->cwd));

    response_html(request, 200);
}
