/* $Cambridge: hermes/src/prayer/prayer/cmd_fullname.c,v 1.3 2003/07/30 13:45:47 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
html_form_table_passwd_field(struct buffer *b,
                             char *desc, char *name, char *value,
                             unsigned long size)
{
    bputs(b, "<tr>" CRLF);

    if (desc && desc[0])
        bprintf(b, "<td>%s</td>", desc);

    bprintf(b, "<td><input name=\"%s\" value=\"", name);
    if (value)
        html_quote_string(b, value);
    bprintf(b, "\" size=\"%lu\"></td>" CRLF, size);

    bputs(b, "</tr>" CRLF);
}

static void generate_form(struct session *session, char *new)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    char *oldname;

    oldname = account_fullname(account, request->pool);

    /* Title bar */
    html_start(session, b, "Change Fullname");
    html_banner_toolbar(session, b, "fullname", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Change Fullname</h2>" CRLF);

    html_form_start(session, b, "POST", "fullname");
    bputs(b, "<table>" CRLF);

    bputs(b, "<tr>" CRLF);
    bputs(b, "<td>Current Fullname:</td>" CRLF);
    bputs(b, "<td>");
    if (oldname)
        bputs(b, account->fullname);
    else
        bputs(b, "(Unknown)");

    bputs(b, "</td>" CRLF);
    bputs(b, "</tr>" CRLF);
    html_form_table_passwd_field(b, "Fullname:", "new", new, 20);
    bputs(b, "</table>" CRLF);

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_change", "Change Fullname");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    html_form_end(b);

    if (config->accountd_fullname_delay > (2 * 60 * 60))
        bprintf(b,
                ("NB: It can take up to %lu hours for fullname changes "
                 "to take effect" CRLF),
                (config->accountd_fullname_delay / (60 * 60)));
    else if (config->accountd_fullname_delay > 120)
        bprintf(b,
                ("NB: It can take up to %lu minutes for fullname changes "
                 "to take effect" CRLF),
                (config->accountd_fullname_delay / 60));
    else if (config->accountd_fullname_delay > 1)
        bprintf(b,
                ("NB: It can take up to %lu seconds for fullname changes "
                 "to take effect" CRLF),
                (config->accountd_fullname_delay));
    html_end(b);
    response_html(request, 200);        /* Success */
}

/* ====================================================================== */

static BOOL process_form(struct session *session, char *new)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;

    if (!(new && new[0])) {
        session_message(session, "New password not supplied");
        return (NIL);
    }

    if (!account_change_fullname(session->account, pool, new)) {
        /* session_message fallback only: should be redundant */
        session_message(session, "Unable to change fullname");
        return (NIL);
    }

    return (T);
}

void cmd_fullname(struct session *session)
{
    struct request *request = session->request;
    char *new;

    if (request->method == POST) {
        struct assoc *h;

        request_decode_post(request);
        h = request->form;

        if (assoc_lookup(h, "sub_change")) {
            if ((new = assoc_lookup(h, "new")))
                new = string_trim_whitespace(new);

            if (!process_form(session, new)) {
                generate_form(session, new);
                return;
            }
        }

        session_redirect(session, request, "manage");
        return;
    }

    generate_form(session, NIL);
}
