/* $Cambridge: hermes/src/prayer/prayer/cmd_help.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static BOOL valid_name(char *s)
{
    if (*s == '.')
        return (NIL);

    if (*s == '/')
        return (NIL);

    while (*s) {
        if ((s[0] == '.') && (s[1] == '.'))
            return (NIL);
        s++;
    }

    return (T);
}


void cmd_help(struct session *session)
{
    struct config_theme *theme = session->theme_help;
    struct request *request = session->request;
    struct options *options = session->options;
    struct prefs *prefs;
    struct config *config = session->config;
    struct buffer *b = request->write_buffer;
    char *help_item;
    FILE *file;
    char *name;
    char *back;

    if (options->prefs_work)
        prefs = options->prefs_work;    /* Only true on prefs screen */
    else
        prefs = options->prefs;

    if (request->argc == 3) {
        back = help_item = pool_printf(request->pool, "%s/%s",
                                       request->argv[1], request->argv[2]);
        memblock_puts(session->last_help, help_item);
    } else if (request->argc == 2) {
        help_item = request->argv[1];

        if (strcmp(help_item, "toolbar") != 0) {
            memblock_puts(session->last_help, help_item);
            back = help_item;
        } else
            back = memblock_string(session->last_help);
    } else
        back = help_item = memblock_string(session->last_help);


    /* Title bar */
    html_help_start(session, b, "Help");
    html_banner_toolbar_help(session, b,
                             pool_strcat(request->pool, "help/",
                                         help_item), back);

    bputs(b, "<h1 align=\"center\">Help Text</h1>" CRLF);

    if (!valid_name(help_item)) {
        bputs(b, "Not a valid help text name!" CRLF);
        html_end(b);
        response_html(request, 200);
        return;
    }

    if (strcmp(help_item, "toolbar") != 0) {
        bputs(b, "<br>" CRLF);
        html_textlink_session(session, b,
                              "help/toolbar", "Explain master toolbar");
        bputs(b, "<br>" CRLF);
    } else {
        bputs(b, "<br>" CRLF);
        html_textlink_session(session, b,
                              pool_strcat(request->pool, "help/", back),
                              "Go Back to Help Page");
        bputs(b, "<br>" CRLF);
    }

    name =
        pool_printf(request->pool, "%s/%s.html", config->help_dir,
                    help_item);

    if ((file = fopen(name, "r")) != NIL) {
        struct buffer *tbuf = buffer_create(request->pool, 16384);
        char *s;
        int c;

        while ((c = fgetc(file)) != EOF)
            bputc(tbuf, c);
        fclose(file);

        s = buffer_fetch(tbuf, 0, buffer_size(tbuf), NIL);

        while ((c = *s)) {
            if (c == '_') {
                if (!strncmp
                    (s, "__PREFIX_SESSION__",
                     strlen("__PREFIX_SESSION__"))) {
                    if (!session->use_short)
                        bputs(b, session->url_prefix_session);
                    s += strlen("__PREFIX__");
                    continue;
                } else if (!strncmp(s, "__ACTION__", strlen("__ACTION__"))) {
                    if (!session->use_short) {
                        bputs(b, session->url_prefix_session);
                        bputs(b, "/action_stub/");
                        bputs(b, help_item);
                    } else
                        html_session_bprintf(session, b, "", "",
                                             "action_stub/%s", help_item);
                    s += strlen("__ACTION__");
                    continue;
                } else if (!strncmp(s, "__PREFIX__", strlen("__PREFIX__"))) {
                    if (!session->use_short)
                        bputs(b, session->url_prefix);
                    s += strlen("__PREFIX__");
                    continue;
                } else if (!strncmp(s, "__ICONS__", strlen("__ICONS__"))) {
                    if (!session->use_short)
                        bputs(b, session->url_prefix);
                    bputs(b, "/icons/");
                    s += strlen("__ICONS__");
                    continue;
                } else
                    if (!strncmp(s, "__SERVICE__", strlen("__SERVICE__")))
                {
                    if (config->login_service_name)
                        html_quote_string(b, config->login_service_name);
                    else
                        bputs(b, "the mail server");
                    s += strlen("__SERVICE__");
                    continue;
                } else if (!strncmp(s, "__BANNER__", strlen("__BANNER__"))) {
                    if (config_theme_check_bgcolor
                        (theme, theme->bgcolor_banner))
                        bprintf(b, " bgcolor=\"%s\"",
                                theme->bgcolor_banner);
                    s += strlen("__BANNER__");
                    continue;
                } else if (!strncmp(s, "__ROW1__", strlen("__ROW1__"))) {
                    if (config_theme_check_bgcolor
                        (theme, theme->bgcolor_row1))
                        bprintf(b, " bgcolor=\"%s\"", theme->bgcolor_row1);
                    s += strlen("__ROW1__");
                    continue;
                } else if (!strncmp(s, "__ROW2__", strlen("__ROW2__"))) {
                    if (config_theme_check_bgcolor
                        (theme, theme->bgcolor_row2))
                        bprintf(b, " bgcolor=\"%s\"", theme->bgcolor_row2);
                    s += strlen("__ROW2__");
                    continue;
                } else if (!strncmp(s, "__STATUS__", strlen("__STATUS__"))) {
                    if (config_theme_check_bgcolor
                        (theme, theme->bgcolor_status))
                        bprintf(b, " bgcolor=\"%s\"",
                                theme->bgcolor_status);
                    s += strlen("__STATUS__");
                    continue;
                } else
                    if (!strncmp
                        (s, "__STATUS_NONE__", strlen("__STATUS_NONE__")))
                {
                    if (config_theme_check_bgcolor
                        (theme, theme->bgcolor_status_none))
                        bprintf(b, " bgcolor=\"%s\"",
                                theme->bgcolor_status_none);
                    s += strlen("__STATUS_NONE__");
                    continue;
                }
            }
            bputc(b, c);
            s++;
        }
    } else
        bprintf(b, "Sorry, no help text for \"%s\" yet!" CRLF, help_item);

    html_end(b);
    response_html(request, 200);        /* Success */
}
