/* $Cambridge: hermes/src/prayer/prayer/cmd_list.c,v 1.3 2004/03/30 09:27:37 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* Couple of small support routines for alt_addr stuff */

static ADDRESS *cmd_list_alt_addr_generate(struct session *session,
                                           struct pool *pool,
                                           struct prefs *prefs)
{
    ADDRESS *alt_addr = NIL;
    char *text;

    if (prefs->from_address && prefs->from_address[0]) {
        if (prefs->alt_addr && prefs->alt_addr[0])
            text = pool_printf(pool, "%s, %s",
                               prefs->from_address, prefs->alt_addr);
        else
            text = pool_strdup(pool, prefs->from_address);
    } else
        text = pool_strdup(pool, prefs->alt_addr);

    if (text && text[0] && !(alt_addr=addr_parse_destructive(text, ""))) {
        session_message(session, "Alt-address invalid - %s", ml_errmsg());
        return(NIL);
    }

    return (alt_addr);
}

static void cmd_list_alt_addr_free(ADDRESS * alt_addr)
{
    mail_free_address(&alt_addr);
}

/* ====================================================================== */

/* cmd_list_toolbar_hdr() ************************************************
 *
 * Generate toolbar above message list
 ************************************************************************/

static void
cmd_list_toolbar_hdr(struct session *session,
                     struct buffer *b, unsigned long zm_offset)
{
    struct config_theme *theme = session->theme_main;
    struct prefs *prefs = session->options->prefs;
    struct msgmap *zm = session->zm;
    unsigned long count = msgmap_size(session->zm);
    unsigned long per_page = prefs->msgs_per_page;
    unsigned long page;
    unsigned long pages;
    char *marked;
    char *messages;
    BOOL use_icons = session->user_agent->use_icons;

    bputs(b, "<table width=\"100%\" cellspacing=0 cellpadding=0>" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);

    if (count > 0) {
        page = ((zm_offset - 1) / prefs->msgs_per_page) + 1;
        pages = ((count - 1) / prefs->msgs_per_page) + 1;
        marked = (msgmap_have_zoom(zm)) ? " selected" : "";
        messages = (count == 1) ? "message" : "messages";
    } else {
        page = 0;
        pages = 0;
        marked = "";
        messages = "messages";
    }

    if (page > 1) {
        html_button_msg_table(session, b, "list",
                              msgmap_value(zm, 1), "leftleft.gif",
                              "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_msg_table(session, b, "list",
                              msgmap_value(zm, zm_offset - per_page),
                              "left.gif", "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        if ((zm_offset + per_page) > count)
            html_button_msg_table(session, b, "list",
                                  msgmap_value(zm, count), "right.gif",
                                  "Next");
        else
            html_button_msg_table(session, b, "list",
                                  msgmap_value(zm, zm_offset + per_page),
                                  "right.gif", "Next");

        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_msg_table(session, b, "list", msgmap_value(zm, count),
                              "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }

    bputs(b, "<td>" CRLF);
    bputs(b, ("<select name =\"aggregate\" size=1>" CRLF
              "<option value=\"delete\">Delete marked</option>" CRLF
              "<option value=\"undelete\">Undelete marked</option>" CRLF
              "<option value=\"read\">Flag as read</option>" CRLF
              "<option value=\"unread\">Flag as unread</option>" CRLF
              "<option value=\"forward\">Forward marked</option>" CRLF
              "<option value=\"copy\" selected>Copy marked</option>" CRLF
              "</select>" CRLF));
    bputs(b, "</td>" CRLF);
    bputs(b, "<td>" CRLF);
    bputs(b, ("<input type=\"submit\" name=\"sub_aggregate\" "
              "value=\"Go\">" CRLF));
    bputs(b, "</td>" CRLF);
    bputs(b, "</tr></table></td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"center\"><strong>\"");
    html_quote_string(b, session->foldername);
    bputs(b, "\" with ");
    bprintf(b, "%lu%s %s" CRLF, count, marked, messages);
    bputs(b, "</strong></td>" CRLF);

    bputs(b, "<td width=\"34%\" align=\"right\">" CRLF);
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_textlink_session_table(session, b, "checkpoint", "Refresh");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "expunge", "Expunge");
    if (prefs->use_mark_persist) {
        bputs(b, "<td>|</td>" CRLF);
        if (msgmap_marked_count(session->zm) > 0)
            html_textlink_session_table(session, b, "unmark/all",
                                        "UnMark All");
        else
            html_textlink_session_table(session, b, "mark/all",
                                        "Mark All");
        bputs(b, "<td>|</td>" CRLF);
        if (msgmap_have_zoom(session->zm))
            html_textlink_session_table(session, b, "zoom", "Unzoom");
        else
            html_textlink_session_table(session, b, "zoom", "Zoom");
    } else if (msgmap_have_zoom(session->zm)) {
        bputs(b, "<td>|</td>" CRLF);
        html_textlink_session_table(session, b, "unmark/all",
                                    "Clear Search");
    }

    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "search", "Search");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    bputs(b, "</td></tr></table>" CRLF);
}

/* ====================================================================== */

/* cmd_list_headings() ***************************************************
 *
 * Generate columns headings for mailbox list. 
 ************************************************************************/

static void cmd_list_headings(struct session *session, struct buffer *b)
{
    struct prefs *prefs = session->options->prefs;

    bputs(b, "<tr>");

    bputs(b, "<td width=\"3%\" align=\"center\">" CRLF);
    if (session->zm->nmsgs > 0)
        html_textlink_session(session, b, "list_sort/arrival", "#");
    else
        bputs(b, "#" CRLF);
    bputs(b, "</td>" CRLF);

    if (prefs->use_mark_persist) {
        bputs(b, "<td width=\"2%\" align=\"center\">");
        html_icon(session, b, "on.gif", "Mark");
        bputs(b, "</td>" CRLF);
    } else {
        bputs(b, "<td align=\"center\">");
        bputs(b, "<input type=\"checkbox\" name=\"0@0\" checked>");
        bputs(b, "</td>");
    }

    if (session->user_agent->use_icons) {
        bputs(b, "<td align=\"center\">");
        html_icon(session, b, "msg-unseen.gif", "Status");
        bputs(b, "</td>" CRLF);
    } else {
        bputs(b, "<td>D</td>");
        bputs(b, "<td>N</td>");
        bputs(b, "<td>A</td>");
    }

    bputs(b, "<td width=\"10%\">");
    html_textlink_session(session, b, "list_sort/date", "Date");
    bputs(b, "</td>" CRLF);
    bputs(b, "<td width=\"30%\">");
    html_textlink_session(session, b, "list_sort/from", "From");
    bputs(b, "</td>" CRLF);
    bputs(b, "<td width=\"5%\">");
    html_textlink_session(session, b, "list_sort/size", "Size");
    bputs(b, "</td>" CRLF);
    bputs(b, "<td width=\"40%\">");
    html_textlink_session(session, b, "list_sort/subject", "Subject");
    bputs(b, "</td>" CRLF);
    bputs(b, "<td nowrap align=\"right\" width=\"10%\">");
    html_textlink_session(session, b, "list_sort/references",
                          "Thread Messages");
    bputs(b, "</td></tr>" CRLF);
}

/* ====================================================================== */

/* cmd_list_msg() ********************************************************
 *
 * Generate entry for single message
 ************************************************************************/

static BOOL
cmd_list_msg(struct session *session,
             struct buffer *b,
             MAILSTREAM * stream,
             unsigned long msgno,
             unsigned long depth, ADDRESS * alt_addr, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    struct config *config = session->config;
    struct prefs *prefs = session->options->prefs;
    struct request *request = session->request;
    MESSAGECACHE *elt;
    ENVELOPE *env;
    ADDRESS *addr;
    BOOL use_to;
    char *string;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;


    if (!((elt = ml_elt(session, stream, msgno)) &&
          (env = ml_fetch_structure(session, stream, msgno, NIL, 0))))
        return (NIL);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    /* Message count */
    bprintf(b, "<td align=\"right\"><strong>%lu.</strong></td>" CRLF,
            msgno);

    bputs(b, "<td align=\"center\">");
    if (prefs->use_mark_persist) {
        if (msgmap_has_mark(session->zm, msgno))
            html_button_msg(session, b, "unmark", msgno, "on.gif", "Yes");
        else
            html_button_msg(session, b, "mark", msgno, "off.gif", "No");
    } else {
        bprintf(b, "<input type=\"checkbox\" name=\"%lu@%lu\">",
                msgno, ml_uid(session, stream, msgno));
    }
    bputs(b, "</td>" CRLF);

    if (session->user_agent->use_icons) {
        bputs(b, "<td>");
        if (elt->deleted)
            html_icon(session, b, "msg-deleted.gif", NIL);
        else if (elt->answered)
            html_icon(session, b, "msg-answered.gif", NIL);
        else if (elt->seen)
            html_icon(session, b, "msg-seen.gif", NIL);
        else
            html_icon(session, b, "msg-unseen.gif", NIL);
        bputs(b, "</td>");
    } else {
        /* Message status flags just like PINE and WING */
        bputs(b, "<td>");
        bputs(b, (elt->deleted) ? "D" : "&nbsp;");
        bputs(b, "</td>");
        bputs(b, "<td>");
        bputs(b, (elt->seen) ? "O" : "N");
        bputs(b, "</td>");
        bputs(b, "<td>");
        bputs(b, (elt->answered) ? "A" : "&nbsp;");
        bputs(b, "</td>");
    }

    /* Message Date */
    if (env->date) {
        char *s;
        MESSAGECACHE tmp;
        mail_parse_date(&tmp, env->date);

        if (prefs->use_full_date)
            s = string_date_to_string_long(tmp.month, tmp.day,
                                           tmp.year + BASEYEAR);
        else
            s = string_date_to_string_short(tmp.month, tmp.day);

        if (s && s[0])
            bprintf(b, "<td nowrap>%s</td>" CRLF, s);
        else
            bprintf(b, "<td nowrap>&nbsp;</td>" CRLF, s);
    } else
        bputs(b, "<td nowrap>(No date)</td>" CRLF);

    /* Try and work out if user sent this message
     * Yes => show "To:" rather than from address in header.
     */
    use_to = NIL;
    if ((addr = env->from)) {
        while (alt_addr) {
            if (!strcasecmp(addr->mailbox, alt_addr->mailbox) &&
                !strcasecmp(addr->host, alt_addr->host)) {
                use_to = T;
                break;
            }
            alt_addr = alt_addr->next;
        }

        if (!use_to &&
            !strcasecmp(addr->mailbox, session->username) &&
            config->local_domain_list) {
            /* Find out if address is local use of CRSid
             *  e.g: dpc22@cam.ac.uk, dpc22@hermes.cam.ac.uk */

            struct list_item *li;
            /* Check for username@[username_domain]. */
            for (li = config->local_domain_list->head; li; li = li->next) {
                if (!strcasecmp(li->name, addr->host)) {
                    use_to = T;
                    break;
                }
            }
        }
    }

    /* From or To address depending on context */
    addr = (use_to) ? env->to : env->from;

    if (addr && addr->personal)
        string = addr->personal;
    else if (addr && addr->mailbox && addr->host)
        string =
            pool_printf(request->pool, "%s@%s", addr->mailbox, addr->host);
    else
        string = "(unknown)";

    string =
        (char *) rfc1522_decode(pool_alloc(request->pool, strlen(string)),
                                strlen(string), string, NIL);

    string = string_prune(request->pool, string, config->list_addr_maxlen);

    if (use_to)
        string = pool_strcat(request->pool, "To: ", string);

    html_textlink_msg_table(session, b, "display", msgno, string);

    /* Message size */
    if (elt->rfc822_size > 1024)
        bprintf(b, "  <td align=\"right\">%luK</td>" CRLF,
                elt->rfc822_size / 1024);
    else
        bprintf(b, "  <td align=\"right\">&lt;1K</td>" CRLF);

    /* Message subject */
    if (env->subject && env->subject[0]) {
        string = env->subject;

        string = (char *)
            rfc1522_decode(pool_alloc(request->pool, strlen(string)),
                           strlen(string), string, NIL);

        string = string_prune(request->pool, string,
                              config->list_subject_maxlen);
    } else
        string = "(No subject)";

    bputs(b, "<td nowrap>");

#if 0                           /* XXX Depth count broken, or at least unusable here */
    /* Indent threads with appropriate upper limit */
    if (depth > 5)
        depth = 5;

    while (depth > 0) {
        bputs(b, "&nbsp;");
        bputs(b, "&nbsp;");
        depth--;
    }
#endif

    html_session_bprintf(session, b, "<a href=\"", "\">" CRLF,
                         "display/%lu/%lu", msgno, ml_uid(session,
                                                          session->stream,
                                                          msgno));

    html_quote_string(b, string);
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td align=\"right\">");
    if (elt->deleted)
        html_textlink_msg(session, b, "undelete", msgno, "Undelete");
    else
        html_textlink_msg(session, b, "delete", msgno, "Delete");
    bputs(b, "</td>");

    bputs(b, "</tr>" CRLF);

    return (T);
}

#if 0                           /* Just an experiment */
/* cmd_list_empty() ******************************************************
 *
 * Generate entry for empty message
 ************************************************************************/

static void
cmd_list_empty(struct session *session,
               struct buffer *b, unsigned long offset, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    /* Message offset */
    bprintf(b, "<td align=\"right\"><strong>%lu.</strong></td>" CRLF,
            offset);

    bputs(b, "</tr>" CRLF);
}
#endif

/* ====================================================================== */

/* cmd_list_toolbar_footer() *********************************************
 *
 * Generate toolbar after message list
 ************************************************************************/

static void
cmd_list_toolbar_footer(struct session *session, struct buffer *b,
                        unsigned long zm_offset)
{
    struct config_theme *theme = session->theme_main;
    struct prefs *prefs = session->options->prefs;
    struct msgmap *zm = session->zm;
    unsigned long count = msgmap_size(session->zm);
    unsigned long per_page = prefs->msgs_per_page;
    unsigned long page;
    unsigned long pages;
    BOOL use_icons = session->user_agent->use_icons;

    bputs(b, "<table width=\"100%\">" CRLF);
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td width=\"33%\">" CRLF);
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);

    if (count > 0) {
        page = ((zm_offset - 1) / prefs->msgs_per_page) + 1;
        pages = ((count - 1) / prefs->msgs_per_page) + 1;
    } else {
        page = 0;
        pages = 0;
    }

    if (page > 1) {
        html_button_msg_table(session, b, "list",
                              msgmap_value(zm, 1), "leftleft.gif",
                              "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_msg_table(session, b, "list",
                              msgmap_value(zm, zm_offset - per_page),
                              "left.gif", "Previous");
    } else {
        html_button_inactive_table(session, b, "leftleft-inactive.gif",
                                   "First");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "left-inactive.gif",
                                   "Previous");
    }

    if (!use_icons)
        bputs(b, "<td>|</td>");

    if (page < pages) {
        if ((zm_offset + per_page) > count)
            html_button_msg_table(session, b, "list",
                                  msgmap_value(zm, count), "right.gif",
                                  "Next");
        else
            html_button_msg_table(session, b, "list",
                                  msgmap_value(zm, zm_offset + per_page),
                                  "right.gif", "Next");

        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_msg_table(session, b, "list", msgmap_value(zm, count),
                              "rightright.gif", "Last");
    } else {
        html_button_inactive_table(session, b, "right-inactive.gif",
                                   "Next");
        if (!use_icons)
            bputs(b, "<td>|</td>");
        html_button_inactive_table(session, b, "rightright-inactive.gif",
                                   "Last");
    }

    bputs(b, "<td>" CRLF);
    bputs(b, ("<select name =\"aggregate2\" size=1>" CRLF
              "<option value=\"delete\">Delete marked</option>" CRLF
              "<option value=\"undelete\">Undelete marked</option>" CRLF
              "<option value=\"read\">Flag as read</option>" CRLF
              "<option value=\"unread\">Flag as unread</option>" CRLF
              "<option value=\"forward\">Forward marked</option>" CRLF
              "<option value=\"copy\" selected>Copy marked</option>" CRLF
              "</select>" CRLF));
    bputs(b, "</td>" CRLF);
    bputs(b, "<td>" CRLF);
    bputs(b, ("<input type=\"submit\" name=\"sub_aggregate2\" "
              "value=\"Go\">" CRLF));
    bputs(b, "</td>" CRLF);
    bputs(b, "</tr></table>" CRLF);
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"33%\" align=\"center\">");
    bputs(b, "<table><tr>");
    bputs(b, "<td>Page:</td>");
    bprintf(b, "<td><input name=\"page\" size=4 value=\"%lu\"></td>",
            page);
    bprintf(b, "<td>/%lu</td>", pages);
    bputs(b,
          "<td><input type=\"submit\" name=\"sub_page\" value=\"Go\"></td>");
    bputs(b, "</tr></table>");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td width=\"34%\" align=\"right\">" CRLF);
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_textlink_session_table(session, b, "checkpoint", "Refresh");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "expunge", "Expunge");
    if (prefs->use_mark_persist) {
        bputs(b, "<td>|</td>" CRLF);
        if (msgmap_marked_count(session->zm) > 0)
            html_textlink_session_table(session, b, "unmark/all",
                                        "UnMark All");
        else
            html_textlink_session_table(session, b, "mark/all",
                                        "Mark All");
        bputs(b, "<td>|</td>" CRLF);
        if (msgmap_have_zoom(session->zm))
            html_textlink_session_table(session, b, "zoom", "Unzoom");
        else
            html_textlink_session_table(session, b, "zoom", "Zoom");
    } else if (msgmap_have_zoom(session->zm)) {
        bputs(b, "<td>|</td>" CRLF);
        html_textlink_session_table(session, b, "unmark/all",
                                    "Clear Search");
    }

    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "search", "Search");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);


    bputs(b, "</td></tr></table>" CRLF);
}

/* ====================================================================== */

/* cmd_list_work() *******************************************************
 *
 * Generate list screen. Parent routine will clean up after error.
 ************************************************************************/

static BOOL cmd_list_work(struct session *session)
{
    struct request *request = session->request;
    struct prefs *prefs = session->options->prefs;
    unsigned long first, current, last;
    MAILSTREAM *stream = session->stream;
    struct buffer *b = request->write_buffer;
    struct msgmap *zm = session->zm;
    unsigned long zm_offset;
    ADDRESS *alt_addr;
    BOOL shade;
    unsigned long count;

    if (!msgmap_update(zm))
        return (NIL);

    if (request->argc == 3) {
        unsigned long msgno = atoi(request->argv[1]);
        unsigned long msguid = atoi(request->argv[2]);

        if ((msgno == 0) || (msgno > zm->nmsgs) ||
            (!(msgno = stream_check_uid(session, stream, msgno, msguid))))
            msgno = zm->nmsgs;

        session->current = msgno;
    }

    /* Expunge event => session->current may no longer be valid? */
    if (session->current > zm->nmsgs)
        session->current = zm->nmsgs;

    zm_offset = msgmap_find(zm, session->current);
    count = msgmap_size(session->zm);

    if (zm_offset > prefs->msgs_per_page)
        first = zm_offset - ((zm_offset - 1) % prefs->msgs_per_page);
    else
        first = 1;

    last = (((first + prefs->msgs_per_page - 1) < count)
            ? (first + prefs->msgs_per_page - 1) : count);

    /* Make new current message midpoint in the range */
    if (last >= first)
        session->current = msgmap_value(zm, (first + last) / 2);

    if (!stream_fill_overview_msgmap(session, request->pool, stream,
                                     session->zm, first, last))
        return (NIL);

    alt_addr = cmd_list_alt_addr_generate(session, request->pool, prefs);

    /* Toolbar */
    html_start(session, b, "Mailbox listing");
    html_banner_toolbar(session, b, "list", HTML_BANNER_CALLBACK_SAVE);

    if (prefs->use_mark_persist)
        html_form_start(session, b, "POST", "aggregate");
    else
        html_form_start(session, b, "POST", "aggregate_tmp");

    /* Generate action line before folder listing */
    cmd_list_toolbar_hdr(session, b, zm_offset);

    if (count > 0) {
        /* Generate message list */
        bputs(b, "<table width=\"100%\">" CRLF);

        /* Column headings */
        cmd_list_headings(session, b);

        shade = T;
        for (current = first; current <= last; current++) {
            if (!cmd_list_msg(session, b, stream,
                              msgmap_value(zm, current),
                              msgmap_depth(zm, current), alt_addr, shade))
                return (NIL);
            shade = (shade) ? NIL : T;
        }

#if 0                           /* Just an experiment really */
        for (last = first + prefs->msgs_per_page - 1; current <= last;
             current++) {
            cmd_list_empty(session, b, current, shade);
            shade = (shade) ? NIL : T;
        }
#endif

        bputs(b, "</table>" CRLF);

        /* Generate action line after folder listing */
        cmd_list_toolbar_footer(session, b, zm_offset);
    }

    html_form_end(b);
    html_end(b);

    /* Finished with alt_addr */
    cmd_list_alt_addr_free(alt_addr);
    return (T);
}

/* ====================================================================== */

void cmd_list(struct session *session)
{
    struct options *options = session->options;
    struct msgmap *zm = session->zm;
    struct request *request = session->request;
    char *cmd, *folder;

    /* Handle POST requests: folder change requests */
    if (request->method == POST) {
        request_decode_post(request);

        if (assoc_lookup(request->form, "sub_folder_dialogue") &&
            (folder = assoc_lookup(request->form, "folder")) && folder[0])
        {

            if ((cmd = assoc_lookup(request->form, "command")) &&
                !strcmp(cmd, "copy")) {
                session->aggregate = T;
                session_redirect(session, request,
                                 pool_strcat(request->pool, "copy_msg/",
                                             folder));
            } else
                session_redirect(session, request,
                                 pool_strcat(request->pool, "change/",
                                             folder));
            return;
        }
    }

    /* Save options if anything changed */
    if (options->save)
        session_streams_save_options(session);

    /* Record last command for compose/cancel and friends */
    session->compose_parent_cmd = "list";

    /* Record last command for copy/cancel */
    session->copy_parent_cmd = "list";

    if (msgmap_have_zoom(zm) && (msgmap_marked_count(zm) == 0)) {
        msgmap_unmark_all(zm);
        msgmap_disable_zoom(zm);
        session_message(session,
                        "No messages marked: disabling zoom mode");
    }

    if (!cmd_list_work(session)) {
        session_redirect(session, request, "restart");
        return;
    }

    /* Send out HTML */
    response_html(request, 200);
}
