/* $Cambridge: hermes/src/prayer/prayer/cmd_prefs.c,v 1.3 2003/10/23 15:14:17 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* ====================================================================== */

static void
single_option(struct buffer *b, char *target, char *text, BOOL selected)
{
    if (selected) {
        bprintf(b, "<option value=\"%s\" selected>%s</option>" CRLF,
                target, text);
    } else {
        bprintf(b, "<option value=\"%s\">%s</option>" CRLF, target, text);
    }
}

/* General options */

static void
generate_general_form(struct session *session, struct prefs *prefs)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* General options */
    html_form_hidden(b, "type", "general");
    bprintf(b, "<h3>General Preferences:</h3>" CRLF);

    /* Main theme */
    if (config->theme_list && (list_length(config->theme_list) > 1)) {
        struct config_theme *theme;

        bputs(b, "<table>" CRLF);
        bputs(b, "<tr>" CRLF);
        bputs(b, "<td>Main theme:</td>" CRLF);
        bputs(b, "<td><select name=\"theme_main_name\" size=1>" CRLF);

        theme = (struct config_theme *) config->theme_list->head;
        while (theme) {
            if (!strcmp(prefs->theme_main_name, theme->name))
                bprintf(b, "<option value=\"%s\" selected>" CRLF,
                        theme->name);
            else
                bprintf(b, "<option value=\"%s\">" CRLF, theme->name);

            html_quote_string(b, theme->description);
            bputs(b, "</option>" CRLF);

            theme = theme->next;
        }
        bputs(b, "</select></td>" CRLF);
        bprintf(b, "</tr>" CRLF);

        bputs(b, "<tr>" CRLF);
        bputs(b, "<td>Help theme:</td>" CRLF);
        bputs(b, "<td><select name=\"theme_help_name\" size=1>" CRLF);

        theme = (struct config_theme *) config->theme_list->head;
        while (theme) {
            if (!strcmp(prefs->theme_help_name, theme->name))
                bprintf(b, "<option value=\"%s\" selected>" CRLF,
                        theme->name);
            else
                bprintf(b, "<option value=\"%s\">" CRLF, theme->name);

            html_quote_string(b, theme->description);
            bputs(b, "</option>" CRLF);

            theme = theme->next;
        }
        bputs(b, "</select></td>" CRLF);
        bputs(b, "</tr>" CRLF);
        bputs(b, "</table>" CRLF);
    }

    /* Start of use_welcome */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Enable welcome screen",
                             "use_welcome", prefs->use_welcome);
    bprintf(b, "</tr>" CRLF);

    /* Start of confirm_logout */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Confirm logout",
                             "confirm_logout", prefs->confirm_logout);
    bputs(b, "</tr>" CRLF);

    /* Start of use_mark_persist */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             "Use persistent marks (slower, sometimes useful)",
                             "use_mark_persist", prefs->use_mark_persist);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_search_zoom */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, ("&nbsp;&nbsp;&nbsp;- "
                                 "Zoom automatically after search"),
                             "use_search_zoom", prefs->use_search_zoom);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_agg_unmark */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, ("&nbsp;&nbsp;&nbsp;- "
                                 "Unmark messages after aggregate operation"),
                             "use_agg_unmark", prefs->use_agg_unmark);
    bprintf(b, "</tr>" CRLF);

    bputs(b, "</table>" CRLF);
}

static BOOL
process_general_prefs(struct session *session,
                      struct prefs *prefs, struct assoc *h)
{
    char *s;

    if ((s = assoc_lookup(h, "theme_main_name"))) {
        prefs->theme_main_name = pool_strdup(prefs->pool, s);
    }

    if ((s = assoc_lookup(h, "theme_help_name"))) {
        prefs->theme_help_name = pool_strdup(prefs->pool, s);
    }

    prefs->use_welcome = (assoc_lookup(h, "use_welcome")) ? T : NIL;
    prefs->use_mark_persist =
        (assoc_lookup(h, "use_mark_persist")) ? T : NIL;
    prefs->use_search_zoom =
        (assoc_lookup(h, "use_search_zoom")) ? T : NIL;
    prefs->use_agg_unmark = (assoc_lookup(h, "use_agg_unmark")) ? T : NIL;
    prefs->confirm_logout = (assoc_lookup(h, "confirm_logout")) ? T : NIL;

    return (T);
}

/* ====================================================================== */

/* Display Preferences. "display1" avoid conflict with master toolbar */

static void
generate_display1_form(struct session *session, struct prefs *prefs)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    int sort_mode = prefs->sort_mode;
    BOOL sort_reverse = prefs->sort_reverse;

    /* Start of folder listing preferencs */
    html_form_hidden(b, "type", "display1");
    bprintf(b, "<h3>Display Preferences:</h3>" CRLF);

    bprintf(b, "<table>" CRLF);
    /* Start of use_icons */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Use icons", "use_icons",
                             prefs->use_icons);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_tail_banner */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Duplicate icons below message",
                             "use_tail_banner", prefs->use_tail_banner);
    bprintf(b, "</tr>" CRLF);

    /* Start of html_inline */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Display text/html sections inline. ",
                             "html_inline", prefs->html_inline);
    bprintf(b, "</tr>" CRLF);

    /* Start of html_inline_auto */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             ("Display text/* sections starting "
                              "&lt;html&gt; inline"),
                             "html_inline_auto", prefs->html_inline_auto);
    bprintf(b, "</tr>" CRLF);

    /* Start of preserve_mimetype */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             ("Preserve MIME type when downloading"
                              " attachments"),
                             "preserve_mimetype",
                             prefs->preserve_mimetype);
    bputs(b, "</tr>" CRLF);
    bprintf(b, "</table>" CRLF);

    /* Start of sort stuff */
    bprintf(b, "<table>" CRLF);

    /* Start of sort mode */
    bputs(b, "<tr><td>Default folder sort mode:</td>" CRLF);
    bputs(b, "<td><select name=\"sort_combined\" size=1>" CRLF);

    single_option(b, "arrival", "Arrival (Fastest)",
                  ((sort_mode == ARRIVAL) && (!sort_reverse)));

    single_option(b, "reverse-arrival", "Reverse Arrival",
                  ((sort_mode == ARRIVAL) && (sort_reverse)));

    single_option(b, "date", "Date",
                  ((sort_mode == DATE) && (!sort_reverse)));

    single_option(b, "reverse-date", "Reverse Date",
                  ((sort_mode == DATE) && (sort_reverse)));

    single_option(b, "from", "From",
                  ((sort_mode == FROM) && (!sort_reverse)));

    single_option(b, "reverse-from", "Reverse From",
                  ((sort_mode == FROM) && (sort_reverse)));

    single_option(b, "to", "To", ((sort_mode == TO) && (!sort_reverse)));

    single_option(b, "reverse-to", "Reverse To",
                  ((sort_mode == TO) && (sort_reverse)));

    single_option(b, "cc", "Cc", ((sort_mode == CC) && (!sort_reverse)));

    single_option(b, "reverse-cc", "Reverse Cc",
                  ((sort_mode == CC) && (sort_reverse)));

    single_option(b, "size", "Message Size",
                  ((sort_mode == SIZE) && (!sort_reverse)));

    single_option(b, "reverse-size", "Reverse Message Size",
                  ((sort_mode == SIZE) && (sort_reverse)));

    single_option(b, "subject", "Subject",
                  ((sort_mode == SUBJECT) && (!sort_reverse)));

    single_option(b, "reverse-subject", "Reverse Subject",
                  ((sort_mode == SUBJECT) && (sort_reverse)));

    single_option(b, "references", "Thread (References)",
                  ((sort_mode == REFERENCES) && (!sort_reverse)));

    single_option(b, "reverse-references", "Reverse Thread (References)",
                  ((sort_mode == REFERENCES) && (sort_reverse)));

    single_option(b, "orderedsubject", "Thread (Ordered Subject)",
                  ((sort_mode == REFERENCES) && (!sort_reverse)));

    single_option(b, "reverse-orderedsubject",
                  "Reverse Thread (Ordered Subject)",
                  ((sort_mode == REFERENCES) && (sort_reverse)));

    bputs(b, "</select></td>" CRLF);
    bputs(b, "</tr>" CRLF);

    /* Msgs per page */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b,
                          "Messages per page:", "msgs_per_page",
                          string_itoa(request->pool, prefs->msgs_per_page),
                          5);
    bprintf(b, "</tr>" CRLF);

    /* Abook per page */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b,
                          "Addressbook entries per page:",
                          "abook_per_page", string_itoa(request->pool,
                                                        prefs->
                                                        abook_per_page),
                          5);
    bprintf(b, "</tr>" CRLF);


    /* Alt-addr */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Alt Addresses:", "alt_addr",
                          prefs->alt_addr, 40);
    bprintf(b, "</tr>" CRLF);

    bprintf(b, "</table>" CRLF);
}

static BOOL
process_display1_prefs(struct session *session,
                       struct prefs *prefs, struct assoc *h)
{
    struct config *config = session->config;
    struct pool *pool = prefs->pool;
    char *s;
    ADDRESS *addr = NIL;
    ADDRESS *a = NIL;
    BOOL rc = T;

    if ((s = assoc_lookup(h, "msgs_per_page"))) {
        int value = atoi(s);

        if (value < config->msgs_per_page_min) {
            session_message(session,
                            "Messages per page too small (minimum allowed: %lu)",
                            config->msgs_per_page_min);
            return (NIL);
        }

        if (value > config->msgs_per_page_max) {
            session_message(session,
                            "Messages per page too large (maximum allowed: %lu)",
                            config->msgs_per_page_max);
            return (NIL);
        }

        prefs->msgs_per_page = value;
    }

    if ((s = assoc_lookup(h, "abook_per_page"))) {
        int value = atoi(s);

        if (value < config->abook_per_page_min) {
            session_message(session,
                            ("Addressbook entries per page too small "
                             "(minimum allowed: %lu)"),
                            config->abook_per_page_min);
            return (NIL);
        }
        if (value > config->abook_per_page_max) {
            session_message(session,
                            ("Addressbook entries per page too large "
                             "(maximum allowed: %lu)"),
                            config->abook_per_page_max);
            return (NIL);
        }

        prefs->abook_per_page = value;
    }

    if ((s = assoc_lookup(h, "alt_addr"))) {
        prefs->alt_addr = pool_strdup(pool, string_trim_whitespace(s));

        if (prefs->alt_addr && prefs->alt_addr[0]) {
            if (!(addr=addr_parse_destructive(s, ""))) {
                session_message(session,
                                "Alt addresses invalid: %s", ml_errmsg());
                rc = NIL;
            } else {
                for (a = addr; a; a = a->next) {
                    if (!(a->host && a->host[0])) {
                        session_message(session,
                                        ("Unqualified address (%s) "
                                         "in Alt Address list"), a->mailbox);
                        rc = NIL;
                        break;
                    }
                }
                mail_free_address(&addr);
            }
        }
    }

    prefs->use_icons = (assoc_lookup(h, "use_icons")) ? T : NIL;
    prefs->use_tail_banner =
        (assoc_lookup(h, "use_tail_banner")) ? T : NIL;
    prefs->html_inline = (assoc_lookup(h, "html_inline")) ? T : NIL;
    prefs->html_inline_auto =
        (assoc_lookup(h, "html_inline_auto")) ? T : NIL;
    prefs->preserve_mimetype =
        (assoc_lookup(h, "preserve_mimetype")) ? T : NIL;

    if ((s = assoc_lookup(h, "sort_combined"))) {
        if (!strncmp(s, "reverse-", strlen("reverse-"))) {
            prefs->sort_reverse = T;
            s += strlen("reverse-");
        } else
            prefs->sort_reverse = NIL;

        if (!strcmp(s, "arrival"))
            prefs->sort_mode = ARRIVAL;
        else if (!strcmp(s, "date"))
            prefs->sort_mode = DATE;
        else if (!strcmp(s, "from"))
            prefs->sort_mode = FROM;
        else if (!strcmp(s, "to"))
            prefs->sort_mode = TO;
        else if (!strcmp(s, "cc"))
            prefs->sort_mode = CC;
        else if (!strcmp(s, "size"))
            prefs->sort_mode = SIZE;
        else if (!strcmp(s, "subject"))
            prefs->sort_mode = SUBJECT;
        else if (!strcmp(s, "references"))
            prefs->sort_mode = REFERENCES;
        else if (!strcmp(s, "orderedsubject"))
            prefs->sort_mode = ORDEREDSUBJECT;
    }
    return (rc);
}

/* ====================================================================== */

/* Folder Preferences */

static void
generate_folder_form(struct session *session, struct prefs *prefs)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Start of folder listing preferencs */
    html_form_hidden(b, "type", "folder");
    bprintf(b, "<h3>Folder Listing Preferences:</h3>" CRLF);
    bprintf(b, "<table>" CRLF);

    /* Start of suppress_dotfiles */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, ("Suppress dotfiles (files starting '.')"
                                 " from folder listing"),
                             "suppress_dotfiles",
                             prefs->suppress_dotfiles);
    bprintf(b, "</tr>" CRLF);

    /* Start of confirm_rm */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Confirm folder deletion",
                             "confirm_rm", prefs->confirm_rm);
    bprintf(b, "</tr>" CRLF);

    bprintf(b, "</table>" CRLF);
    bprintf(b, "<table>" CRLF);

    /* Maildir */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Mail directory:", "maildir", prefs->maildir,
                          20);
    bprintf(b, "</tr>" CRLF);

    /* sent_mail_folder */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Sent Mail Folder:",
                          "sent_mail_folder", prefs->sent_mail_folder, 20);
    bprintf(b, "</tr>" CRLF);

    /* postponed_folder */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Postponed Messages Folder:",
                          "postponed_folder", prefs->postponed_folder, 20);
    bprintf(b, "</tr>" CRLF);

    bprintf(b, "</table>" CRLF);
}

static BOOL
process_folder_prefs(struct session *session,
                     struct prefs *prefs, struct assoc *h)
{
    struct pool *pool = prefs->pool;
    char *s;
    BOOL old_dotfiles = prefs->suppress_dotfiles;

    prefs->suppress_dotfiles =
        (assoc_lookup(h, "suppress_dotfiles")) ? T : NIL;

    if (prefs->suppress_dotfiles != old_dotfiles)
        dircache_invalidate(session, "");

    if ((s = assoc_lookup(h, "maildir"))) {
        s = string_trim_whitespace(s);

        if (strcmp(s, prefs->maildir) != 0)
            prefs->maildir = pool_strdup(pool, s);

        if (!string_filename_valid(s)) {
            session_message(session, "Invalid Mail Directory Name");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "sent_mail_folder"))) {
        s = string_trim_whitespace(s);

        if (strcmp(s, prefs->sent_mail_folder) != 0)
            prefs->sent_mail_folder = pool_strdup(pool, s);

        if (*s == '\0') {
            session_message(session,
                            "Invalid (Empty) Sent Mail Folder Name");
            return (NIL);
        }

        if (!string_filename_valid(s)) {
            session_message(session, "Invalid Sent Mail Folder Name");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "postponed_folder"))) {
        s = string_trim_whitespace(s);

        if (strcmp(s, prefs->postponed_folder) != 0)
            prefs->postponed_folder = pool_strdup(pool, s);

        if (*s == '\0') {
            session_message(session,
                            "Invalid (Empty) Postponed Folder Name");
            return (NIL);
        }

        if (!string_filename_valid(s)) {
            session_message(session, "Invalid Postponed Folder Name");
            return (NIL);
        }
    }

    prefs->confirm_rm = (assoc_lookup(h, "confirm_rm")) ? T : NIL;

    /* Check if maildir included in postponed_folder or sent_mail_folder */
    if (prefs->maildir && prefs->maildir[0]) {
        unsigned long len = strlen(prefs->maildir);

        /* Check postponed_folder */
        if (prefs->postponed_folder && prefs->postponed_folder[0] &&
            !strncmp(prefs->postponed_folder, prefs->maildir, len) &&
            (prefs->postponed_folder[len] == '/'))
            prefs->postponed_folder += len + 1;

        /* Check sent_mail_folder */
        if (prefs->sent_mail_folder && prefs->sent_mail_folder[0] &&
            !strncmp(prefs->sent_mail_folder, prefs->maildir, len) &&
            (prefs->sent_mail_folder[len] == '/'))
            prefs->sent_mail_folder += len + 1;
    }

    return (T);
}

/* ====================================================================== */

/* Compose Preferences */

static void
generate_compose1_form(struct session *session, struct prefs *prefs)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Compose preferences */
    html_form_hidden(b, "type", "compose1");
    bprintf(b, "<h3>Compose Preferences:</h3>" CRLF);

    /* From Personal */
    bprintf(b, "<table>" CRLF);
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Personal Name:", "from_personal",
                          prefs->from_personal, 40);

    if (!(prefs->from_personal && prefs->from_personal[0]) &&
        config->local_domain_list) {
        struct list_item *li;
        char *value;
        char *default_domain = prefs->default_domain;

        bputs(b, "<td>Default: " CRLF);

        for (li = config->local_domain_list->head; li; li = li->next) {
            struct config_local_domain *cld =
                (struct config_local_domain *) li;

            if (cld->cdb_map && !strcmp(default_domain, cld->name) &&
                cdb_find(cld->cdb_map,
                         session->username, strlen(session->username),
                         &value)
                && value && value[0]
                && (value = string_trim_whitespace(value))) {
                html_quote_string(b, value);
                free(value);
                break;
            }
        }
        if (li == NIL)          /* Not found */
            html_quote_string(b, "<None>");

        bputs(b, "</td>" CRLF);
    }
    bputs(b, "</tr>" CRLF);

    /* From address */
    if (!config->fix_from_address) {
        bputs(b, "<tr>" CRLF);
        html_form_table_field(b, "From Address:", "from_address",
                              prefs->from_address, 40);

        if (!(prefs->from_address && prefs->from_address[0])) {
            bputs(b, "<td>Default: " CRLF);
            bprintf(b, "%s@%s</td>" CRLF,
                    session->username, prefs->default_domain);
        }
        bprintf(b, "</tr>" CRLF);
    }

    /* Reply-To */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Default Reply-To:", "default_reply_to",
                          prefs->default_reply_to, 40);
    bprintf(b, "</tr>" CRLF);
    bprintf(b, "</table>" CRLF);

    /* Start of signature */
    bprintf(b, "<h3>Signature:</h3>" CRLF);
    html_form_textarea(session, b, "signature", prefs->signature, 75, 4);
}

static BOOL
process_compose_prefs(struct session *session,
                      struct prefs *prefs, struct assoc *h)
{
    struct config *config = session->config;
    struct pool *pool = prefs->pool;
    char *s;
    ADDRESS *a = NIL;
    ADDRESS *addr = NIL;
    BOOL rc = T;

    if ((s = assoc_lookup(h, "from_personal")))
        prefs->from_personal =
            pool_strdup(pool, string_trim_whitespace(s));

    if (!config->fix_from_address && (s = assoc_lookup(h, "from_address"))) {
        prefs->from_address = pool_strdup(pool, string_trim_whitespace(s));
        if (prefs->from_address[0]) {
            if (!(addr=addr_parse_destructive(s, ""))) {
                session_message(session,
                                "From Address invalid: %s", ml_errmsg());
                rc = NIL;
            } else {
                if ((addr->next) || (addr->personal && addr->personal[0]) ||
                    (!(addr->host && addr->host[0]))) {
                    session_message(session,
                                    ("From Address must be single, simple "
                                     "and fully qualified email address"));
                    rc = NIL;
                }
                mail_free_address(&addr);
            }
        }
    }

    if ((s = assoc_lookup(h, "default_reply_to"))) {
        prefs->default_reply_to =
            pool_strdup(pool, string_trim_whitespace(s));

        if (prefs->default_reply_to[0]) {
            if (!(addr = addr_parse_destructive(s, ""))) {
                session_message(session,
                                "Default Reply-To Address invalid: %s",
                                ml_errmsg());
                rc = NIL;
            } else {
                for (a = addr; a; a = a->next) {
                    if (!(a->host && a->host[0])) {
                        session_message(session, ("Unqualified address in "
                                                  "Default Reply-To list"));
                        rc = NIL;
                        break;
                    }
                }
                mail_free_address(&addr);
            }
        }
    }

    if ((s = assoc_lookup(h, "signature")))
        prefs->signature = pool_strdup(pool, s);

    return (rc);
}

/* ====================================================================== */

/* Compose2 Preferences */

static void
generate_compose2_form(struct session *session, struct prefs *prefs)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Compose preferences */
    html_form_hidden(b, "type", "compose2");
    bprintf(b, "<h3>Extra Compose Preferences:</h3>" CRLF);

    /* Start of use_sent_mail */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Use sent-mail folder",
                             "use_sent_mail", prefs->use_sent_mail);
    bputs(b, "</tr>" CRLF);

    /* Start of line_wrap_on_reply */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Line wrap on reply",
                             "line_wrap_on_reply",
                             prefs->line_wrap_on_reply);
    bputs(b, "</tr>" CRLF);

    /* Start of line_wrap_on_spell */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Line wrap on spell check",
                             "line_wrap_on_spell",
                             prefs->line_wrap_on_spell);
    bputs(b, "</tr>" CRLF);

    /* Start of line_wrap_on_send */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Line wrap on send",
                             "line_wrap_on_send",
                             prefs->line_wrap_on_send);
    bputs(b, "</tr>" CRLF);

    /* Start of line_wrap_advanced */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Enable manual line wrap options",
                             "line_wrap_advanced",
                             prefs->line_wrap_advanced);
    bputs(b, "</tr></table>" CRLF);

    /* Start of line_wrap_len */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr><td>Line Wrap at:</td><td>");
    bprintf(b, "<input name=\"line_wrap_len\" value=\"%lu\" size=\"3\">",
            prefs->line_wrap_len);
    bputs(b, "columns</td></tr>" CRLF);

    /* Start of Normal window size */
    bputs(b, "<tr><td>Normal Compose window size:</td>" CRLF);
    bprintf(b,
            ("<td><input name=\"small_cols\" value=\"%lu\" size=\"3\">"
             "columns</td>" CRLF), prefs->small_cols);
    bprintf(b, ("<td><input name=\"small_rows\" value=\"%lu\""
                " size=\"3\"> rows" CRLF), prefs->small_rows);
    bputs(b, "</td></tr>" CRLF);

    /* Start of Large window size */
    bputs(b, "<tr><td>Large Compose window size</td>");
    bprintf(b, ("<td><input name=\"large_cols\" value=\"%lu\" size=\"3\">"
                "columns</td>" CRLF), prefs->large_cols);
    bprintf(b,
            "<td><input name=\"large_rows\" value=\"%lu\" size=\"3\">rows"
            CRLF, prefs->large_rows);
    bputs(b, "</td></tr>" CRLF);

    /* Start of default domain */
    if (config->local_domain_list &&
        (list_length(config->local_domain_list) > 1)) {
        struct list_item *li;

        bputs(b, "<tr>" CRLF);
        bputs(b, "<td>Default domain:</td>" CRLF);
        bputs(b, "<td><select name=\"default_domain\" size=1>" CRLF);

        for (li = config->local_domain_list->head; li; li = li->next) {
            struct config_local_domain *cld =
                (struct config_local_domain *) li;

            if (!strcmp(prefs->default_domain, cld->name))
                bprintf(b,
                        "<option value=\"%s\" selected>%s</option>" CRLF,
                        cld->name, cld->name);
            else
                bprintf(b, "<option value=\"%s\">%s</option>" CRLF,
                        cld->name, cld->name);
        }
        bputs(b, "</select></td>" CRLF);
        bprintf(b, "</tr>" CRLF);
    }

    if (config->ispell_language_list) {
        struct list_item *li;

        bputs(b, "<tr>" CRLF);
        bputs(b, "<td>Spell Check Language:</td>" CRLF);
        bputs(b, "<td><select name=\"ispell_language\" size=1>" CRLF);

        for (li = config->ispell_language_list->head; li; li = li->next) {
            struct config_language *lang = (struct config_language *) li;

            if (!strcmp(prefs->ispell_language, lang->name))
                bprintf(b, "<option value=\"%s\" selected>" CRLF,
                        lang->name);
            else
                bprintf(b, "<option value=\"%s\">" CRLF, lang->name);

            html_quote_string(b, lang->desc);
            bputs(b, "</option>" CRLF);
        }
        bputs(b, "</select></td>" CRLF);
        bprintf(b, "</tr>" CRLF);
    }

    bprintf(b, "</table>" CRLF);
}

/* ====================================================================== */

static BOOL
process_compose2_prefs(struct session *session,
                       struct prefs *prefs, struct assoc *h)
{
    struct pool *pool = prefs->pool;
    char *s;
    int value;

    if ((s = assoc_lookup(h, "small_cols"))) {
        if (((value = atoi(s)) > 0) && (value <= 120)) {
            prefs->small_cols = value;
        } else {
            session_message(session, "Small columns option out of range");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "small_rows"))) {
        if (((value = atoi(s)) > 0) && (value <= 120)) {
            prefs->small_rows = value;
        } else {
            session_message(session, "Small rows option out of range");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "large_cols"))) {
        if (((value = atoi(s)) > 0) && (value <= 120)) {
            prefs->large_cols = value;
        } else {
            session_message(session, "Large columns option out of range");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "large_rows"))) {
        if (((value = atoi(s)) > 0) && (value <= 120)) {
            prefs->large_rows = value;
        } else {
            session_message(session, "Large rows option out of range");
            return (NIL);
        }
    }

    if ((s = assoc_lookup(h, "line_wrap_len"))) {
        if (((value = atoi(s)) > 40) && (value <= 120)) {
            prefs->line_wrap_len = value;
        } else {
            session_message(session,
                            "Line wrap length option out of range");
            return (NIL);
        }
    }

    prefs->line_wrap_on_reply
        = (assoc_lookup(h, "line_wrap_on_reply")) ? T : NIL;

    prefs->line_wrap_on_send
        = (assoc_lookup(h, "line_wrap_on_send")) ? T : NIL;

    prefs->line_wrap_on_spell
        = (assoc_lookup(h, "line_wrap_on_spell")) ? T : NIL;

    prefs->line_wrap_advanced
        = (assoc_lookup(h, "line_wrap_advanced")) ? T : NIL;

    prefs->use_sent_mail = (assoc_lookup(h, "use_sent_mail")) ? T : NIL;

    if ((s = assoc_lookup(h, "default_domain")))
        prefs->default_domain =
            pool_strdup(pool, string_trim_whitespace(s));

    if ((s = assoc_lookup(h, "ispell_language")))
        prefs->ispell_language =
            pool_strdup(pool, string_trim_whitespace(s));

    return (T);
}

/* ====================================================================== */

/* Session Preferences */

static void
generate_session_form(struct session *session, struct prefs *prefs)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Compose preferences */
    html_form_hidden(b, "type", "session");
    bprintf(b, ("<h3>Session Preferences "
                "(actual behaviour depends on Web browser):</h3>" CRLF));
    bprintf(b, "<table>" CRLF);

    /* Start of use_cookie */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Allow HTTP Cookie for Session ID",
                             "use_cookie", prefs->use_cookie);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_substitution */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             ("Allow page substitution rather than HTTP redirect"
                              " (faster, less predicable URLs)"),
                             "use_substitution", prefs->use_substitution);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_http_1_1 */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Allow HTTP/1.1",
                             "use_http_1_1", prefs->use_http_1_1);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_pipelining */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Allow HTTP/1.1 pipelining",
                             "use_pipelining", prefs->use_pipelining);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_embed_http */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             "Allow HTTP links embedded in HTTPS connection",
                             "use_embed_http", prefs->use_embed_http);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_pipelining */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b,
                             ("Allow HTTP 1.1 and 1.0 persistent connections"
                              " (strongly recommended!)"),
                             "use_persist", prefs->use_persist);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_short */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Allow Short URLs",
                             "use_short", prefs->use_short);
    bprintf(b, "</tr>" CRLF);

    /* Start of use_gzip */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Allow GZIP compresssion",
                             "use_gzip", prefs->use_gzip);
    bprintf(b, "</tr>" CRLF);

    /* Start of is_netscape4 */
    bprintf(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Is Netscape 4",
                             "is_netscape4", prefs->is_netscape4);
    bprintf(b, "</tr>" CRLF);

    bputs(b, "</table>" CRLF);
}


static BOOL
process_session_prefs(struct session *session,
                      struct prefs *prefs, struct assoc *h)
{
    prefs->use_cookie = (assoc_lookup(h, "use_cookie")) ? T : NIL;
    prefs->use_substitution =
        (assoc_lookup(h, "use_substitution")) ? T : NIL;
    prefs->use_http_1_1 = (assoc_lookup(h, "use_http_1_1")) ? T : NIL;
    prefs->use_pipelining = (assoc_lookup(h, "use_pipelining")) ? T : NIL;
    prefs->use_embed_http = (assoc_lookup(h, "use_embed_http")) ? T : NIL;
    prefs->use_persist = (assoc_lookup(h, "use_persist")) ? T : NIL;
    prefs->use_short = (assoc_lookup(h, "use_short")) ? T : NIL;
    prefs->use_gzip = (assoc_lookup(h, "use_gzip")) ? T : NIL;
    prefs->is_netscape4 = (assoc_lookup(h, "is_netscape4")) ? T : NIL;

    return (T);
}

/* ====================================================================== */

static void
generate_form(struct session *session, char *type, struct prefs *prefs)
{
    struct user_agent *user_agent = session->user_agent;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "User Preferences");

    /* Form body */
    html_form_start(session, b, "POST", "prefs");

    /* Toolbar */
    html_banner_toolbar(session, b, "prefs", HTML_BANNER_CALLBACK_ALL);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Preferences</h2>" CRLF);

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_general", "General");
    html_form_table_submit(b, "sub_display1", "Display");
    html_form_table_submit(b, "sub_compose1", "Compose");
    html_form_table_submit(b, "sub_compose2", "Extra Compose");
    html_form_table_submit(b, "sub_folder", "Folder");
    if (user_agent->use_debug)
        html_form_table_submit(b, "sub_session", "Session");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    if (!strcmp(type, "general"))
        generate_general_form(session, prefs);
    else if (!strcmp(type, "display1"))
        generate_display1_form(session, prefs);
    else if (!strcmp(type, "folder"))
        generate_folder_form(session, prefs);
    else if (!strcmp(type, "compose1"))
        generate_compose1_form(session, prefs);
    else if (!strcmp(type, "compose2"))
        generate_compose2_form(session, prefs);
    else if (!strcmp(type, "session"))
        generate_session_form(session, prefs);

    /* Submit, cancel body */
    bprintf(b, "<br>" CRLF);
    bprintf(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_change", "Change for this session");
    html_form_table_submit(b, "sub_save",
                           "Change for this and future sessions");
    bprintf(b, "</tr></table>" CRLF);

    /* End of form */
    html_form_end(b);
    html_end(b);

    response_html(request, 200);
}

/* ====================================================================== */

void cmd_prefs(struct session *session)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct prefs *prefs;
    struct pool *pool;
    struct assoc *h = NIL;
    char *s;
    char *type;

    if (!options->prefs_work)
        options->prefs_work = prefs_copy(options->prefs);

    prefs = options->prefs_work;
    pool = prefs->pool;

    if (request->method != POST) {
        generate_form(session, "general", prefs);
        session_log(session, "[cmd_prefs] General Preferences");
        return;
    }

    request_decode_post(request);
    h = request->form;

    /* Cancel doesn't need any preferences */
    if ((s = assoc_lookup(h, "sub_cancel"))) {
        /* Free up working preferences and return to list screen */
        prefs_free(options->prefs_work);
        options->prefs_work = NIL;

        session_redirect(session, request, "manage");
        return;
    }

    /* Make sure that working preferences updated before we go anywhere */
    if (!(type = assoc_lookup(h, "type")))
        type = "";

    if (!strcmp(type, "general")) {
        session_log(session, "[cmd_prefs] General Preferences");
        if (!process_general_prefs(session, prefs, h)) {
            generate_form(session, "general", prefs);
            return;
        }
    } else if (!strcmp(type, "display1")) {
        session_log(session, "[cmd_prefs] Display Preferences");
        if (!process_display1_prefs(session, prefs, h)) {
            generate_form(session, "display1", prefs);
            return;
        }
    } else if (!strcmp(type, "folder")) {
        session_log(session, "[cmd_prefs] Folder Preferences");
        if (!process_folder_prefs(session, prefs, h)) {
            generate_form(session, "folder", prefs);
            return;
        }
    } else if (!strcmp(type, "compose1")) {
        session_log(session, "[cmd_prefs] Compose Preferences");
        if (!process_compose_prefs(session, prefs, h)) {
            generate_form(session, "compose1", prefs);
            return;
        }
    } else if (!strcmp(type, "compose2")) {
        session_log(session, "[cmd_prefs] Extra Compose Preferences");
        if (!process_compose2_prefs(session, prefs, h)) {
            generate_form(session, "compose2", prefs);
            return;
        }
    } else if (!strcmp(type, "session")) {
        session_log(session, "[cmd_prefs] Session Preferences");
        if (!process_session_prefs(session, prefs, h)) {
            generate_form(session, "session", prefs);
            return;
        }
    }

    /* Toolbar actions */
    if ((s = assoc_lookup(h, "sub_help"))) {
        if (type)
            session_redirect(session, request,
                             pool_strcat(request->pool, "help/prefs/",
                                         type));
        else
            session_redirect(session, request, "help/prefs");
        return;
    }

    if (assoc_lookup(h, "sub_folder_dialogue")) {
        char *name = assoc_lookup(request->form, "folder");

        if (name && name[0]) {
            session_redirect(session, request,
                             pool_strcat(request->pool, "change/", name));
            return;
        } else
            session_message(session, "Invalid mail folder");
    }

    if (assoc_lookup(h, "sub_list")) {
        session_redirect(session, request, "list");
        return;
    }

    if (assoc_lookup(h, "sub_display")) {
        session_redirect(session, request, "display");
        return;
    }

    if (assoc_lookup(h, "sub_compose")) {
        session->compose_parent_cmd = "prefs";
        session_redirect(session, request, "compose");
        return;
    }

    if (assoc_lookup(h, "sub_abook_list")) {
        session->abook_parent_cmd = "prefs";
        session_redirect(session, request, "abook_list");
        return;
    }

    if (assoc_lookup(h, "sub_folders")) {
        session_redirect(session, request, "folders");
        return;
    }

    if (assoc_lookup(h, "sub_manage")) {
        session_redirect(session, request, "manage");
        return;
    }

    if (assoc_lookup(h, "sub_logout")) {
        session_redirect(session, request, "logout/prefs");
        return;
    }

    /* Switch to other preference screens */
    if (assoc_lookup(h, "sub_general")) {
        generate_form(session, "general", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_display1")) {
        generate_form(session, "display1", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_folder")) {
        generate_form(session, "folder", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_compose1")) {
        generate_form(session, "compose1", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_compose2")) {
        generate_form(session, "compose2", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_session")) {
        generate_form(session, "session", prefs);
        return;
    }

    if (assoc_lookup(h, "sub_save")) {
        /* Update save preferences */
        if (options->prefs_save)
            prefs_free(options->prefs_save);
        options->prefs_save = prefs_copy(options->prefs_work);

        options->save = T;
    }

    /* Update live preferences */
    if (options->prefs)
        prefs_free(options->prefs);
    options->prefs = prefs_copy(options->prefs_work);
    session_use_prefs(session, prefs, NIL);

    /* Free up working preferences and return to list screen */
    prefs_free(options->prefs_work);
    options->prefs_work = NIL;

    session_redirect(session, request, "manage");
}
