/* $Cambridge: hermes/src/prayer/prayer/cmd_quota.c,v 1.6 2004/10/28 13:48:37 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* ====================================================================== */

/* Slightly naughty interface to undocumented QUOTAROOT support in c-client */

#include "imap4r1.h"         

static unsigned long my_quota_active  = NIL;
static unsigned long my_quota_current = 0;
static unsigned long my_quota_limit   = 0;

static void
my_quotalist_callback(MAILSTREAM *stream, char *qroot, QUOTALIST *qlist)
{
    while (qlist) {
        if (qlist->name && !strcmp(qlist->name, "STORAGE")) {
            my_quota_current = qlist->usage;
            my_quota_limit   = qlist->limit;
            my_quota_active  = T;
        }
        qlist = qlist->next;
    }
}

static BOOL
mailstore_quotacheck(struct session *session,
                     unsigned long *currentp, unsigned long *limitp)
{
    void *old_callback;

    my_quota_active  = NIL;
    my_quota_current = 0;
    my_quota_limit   = 0;
    *currentp = 0;
    *limitp   = 0;

    old_callback = mail_parameters (session->inbox_stream, GET_QUOTA, NIL);
    mail_parameters (session->inbox_stream, SET_QUOTA, my_quotalist_callback);
    if (!imap_getquotaroot(session->inbox_stream, "INBOX")) {
        return(NIL);
    }
    mail_parameters (session->inbox_stream, SET_QUOTA, old_callback);

    if (!my_quota_active)
        return(NIL);

    *currentp = my_quota_current;
    *limitp   = my_quota_limit;
    return(T);
}

/* ====================================================================== */

static void
html_quota_summary(struct buffer *b,
                   char *col0, char *col1, char *col2, char *col3)
{
    bputs(b, "<tr align=\"right\">" CRLF);
    bprintf(b, "<td align=\"left\">%s</td>" CRLF, col0);
    bprintf(b, "<td>%s</td>" CRLF, col1);
    bprintf(b, "<td>%s</td>" CRLF, col2);
    bprintf(b, "<td>%s</td>" CRLF, col3);
    bputs(b, "</tr>" CRLF);
}

static void
html_quota_mbytes(struct buffer *b, unsigned long kbytes)
{
    unsigned long whole    = kbytes / 1024;
    unsigned long fraction = ((kbytes % 1024) * 100) / 1024;

    if (fraction > 9)
        bprintf(b, "<td>%lu.%lu</td>"CRLF, whole, fraction);
    else
        bprintf(b, "<td>%lu.0%lu</td>"CRLF, whole, fraction);
}

static void
html_quota_percent(struct buffer *b,
                   unsigned long used, unsigned long limit)
{
    char buf[10], *s;
    int  i, digit;

    if (used >= limit) {
        bputs(b, "<td>100.00%</td>");
        return;
    }

    /* Calculate first 4 places of percentage (may have leading zeros) */
    s = buf;
    for (i = 0 ; i < 4 ; i++) {
        used  *= 10;
        digit  = used/limit;
        used  -= (digit*limit);

        *s++ = '0' + digit;
        if (i == 1) *s++ = '.';
    }
    *s = '\0';

    /* Trim leading zero in "0x.yz */
    bprintf(b, "<td>%s%%</td>"CRLF, (buf[0] == '0') ? buf+1 : buf);
}

static void
html_quota_line(struct buffer *b, char *desc,
                unsigned long used, unsigned long limit, int make_mbytes)
{
    bputs(b, "<tr align=\"right\">"CRLF);
    bprintf(b, "<td align=\"left\">%s</td>"CRLF, desc);

    if (make_mbytes) {
        html_quota_mbytes(b, used);
        html_quota_mbytes(b, limit);
    } else {
        bprintf(b, "<td>%lu</td>"CRLF, used);
        bprintf(b, "<td>%lu</td>"CRLF, limit);
    }
    html_quota_percent(b, used, limit);
}

void cmd_quota(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    unsigned long current, limit;
 
    if (request->method == POST) {
        session_redirect(session, request, "manage");
        return;
    }

    if (!account_check_quota(session->account, request->pool)) {
        /* session_message fallback only: should be redundant */
        session_message(session, "Unable to check quota");
    }

    /* Title bar */
    html_start(session, b, "Check Quota");
    html_banner_toolbar(session, b, "quota", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Disk Quota use</h2>" CRLF);

    bputs(b, "<table border=1>" CRLF);
    html_quota_summary(b, "", "Used", "Limit", "Fraction");
    html_quota_line(b, "File Quota", account->iused, account->ilimit, 0);

    html_quota_line(b, "Block Quota (MBytes)",
                    account->bused, account->blimit, 1);

    if (mailstore_quotacheck(session, &current, &limit))
        html_quota_line(b, "Mailstore Quota (MBytes)", current, limit, 1);

    bputs(b, "</table>" CRLF);

    html_form_start(session, b, "POST", "quota");
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_back", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);
    html_end(b);

    response_html(request, 200);        /* Success */
}
