/* $Cambridge: hermes/src/prayer/prayer/cmd_rename.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    if (session->cwd && session->cwd[0]) {
        html_session_bprintf(session, b, "<td><a href=\"", "\">",
                             "rename?cwd=%s",
                             string_canon_path_encode(pool, "", parent));

        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a></td></tr>" CRLF);
    } else {
        bputs(b, "<tr><td>");
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</td></tr>" CRLF);
    }
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "rename?cwd=%s", name);

    html_icon(session, b, "right.gif", "[directory]");
    bprintf(b, "</a></td>" CRLF);
    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "rename?cwd=%s", name);
    html_quote_string(b, dir);
    bprintf(b, "</a></td></tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td><a>");
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);
    bputs(b, "<td><a>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</a></td></tr>" CRLF);
}

/* ====================================================================== */


static void
rename_html_folderlist(struct session *session,
                       struct pool *pool,
                       struct buffer *b, struct dirlist *dl)
{
    struct config_theme *theme = session->theme_main;
    BOOL shaded = NIL;

    /* Title Bar */
    html_start(session, b, "Rename mail folder");
    html_banner_toolbar(session, b, "rename", 0);

    /* Title line */
    if (session->rename_foldername)
        bprintf(b,
                "<h2 align=\"center\">Rename folder %s to ...</h2>" CRLF,
                session->rename_foldername);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "rename");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>" CRLF);
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    bputs(b, "</td></tr></table>" CRLF);
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table>" CRLF);

    /* Horizontal rule */
    bputs(b, "<hr>");
    html_form_start(session, b, "POST", "rename_item");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>" CRLF);
    html_form_field(b, "New name", "name", "", 16);
    html_form_submit(b, "submit", "Rename");
    html_form_submit(b, "cancel", "Cancel");
    bputs(b, "</td></tr></table>" CRLF);
    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

void cmd_rename(struct session *session)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != NIL))
            string_strdup(&session->dir_filter, s);
    }

    if (request->argc > 1) {
        string_strdup(&session->rename_foldername, request->argv[1]);
        string_canon_decode(session->rename_foldername);
    }

    rename_html_folderlist(session, pool, b,
                           dircache_fetch(session, session->cwd));

    response_html(request, 200);
}
