/* $Cambridge: hermes/src/prayer/prayer/cmd_reply.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void buffer_puts_mime_quote_html(struct buffer *b, char *s)
{
    unsigned long len = strlen(s) + 20;
    char *buffer = pool_alloc(b->pool, len);    /* Decoded form smaller */
    char *d;

    d = (char *) rfc1522_decode((unsigned char *) buffer, len, s, NIL);

    html_quote_string(b, d);
}

/* ====================================================================== */

void cmd_reply(struct session *session)
{
    struct request *request = session->request;
    struct draft *draft = session->draft;
    char *s, *opt_cc = NIL;
    MAILSTREAM *stream = session->stream;
    MESSAGECACHE *elt;
    ENVELOPE *env;
    ADDRESS *sender;
    struct buffer *b;
    unsigned long msgno, msguid;
    BOOL have_reply_flag = NIL;
    BOOL reply_all = NIL;

    if (request->argc >= 4) {
        msgno = atoi(request->argv[2]);
        msguid = atoi(request->argv[3]);
        session->current = msgno;

        have_reply_flag = T;
        if (!strcmp(request->argv[1], "all"))
            reply_all = T;
    } else if (request->argc == 3) {
        msgno = atoi(request->argv[1]);
        msguid = atoi(request->argv[2]);
        session->current = msgno;

    } else if (request->argc == 2) {
        msgno = session->current;
        msguid = ml_uid(session, stream, msgno);
        session->current = msgno;

        have_reply_flag = T;
        if (!strcmp(request->argv[1], "all"))
            reply_all = T;
    } else {
        msgno = session->current;
        msguid = ml_uid(session, stream, msgno);
    }

    if (!(msgno = stream_check_uid(session, stream, msgno, msguid))) {
        session_redirect(session, request, "list");
        return;
    }

    if (!((elt = ml_elt(session, stream, msgno)) &&
          (env = ml_fetch_structure(session, stream, msgno, NIL, 0)))) {
        session_redirect(session, request, "restart");
        return;
    }

    session->reply_all = NIL;   /* Clear Reply to all flag */
    draft_role_set(draft, NIL); /* Clear Role indication */

    sender = (env->reply_to) ? (env->reply_to) : (env->from);

    /* Calculate possible Cc value from message recipients */
    if (env->to) {
        opt_cc =
            addr_text_exclude(session, request->pool, env->to, sender);

        if (env->cc) {
            s = addr_text_exclude(session, request->pool, env->cc, sender);
            if (s && s[0]) {
                if (opt_cc && opt_cc[0])
                    opt_cc = pool_strcat3(request->pool, opt_cc, ", ", s);
                else
                    opt_cc = pool_strdup(request->pool, s);
            }
        }
    } else if (env->cc)
        opt_cc =
            addr_text_exclude(session, request->pool, env->cc, sender);

    if (have_reply_flag) {
        /* Don't need to ask user here */
        session->reply_all = reply_all;
        cmd_reply1(session);
        return;
    }

    if (!(opt_cc && opt_cc[0])) {
        /* Don't need to ask user here */
        cmd_reply1(session);
        return;
    }

    b = request->write_buffer;

    session_message(session, "Replying to message %lu", msgno);

    html_start(session, b, "Reply options");
    html_banner_toolbar(session, b, "reply", 0);

    bputs(b, "<h1 align=\"center\">Reply Options</h1>");

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr><td><strong>Sender:</strong></td><td>");
    buffer_puts_mime_quote_html(b, addr_text(request->pool, sender));
    bputs(b, "</td></tr>" CRLF);
    bputs(b, "<tr><td><strong>Recipients:</strong></td><td>");
    buffer_puts_mime_quote_html(b, opt_cc);
    bputs(b, "</td></tr>" CRLF);
    if (env->subject && env->subject[0]) {
        bputs(b, "<tr><td><strong>Subject:</strong></td><td>");
        buffer_puts_mime_quote_html(b, env->subject);
        bputs(b, "</td></tr>" CRLF);
    } else {
        bputs(b, ("<tr><td><strong>Subject:</strong></td>"
                  "<td>No subject</td></tr>" CRLF));
    }
    bputs(b, "</table><br>" CRLF);

    html_form_start(session, b, "POST", "reply1");
    html_form_submit(b, "cancel", "Cancel");
    html_form_submit(b, "single", "Reply to sender only");
    html_form_submit(b, "all", "Reply to sender and all recipients");
    html_form_end(b);
    html_end(b);
    /* Send out the response */
    response_html(request, 200);
}
