/* $Cambridge: hermes/src/prayer/prayer/cmd_restart.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* XXX Possible to push MAILSTREAM stuff down into session_streams? */

void cmd_restart(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct msgmap *zm = session->zm;
    MAILSTREAM *stream;
    struct assoc *h;
    char *s;

    if (request->method != POST) {
        if (session->inbox_stream)
            ml_close(session, session->inbox_stream);

        if (session->other_stream)
            ml_close(session, session->other_stream);

        if (session->draft_stream)
            ml_close(session, session->draft_stream);

        session->inbox_stream = NIL;
        session->other_stream = NIL;
        session->draft_stream = NIL;
        session->stream = NIL;

        session_message_clear(session);

        if (ml_have_error() && (s = ml_errmsg()) && s[0])
            session_message(session, "%s", s);
        else
            session_message(session, ("Lost connection to IMAP server "
                                      "(possible concurrent access?)"));

        html_start(session, b, "Mailbox access error");
        if (session->foldername) {
            session_log(session, "[cmd_restart] Restart: %s",
                        session->foldername);
            bprintf(b,
                    "<h1 align=\"center\">Mailbox access error: %s</h1>",
                    session->foldername);
        } else {
            session_log(session,
                        "[cmd_restart] Restart: (No active folder)");
            bprintf(b, "<h1 align=\"center\">Mailbox access error</h1>");
        }

        html_session_message(session, b);

        html_form_start(session, b, "POST", "restart");
        html_form_submit(b, "exit", "Logout");

        if (!strcasecmp(session->foldername, "INBOX"))
            html_form_submit(b, "reopen", "Reopen mail inbox");
        else
            html_form_submit(b, "reopen", "Reopen this mailbox");

        html_form_submit(b, "folders", "List folders");
        html_form_end(b);
        html_end(b);

        /* Send out the response */
        response_html(request, 200);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "exit")) {
        session->draft_stream = NIL;
        session->other_stream = NIL;
        session->inbox_stream = NIL;
        session_redirect(session, request, "exit");
        return;
    }

    if (assoc_lookup(h, "reopen") && session->foldername) {
        if (!strcasecmp(session->foldername, "inbox")) {
            s = session_mailbox(session, request->pool, "inbox");
            /* Reopen inbox folder */
            if ((stream = ml_open(session, NIL, s, 0))) {
                session->inbox_last_ping_time = time(NIL);
                session->stream = session->inbox_stream = stream;
            }
        } else
            if (!strcasecmp
                (session->foldername, session->draft_foldername)) {
            /* Reopen draft folder */
            s = session_mailbox(session, request->pool,
                                session->draft_foldername);
            if ((stream = ml_open(session, NIL, s, 0))) {
                session->draft_last_ping_time = time(NIL);
                session->stream = session->draft_stream = stream;
            }
        } else {
            /* Reopen other folder */
            s = session_mailbox(session, request->pool,
                                session->foldername);
            if ((stream = ml_open(session, NIL, s, 0))) {
                session->other_last_ping_time = time(NIL);
                session->stream = session->other_stream = stream;
            }
        }

        /* XXX Following code repeated in several places: factor out as routine */
        /* initialise the zoommap */
        msgmap_associate(zm, session, stream);
        msgmap_update(zm);

        /* Set current and last_displayed message to last or first in zoommap
         * (cmd_welcome may override with this last unread message) */
        if (zm->nmsgs > 0) {
            if (zm->sort_reverse)
                session->current = msgmap_value(zm, 1);
            else
                session->current = msgmap_value(zm, zm->nmsgs);
        } else
            session->current = 0;
        session->last_displayed = session->current;

        session_redirect(session, request, "list");
        return;
    }

    /* Shut down other_stream, draft_stream, attempt to reopen INBOX */
    string_free((void **) &session->other_foldername);
    session->other_stream = NIL;
    session->draft_stream = NIL;

    s = session_mailbox(session, request->pool, "inbox");
    if (!(stream = ml_open(session, NIL, s, 0))) {
        session_message(session, ("Unable to reestablish connection"
                                  " with IMAP server "));

        html_start(session, b, "Mailbox access error");
        bputs(b, "<h1 align=\"center\">Mailbox access error</h1>");
        html_session_message(session, b);

        html_form_start(session, b, "POST", "restart");
        html_form_submit(b, "exit", "Logout");
        html_form_submit(b, "reopen", "Reopen this mailbox");
        html_form_submit(b, "folders", "List mail folders");
        html_form_end(b);
        html_end(b);
        /* Send out the response */
        response_html(request, 200);
        return;
    }

    session->inbox_last_ping_time = time(NIL);
    session->stream = session->inbox_stream = stream;

    /* XXX Following code repeated in several places: factor out as routine */
    /* initialise the zoommap */
    msgmap_associate(zm, session, stream);
    msgmap_update(zm);

    /* Set current and last_displayed message to last or first in zoommap
     * (cmd_welcome may override with this last unread message) */
    if (zm->nmsgs > 0) {
        if (zm->sort_reverse)
            session->current = msgmap_value(zm, 1);
        else
            session->current = msgmap_value(zm, zm->nmsgs);
    } else
        session->current = 0;
    session->last_displayed = session->current;

    session_redirect(session, request, "folders");
}
