/* $Cambridge: hermes/src/prayer/prayer/cmd_roles_entry.c,v 1.2 2003/10/23 14:44:56 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session, struct assoc *h)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    char *s;

    /* Title bar */
    html_start(session, b, "Roles Entry");
    html_banner_toolbar(session, b, "roles_entry", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Roles Entry</h2>" CRLF);

    /* Form for roles entry */
    html_form_start(session, b, "POST", "roles_entry");
    bprintf(b, "<table>" CRLF);

    /* Alias name */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Alias", "alias", assoc_lookup(h, "alias"),
                          20);
    bprintf(b, "</tr>" CRLF);

    /* Personal Name */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "From personal name", "personal",
                          assoc_lookup(h, "personal"), 43);
    bprintf(b, "</tr>" CRLF);

    if (!config->fix_from_address) {
        /* From address */
        bprintf(b, "<tr>" CRLF);
        html_form_table_field(b, "From address", "from",
                              assoc_lookup(h, "from"), 43);
        bprintf(b, "</tr>" CRLF);
    }

    /* Reply-To address */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Reply to address", "reply_to",
                          assoc_lookup(h, "reply_to"), 43);
    bprintf(b, "</tr>" CRLF);

    /* Fcc */
    bprintf(b, "<tr>" CRLF);
    html_form_table_field(b, "Fcc", "fcc", assoc_lookup(h, "fcc"), 43);
    bprintf(b, "</tr>" CRLF);

    /* End of table */
    bprintf(b, "</table>" CRLF);

    /* Start of signature */
    bprintf(b, "<h3>Signature:</h3>" CRLF);
    bprintf(b,
            "<textarea name=\"signature\" rows=\"4\" cols=\"80\">" CRLF);
    if ((s = assoc_lookup(h, "signature")))
        html_quote_string(b, s);
    bprintf(b, "</textarea>" CRLF);

    /* Submit options for form */
    bputs(b, "<table><tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_update", "Add/Update entry");
    html_form_table_submit(b, "sub_delete", "Delete entry");
    bprintf(b, "</tr></table>" CRLF);
    html_form_end(b);

    html_end(b);

    response_html(request, 200);        /* Success */
}

void cmd_roles_entry(struct session *session)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct options *options = session->options;
    struct assoc *h = NIL;
    char *alias;
    char *personal;
    char *from;
    char *reply_to;
    char *fcc;
    char *signature;

    if (request->method != POST) {
        if (request->get_suffix) {
            request_decode_get(request);
            h = request->form;
        }
        generate_form(session, h);
        return;
    }

    request_decode_post(request);
    h = request->form;

    if (assoc_lookup(h, "sub_cancel")) {
        session_redirect(session, request, "roles_list");
        return;
    }

    if ((alias = assoc_lookup(h, "alias")))
        alias = string_trim_whitespace(alias);

    if (assoc_lookup(h, "sub_delete")) {
        if (alias && alias[0]) {
            role_delete(options->role_list, alias);
            options->save = T;
            session_message(session, "Deleted role: \"%s\"", alias);
        } else
            session_message(session, "No role supplied to delete");

        session_redirect(session, request, "roles_list");
        return;
    }

    if (!assoc_lookup(h, "sub_update")) {
        session_message(session, "Invalid submit method for roles_entry");
        generate_form(session, h);
        return;
    }

    if ((personal = assoc_lookup(h, "personal")))
        personal = string_trim_whitespace(personal);

    if (config->fix_from_address) {
        from = "";
    } else {
        if ((from = assoc_lookup(h, "from")))
            from = string_trim_whitespace(from);
    }

    if ((reply_to = assoc_lookup(h, "reply_to")))
        reply_to = string_trim_whitespace(reply_to);

    if ((fcc = assoc_lookup(h, "fcc")))
        fcc = string_trim_whitespace(fcc);

    signature = assoc_lookup(h, "signature");

    if (!(alias && alias[0])) {
        session_message(session, "No alias supplied");
        generate_form(session, h);
        return;
    }

    if (!(personal && from && reply_to && fcc && signature)) {
        session_message(session,
                        "Fields missing from form (program error)");
        session_log(session, "Fields missing from form (program error)");
        generate_form(session, h);
        return;
    }

    if (from && from[0]) {
        ADDRESS *addr = addr_parse(request->pool, from, "");
        BOOL error = T;

        if (!addr)
            session_message(session, "From Address invalid: %s", ml_errmsg());
        else if (addr && addr->next)
            session_message(session,
                            "From address must be single address");
        else if (!(addr->host && addr->host[0]))
            session_message(session, "From address must be qualified");
        else if (addr->personal && addr->personal[0])
            session_message(session,
                            "From address should not contain personal name");
        else
            error = NIL;

        if (addr)
            mail_free_address(&addr);

        if (error) {
            generate_form(session, h);
            return;
        }
    }

    if (reply_to && reply_to[0]) {
        ADDRESS *addr = addr_parse(request->pool, reply_to, "");
        ADDRESS *a = NIL;
        BOOL error = NIL;

        if (!addr) {
            session_message(session, "Reply-To Address invalid: %s",
                            ml_errmsg());
            error = T;
        } else {
            for (a = addr; a; a = a->next) {
                if (!(a->host && a->host[0])) {
                    session_message(session,
                                    "Unqualified address in Reply-To list");
                    error = T;
                    break;
                }
            }
            mail_free_address(&addr);
        }
        if (error) {
            generate_form(session, h);
            return;
        }
    }

    if (fcc && fcc[0] && !string_filename_valid(fcc)) {
        session_message(session, "Invalid Fcc name");
        generate_form(session, h);
        return;
    }

    role_update(options->role_list,
                alias, personal, from, reply_to, fcc, signature);

    session_message(session, "Updated role: \"%s\"", alias);
    options->save = T;
    session_redirect(session, request, "roles_list");
}
