/* $Cambridge: hermes/src/prayer/prayer/cmd_sieve.c,v 1.9 2004/03/23 11:52:45 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* Check whether string empty */
static BOOL
cmd_sieve_has_content(char *s)
{
    char c;

    while ((c=*s++)) {
        if ((c != ' ') && (c != '\t') && (c != '\015') && (c != '\012'))
            return(T);
    }
    return(NIL);
}

/* Replace a few daft error messages from Sieve */
static char *
cmd_sieve_substitute(char *s)
{
   if (!strcmp(s, "parse error, unexpected $undefined."))
        s = "syntax error";

   return(s);
}

static void
cmd_sieve_print_error_line(struct session *session, struct buffer *b, char *s)
{
    struct config_theme *theme = session->theme_main;
    unsigned long lineno;

    if ((strncmp(s, "line ", strlen("line ")) != 0) ||
        ((lineno=atoi(s+strlen("line "))) == 0)) {

        while (string_isspace(*s))
            s++;

        bprintf(b, CRLF"%s"CRLF, cmd_sieve_substitute(s));
        return;
    }

    s += strlen("line ");

    while (*s && Uisdigit(*s))
        s++;
        
    if ((s[0] == ':') && (s[1] == ' '))
        s += 2;

    while (string_isspace(*s))
        s++;

    bprintf(b, "<font color=\"%s\"> Line %lu:</font> %s"CRLF,
            theme->fgcolor_quote2, lineno, cmd_sieve_substitute(s));
}

static void
cmd_sieve_merge_text(struct session *session, struct buffer *b,
                     char *script, char *error)
{
    struct config_theme *theme = session->theme_main;
    unsigned long lineno = 0;
    unsigned long tmp_lineno = 0;
    unsigned long err_lineno = 0;
    char *line, *errline, tmp[64];

    tmp_lineno = 0;
    while ((errline = string_get_line(&error))) {
        tmp_lineno++;
        if (!strncmp(errline, "line ", strlen("line ")) &&
            ((err_lineno = atoi(errline + strlen("line "))) > 0))
            break;  /* Got a numbered line */

        if ((tmp_lineno == 1) && !strcmp(errline, "script errors:"))
            continue;

        /* Print unmatched lines at the start */
        cmd_sieve_print_error_line(session, b, errline);
    }

    lineno = 0;
    while ((line=string_get_line(&script))) {
        lineno++;
        sprintf(tmp, "%5lu", lineno);
        bprintf(b, "<font color=\"%s\">%s:</font> %s"CRLF,
                theme->fgcolor_quote1, tmp, line);

        if (errline && (lineno == err_lineno)) {
            bputs(b, ""CRLF);
            cmd_sieve_print_error_line(session, b, errline);

            /* Check for possible continuation lines (no linenumber given)
               or further errors on the same line */
            while ((errline = string_get_line(&error))) {
                if (!strncmp(errline, "line ", strlen("line ")) &&
                    ((err_lineno = atoi(errline + strlen("line "))) > lineno))
                    break;

                cmd_sieve_print_error_line(session, b, errline);
            }
            bputs(b, ""CRLF);
        }
    }
    /* Anything left at the end? */
    while (errline) {
        cmd_sieve_print_error_line(session, b, errline);
        errline = string_get_line(&error);
    }
}

static void
generate_sieve_error(struct session *session, char *script, char *error)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Sieve filtering");
    html_banner_toolbar(session, b, "sieve", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Sieve Filtering</h2>" CRLF);

    bputs(b, "Failed to upload script. Sieve server reported:");

    bputs(b, "<pre>"CRLF);
    cmd_sieve_merge_text(session, b, script, error);
    bputs(b, "</pre>"CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "sieve");
    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_sieve", "Back to Sieve Page");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void
generate_connect_error(struct session *session)
{
    struct request *request = session->request;
    char *msg = sieve_fetch_message(session->sieve);
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Sieve Filtering");
    html_banner_toolbar(session, b, "sieve", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Sieve Filtering</h2>" CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg)
        html_quote_string(b, msg);
    else
        bputs(b, "Unable to check mail processing status with server");
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "sieve");

    /* Cancel button */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);
}

static void
generate_form(struct session *session)
{
    struct prefs *prefs = session->options->prefs;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct sieve *sieve = session->sieve;
    char *script = sieve_current(sieve);

    /* Title bar */
    html_start(session, b, "Sieve filtering");
    html_banner_toolbar(session, b, "sieve", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Sieve Filtering</h2>" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "sieve");

    /* Start of sieve script */
    bprintf(b, "<textarea name=\"script\" rows=\"%d\" cols=\"%d\">"CRLF,
            prefs->small_rows, prefs->small_cols);

    if (script)
        html_quote_string(b, script);

    bputs(b, "</textarea>" CRLF);

    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Manage screen");
    html_form_table_submit(b, "sub_apply", "Apply");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* End table */
    html_form_end(b);
    html_end(b);
    response_html(request, 200);        /* Success */
}

BOOL
my_sieve_check(struct sieve *sieve, struct session *session)
{
    char *script;

    if (sieve->checked)
        return(T);

    if (!sieve_fetch(sieve, session, "sieve", &script))
        return(NIL);

    sieve_current_record(sieve, script);
    sieve_live_record(sieve, script);

    sieve->checked = T;
    return(T);
}

void cmd_sieve(struct session *session)
{
    struct request *request = session->request;
    struct sieve  *sieve = session->sieve;
    struct account *account = session->account;
    struct assoc *h = NIL;
    char *script;

    if (request->method == POST) {
        /* Form post */
        request_decode_post(request);
        h = request->form;
    }

    if ((request->method != POST) || assoc_lookup(h, "sub_sieve")) {
        if (!my_sieve_check(sieve, session))
            generate_connect_error(session);
        else
            generate_form(session);
        return;
    }

    if (!assoc_lookup(h, "sub_apply")) {
        session_redirect(session, request, "manage");   /* parent screen */
        return;
    }

    if (!(script = assoc_lookup(h, "script"))) {
        session_message(session, "Invalid form input");
        session_redirect(session, request, "manage");   /* parent screen */
        return;
    }

    if (script && cmd_sieve_has_content(script)) {
        sieve_current_record(sieve, script);

        if (!(sieve_upload(sieve, session, "sieve", script) &&
              sieve_activate(sieve, session, "sieve"))) {
            char *error = sieve_fetch_message(sieve);

            generate_sieve_error(session, script, error);
            return;
        }
        sieve_live_record(sieve, script);
        session_message(session, "Sieve script sucessfully installed");
        account->mail_checked = NIL; /* Blat existing automatic filtering */
    } else {
        sieve_current_record(sieve, NIL);

        if (sieve_live(sieve)) {
            if (sieve_delete(sieve, session, "sieve")) {
                session_message(session, "Removed sieve filter");
                sieve_live_record(sieve, NIL);
            } else
                session_message(session, "Problem removing sieve filter");
        }
    }
    session_redirect(session, request, "manage");
}
