/* $Cambridge: hermes/src/prayer/prayer/cmd_transfer.c,v 1.5 2004/03/23 11:37:56 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_inbox_line(struct session *session,
                    struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td width=\"10%%\"><a href=\"%s/NOSEQ/transfer/INBOX\">",
            session->url_prefix_bsession);

    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td width=\"40%\">INBOX</td>" CRLF);

    bprintf(b,
            "<td width=\"50%%\"><a href=\"%s/NOSEQ/transfer/INBOX\">" CRLF,
            session->url_prefix_bsession);
    bputs(b, "Download</a></td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *parent, *s;

    /* Generate path for parent */
    parent = pool_strdup(session->pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    if (session->cwd && session->cwd[0]) {
        bputs(b, "<td>" CRLF);
        html_session_bprintf(session, b, "<a href=\"", "\">",
                             "transfer?cwd=%s",
                             string_canon_path_encode(pool, "", parent));
        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a></td>" CRLF);
    } else {
        bputs(b, "<td>");
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</td>" CRLF);
    }

    bputs(b, "</tr>");
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td><a href=\"%s/transfer?cwd=%s\">" CRLF,
            session->url_prefix_session, name);
    html_icon(session, b, "right.gif", "[directory]");
    bprintf(b, "</a></td>" CRLF);

    html_session_bprintf(session, b,
                         "<td><a href=\"", "\">" CRLF, "transfer?cwd=%s",
                         name);
    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);

    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_favourite_folder_line(struct session *session,
                               struct pool *pool,
                               struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, "", folder);
    char *fname = string_filename_encode(pool, folder);
    char *basename;

    if (!(basename = strrchr(folder, '/')))
        basename = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td><a href=\"%s/NOSEQ/tranfer/%s/%s\">" CRLF,
            session->url_prefix_bsession, name, fname);
    html_icon(session, b, "dir.gif", "[folder]");
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td><a>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</a></td>" CRLF);

    bprintf(b, "<td><a href=\"%s/NOSEQ/transfer/%s/%s\">" CRLF,
            session->url_prefix_bsession, name, fname);
    bputs(b, "Download</a></td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, folder);
    char *fname = string_filename_encode(pool, folder);
    char *basename;

    if (!(basename = strrchr(folder, '/')))
        basename = folder;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bprintf(b, "<td><a href=\"%s/NOSEQ/transfer/%s/%s\">" CRLF,
            session->url_prefix_bsession, name, fname);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</a></td>" CRLF);

    bputs(b, "<td>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</td>" CRLF);

    bprintf(b, "<td><a href=\"%s/NOSEQ/transfer/%s/%s\">" CRLF,
            session->url_prefix_bsession, name, fname);
    bputs(b, "Download</a></td>" CRLF);
    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */


static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b, struct dirlist *dl)
{
    struct config_theme *theme = session->theme_main;
    struct prefs *prefs = session->options->prefs;
    struct favourite_list *fl = session->options->favourite_list;
    struct list_item *li;
    BOOL shaded = NIL;

    /* Title Bar */
    html_start(session, b, "Transfer");
    html_banner_toolbar(session, b, "transfer", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Mailbox Transfer</h2>" CRLF);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "transfer");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td><table><tr><td>" CRLF);
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, "folders", "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/transfer",
                                "Refresh");
    bputs(b, "</tr></table></td></tr></table>" CRLF);
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">" CRLF);

    /* Inbox item */
    if (session->cwd && session->cwd[0]) {
        generate_inbox_line(session, pool, b, shaded);
        shaded = (shaded) ? NIL : T;
    }

    if (list_length(fl->list)) {
        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");

        /* Replace td with something which stops silly spacing */
        bputs(b, "<td>Favourite mail folders:</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);

        shaded = NIL;
        for (li = fl->list->head; li; li = li->next) {
            generate_favourite_folder_line(session, pool, b, li->name,
                                           shaded);
            shaded = (shaded) ? NIL : T;
        }

        if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
        else
            bputs(b, "<tr>");

        bputs(b, "<td>Normal folder listing:</td>");
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "<td>&nbsp;</td>" CRLF);
        bputs(b, "</tr>" CRLF);
    }

    shaded = NIL;

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (prefs->suppress_dotfiles && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    /* Upload form */
    html_session_bputs(session, b,
                       "<form method=\"POST\" action=\"",
                       "\" enctype=\"multipart/form-data\">" CRLF,
                       "upload_select");

    bputs(b, "<table width=\"100%\">");
    if (config_theme_check_bgcolor(theme, theme->bgcolor_banner))
        bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_banner);
    else
        bputs(b, "<tr>");
    bputs(b, "<td>" CRLF);
    bputs(b, "<table><tr><td>" CRLF);
    bputs(b, "File: " CRLF);
    bputs(b,
          "<input align=\"middle\" type=\"file\" name=\"file\" size=40>"
          CRLF);
    bputs(b, "<input type=\"submit\" value=\"Upload\">");
    bputs(b, "</td></tr></table></td>" CRLF);
    bputs(b, "<td align=\"right\"><table><tr><td>" CRLF);
    html_textlink_session_table(session, b, "folders", "Cancel");
    bputs(b, "<td>|</td>" CRLF);
    html_textlink_session_table(session, b, "dir_check/transfer",
                                "Refresh");
    bputs(b, "</tr></table>" CRLF);
    bputs(b, "</td></tr></table>" CRLF);
    bputs(b, "</form" CRLF);
    html_end(b);
}

/* ====================================================================== */

/* unix_pseudo stolen from c-client */
extern unsigned long unix_pseudo (MAILSTREAM *stream,char *hdr);

static BOOL download(struct session *session, char *name)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    MAILSTREAM *tstream;
    unsigned long msgno, len;
    char *text, *s, c;
    char lastc = '\n';
    MESSAGECACHE *elt;
    ENVELOPE *env;
    ADDRESS *addr;
    time_t now = time(NIL);
    int flag;
    char tmp[MAILTMPLEN];
    unsigned long uid;

    session->xfer_stream
        = ml_open(session, session->xfer_stream,
                  session_mailbox(session, request->pool, name),
                  OP_READONLY);

    if (!session->xfer_stream) {
        session_message(session, "Failed to open mail folder: %s", name);
        session_log(session,
                    "[cmd_transfer] Failed to open mail folder: %s", name);
        return (NIL);
    }

    tstream = session->xfer_stream;

    /* Fetch overview for all messages in folder */
    if (!ml_fetch_overview(session, tstream, "1:*", NIL)) {
        session_message(session, "Failed to fetch folder overview: %s",
                        name);
        session_log(session,
                    "[cmd_transfer]] Failed to fetch overview: %s", name);
        return (NIL);
    }

    /* Fudge up unix pseudo message for Keywords etc */
    unix_pseudo(tstream, tmp);
    bputs(b, tmp);

    for (msgno = 1; msgno <= tstream->nmsgs; msgno++) {
        if (!((uid = ml_uid(session, tstream, msgno)) &&
              (elt = ml_elt(session, tstream, msgno)) &&
              (env = ml_fetch_structure(session, tstream, msgno, NIL, 0))))
            return (NIL);

        if ((addr = env->sender) && addr->mailbox && addr->host)
            bprintf(b, "From %s@%s %s", addr->mailbox, addr->host,
                    ctime(&now));
        else
            bprintf(b, "From unknown@nowhere.com %s", ctime(&now));


        if (!
            (text =
             ml_fetch_header(session, tstream, msgno, NIL, NIL, &len, 0)))
        {
            session_message(session,
                            "Couldn't fetch header for message %lu",
                            msgno);
            session_log(session,
                        ("[cmd_transfer] Couldn't fetch header for message %lu"
                         "in folder: %s"), msgno, name);
            return (NIL);
        }

        /* Copy header replacing CRLF, CR or LF with LF */
        s = text;
        while ((c = *s)) {
            /* Chomp final CRLF sequence */
            if ((s[0] == '\015') && (s[1] == '\012') && (s[2] == '\0'))
                break;

            if (((s[0] == '\015') || (s[0] == '\012')) && (s[1] == '\0'))
                break;

            if ((c == '\015') || (c == '\012')) {
                /* Replace CRLF with single LF */
                s += ((s[0] == '\015') && (s[1] == '\012')) ? 2 : 1;
                bputc(b, '\n');

                /* Quote "From" on following line */
                if (!strncmp(s, "From ", strlen("From ")))
                    bputc(b, '>');

                continue;
            }
            bputc(b, c);
            s++;
        }

        bputs(b, "Status: ");
        if (elt->seen)     bputc(b, 'R');
        if (!elt->recent)  bputc(b, 'O');
        bputs(b, "\nX-Status: ");
        if (elt->deleted)  bputc(b, 'D');
        if (elt->flagged)  bputc(b, 'F');
        if (elt->answered) bputc(b, 'A');
        if (elt->draft)    bputc(b, 'T');
        bputs(b, "\nX-Keywords:");
        for (flag = 0 ; flag < NUSERFLAGS ; flag++) {
            if ((elt->user_flags & (1<<flag)) && tstream->user_flags[flag]) {
                bprintf(b, " %s", tstream->user_flags[flag]);
            }
        }
        bprintf(b, "\nX-UID: %lu\n\n", uid);

        if (!(text = ml_fetch_text(session, tstream, msgno, NIL, &len, 0))) {
            session_message(session, "Couldn't fetch body for message %lu",
                            msgno);
            session_log(session,
                        ("[cmd_transfer] Couldn't fetch body for message %lu"
                         "in folder: %s"), msgno, name);
            return (NIL);
        }

        /* Copy body replacing CRLF, CR or LF with LF */
        s = text;
        while ((c = *s)) {
            if ((c == '\015') || (c == '\012')) {
                /* Replace CRLF with single LF */
                s += ((s[0] == '\015') && (s[1] == '\012')) ? 2 : 1;
                bputc(b, '\n');
                lastc = '\n';

                /* Quote "From" on following line */
                if (!strncmp(s, "From ", strlen("From ")))
                    bputc(b, '>');

                continue;
            }
            lastc = c;
            bputc(b, c);
            s++;
        }
        if (lastc != '\n')
            bputc(b, '\n');
        bputc(b, '\n');
    }

    response_raw(request, name, "application/octet-stream", 200);
    return (T);
}

/* ====================================================================== */

void cmd_transfer(struct session *session)
{
    struct request *request = session->request;
    struct pool *pool = request->pool;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    request_decode_get(request);
    h = request->form;

    if ((s = assoc_lookup(h, "cwd"))) {
        string_canon_decode(s);

        if (string_filename_valid(s))
            string_strdup(&session->cwd, s);
        else
            session_message(session, "Path contained illegal characters");
    }

    if ((s = assoc_lookup(h, "filter"))
        && (strcmp(s, session->dir_filter) != NIL))
        string_strdup(&session->dir_filter, s);

    if (request->argc > 1) {
        if (!download(session, string_canon_decode(request->argv[1])))
            session_redirect(session, request, "restart");

        return;
    }

    html_folderlist(session, pool, b,
                    dircache_fetch(session, session->cwd));

    response_html(request, 200);
}
