/* $Cambridge: hermes/src/prayer/prayer/cmd_upload_select.c,v 1.2 2004/04/20 08:59:37 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
generate_parent_line(struct session *session,
                     struct pool *pool, struct buffer *b, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *parent, *s;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    /* Generate path for parent */
    parent = pool_strdup(session->pool, session->cwd);
    if ((s = strrchr(parent, '/')))
        *s = '\0';
    else
        *parent = '\0';

    if (session->cwd && session->cwd[0]) {
        bputs(b, "<td width=\"10%\">" CRLF);
        html_session_bprintf(session, b,
                             "<a href=\"", "\">", "upload_select?cwd=%s",
                             string_canon_path_encode(pool, "", parent));

        html_icon(session, b, "left.gif", "[back]");
        bputs(b, "</a></td>" CRLF);
    } else {
        bputs(b, "<td width=\"10%\">");
        html_icon(session, b, "left-inactive.gif", "[back]");
        bputs(b, "</td>" CRLF);
    }
    bputs(b, "<td width=\"40%\">&nbsp;</td>" CRLF);
    bputs(b, "<td width=\"40%\">&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_directory_line(struct session *session,
                        struct pool *pool,
                        struct buffer *b, char *dir, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;
    char *name = string_canon_path_encode(pool, session->cwd, dir);

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_session_bprintf(session, b,
                         "<a href=\"", "\">" CRLF, "upload_select?cwd=%s",
                         name);

    html_icon(session, b, "right.gif", "[directory]");
    bprintf(b, "</a></td>" CRLF);
    html_session_bprintf(session, b, "<td><a href=\"", "\">" CRLF,
                         "upload_select?cwd=%s", name);

    html_quote_string(b, dir);
    bprintf(b, "</a></td>" CRLF);
    bputs(b, "<td>&nbsp;</td>" CRLF);
    bputs(b, "</tr>" CRLF);
}

static void
generate_folder_line(struct session *session,
                     struct pool *pool,
                     struct buffer *b, char *folder, BOOL shaded)
{
    struct config_theme *theme = session->theme_main;
    char *color = (shaded) ? theme->bgcolor_row1 : theme->bgcolor_row2;

    if (config_theme_check_bgcolor(theme, color))
        bprintf(b, "<tr bgcolor=\"%s\">", color);
    else
        bputs(b, "<tr>");

    bputs(b, "<td>" CRLF);
    html_icon(session, b, "dir.gif", "[mailbox]");
    bputs(b, "</td>" CRLF);

    bputs(b, "<td>" CRLF);
    html_quote_string(b, folder);
    bputs(b, "</td>" CRLF);

    html_textlink_mailbox_table(session, b, "upload_xfer", folder,
                                "Append");
    bprintf(b, "</tr>" CRLF);
}

/* ====================================================================== */

static void
html_folderlist(struct session *session,
                struct pool *pool, struct buffer *b, struct dirlist *dl)
{
    BOOL shaded = NIL;

    /* Title bar */
    html_start(session, b, "Select Target Mailfolder");
    html_banner_toolbar(session, b, "upload_select", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Upload local folder \"");
    if (session->upload_name)
        html_quote_string(b, session->upload_name);
    bputs(b, "\" as folder</h2>" CRLF);

    /* Change directory form, including back button */
    html_form_start(session, b, "GET", "upload_select");
    html_form_field(b, "Directory", "cwd", session->cwd, 32);
    html_form_field(b, "Filter", "filter", session->dir_filter, 12);
    html_form_submit(b, "submit", "Apply");
    html_form_end(b);

    /* Start table for mailfolder listing */
    bputs(b, "<table width=\"100%\">");

    /* Parent directory link */
    generate_parent_line(session, pool, b, shaded);
    shaded = (shaded) ? NIL : T;

    for (; dl; dl = dl->next) {
        if (session->options->prefs->suppress_dotfiles
            && (dl->name[0] == '.'))
            continue;

        if (!(dirlist_filter_check(dl->name, session->dir_filter)))
            continue;;

        if (dl->isdir)
            generate_directory_line(session, pool, b, dl->name, shaded);
        else
            generate_folder_line(session, pool, b, dl->name, shaded);

        shaded = (shaded) ? NIL : T;
    }

    /* End table for mailfolder listing */
    bprintf(b, "</table><br>" CRLF);

    /* Horizontal rule */
    bprintf(b, "<hr>" CRLF);

    html_form_start(session, b, "GET", "upload_xfer");
    html_form_field(b, "Upload folder with name:", "name",
                    session->upload_name, 40);

    html_form_submit(b, "create", "Create");
    html_form_submit(b, "append", "Append");
    html_form_submit(b, "cancel", "Cancel");
    html_form_end(b);
    html_end(b);
}

/* ====================================================================== */

static BOOL upload(struct session *session,
                   char *name, char *data, unsigned long len)
{
    struct config *config = session->config;
    struct request *request = session->request;
    char *tmpfname;
    FILE *tmpfile;
    int fd;

    tmpfname = pool_printf(request->pool, "%s/%s:%lu:XXXXXX",
                           config->tmp_dir, session->username,
                           (unsigned long) getpid());

    if ((fd = mkstemp(tmpfname)) < 0)
        return (NIL);

    /* Transfer data to temporary file */
    if ((tmpfile = fdopen(fd, "w")) == NIL) {
        session_message(session, "Failed to open temporary file");
        return (NIL);
    }

    while (len > 0) {
        fputc((*data) ? (*data) : ' ', tmpfile);

        data++;
        len--;
    }
    fclose(tmpfile);

    if (session->upload_file) {
        unlink(session->upload_file);
        string_free((void **) &session->upload_file);
    }
    session->upload_file = pool_strdup(NIL, tmpfname);
    return (T);
}

/* ====================================================================== */

BOOL
process_upload_request(struct session * session, struct request * request)
{
    char *s, *disp;
    char *start, *end;
    struct assoc *hdrs = assoc_create(request->pool, 16, T);
    char *name = NIL;
    unsigned long len;

    request_decode_post_multipart(request, hdrs, &start, &end);
    *end = '\0';

    if (end > start)
        len = (char *) end - (char *) start;
    else
        len = 0;

    if ((disp = assoc_lookup(hdrs, "content-disposition"))) {
        for (s = disp; *s; s++) {
            if (!strncasecmp(s, "filename=\"", strlen("filename=\""))) {
                name = s = s + strlen("filename=\"");
                while (*s && (*s != '"'))
                    s++;
                *s = '\0';
                break;
            }
        }
    }
    if (name && name[0]) {
        if ((s = strrchr(name, '/')))
            name = s + 1;
        else if ((s = strrchr(name, '\\')))
            name = s + 1;
    } else
        name = "Upload";

    if (!upload(session, name, start, len))
        return (NIL);

    session_message(session, "Copied mailbox to halfway point");
    session->upload_name = pool_strdup(NIL, name);
    return (T);
}

void cmd_upload_select(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct assoc *h = NIL;
    char *s;

    if (request->method == POST) {
        if (!process_upload_request(session, request)) {
            session_redirect(session, request, "restart");
            return;
        }
    }

    if (request->get_suffix) {
        request_decode_get(request);
        h = request->form;

        if ((s = assoc_lookup(h, "cwd"))) {
            string_canon_decode(s);

            if (string_filename_valid(s))
                string_strdup(&session->cwd, s);
            else
                session_message(session,
                                "Path contained illegal characters");
        }

        if ((s = assoc_lookup(h, "filter"))
            && (strcmp(s, session->dir_filter) != NIL))
            string_strdup(&session->dir_filter, s);
    }

    html_folderlist(session, request->pool, b,
                    dircache_fetch(session, session->cwd));

    response_html(request, 200);

}
