/* $Cambridge: hermes/src/prayer/prayer/cmd_welcome.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static char *read_welcome(struct pool *pool, char *filename)
{
    int fd;
    struct stat sbuf;
    unsigned long size;
    char *result;

    /* Read in from file */
    if ((fd = open(filename, O_RDONLY)) < 0)
        return (NIL);

    if ((fstat(fd, &sbuf)) < 0)
        return (NIL);

    if ((size = sbuf.st_size) == 0)
        return (NIL);

    result = pool_alloc(pool, sbuf.st_size + 1);

    if ((read(fd, result, size)) < size)
        return (NIL);

    close(fd);

    result[size] = '\0';
    return (result);
}

static void generate_welcome_page(struct session *session, char *string)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    char *s, c;

    /* Title bar */
    html_start(session, b, "Welcome screen");
    html_banner_toolbar(session, b, "welcome", 0);

    /* Title line */

    bputs(b, "<h2 align=\"center\">");

    if (config->login_service_name)
        bprintf(b, "Welcome to the %s Webmail Service" CRLF,
                config->login_service_name);
    else
        bputs(b, "Welcome to the Prayer Webmail Service" CRLF);

    bputs(b, "</h2>" CRLF);

    s = string;
    while ((c = *s)) {
        if (c == '_') {
            if (!strncmp(s, "__TIMEOUT__", strlen("__TIMEOUT__"))) {
                unsigned long hours = config->session_timeout / (60 * 60);
                unsigned long mins = (config->session_timeout / 60) % 60;

                if ((hours > 1) && (mins > 1))
                    bprintf(b, "%lu hours and %lu minutes", hours, mins);
                else if ((hours > 1) && (mins == 1))
                    bprintf(b, "%lu hours and 1 minute", hours);
                else if (hours > 1)
                    bprintf(b, "%lu hours", hours);
                else if (hours == 1)
                    bprintf(b, "1 hour");
                else if (mins > 1)
                    bprintf(b, "%lu minutes", mins);
                else if (mins == 1)
                    bprintf(b, "1 minute");     /* BOFH admin at work! */

                s += strlen("__TIMEOUT__");
                continue;
            }
            if (!strncmp
                (s, "__TIMEOUT_COMPOSE__", strlen("__TIMEOUT_COMPOSE__")))
            {
                unsigned long hours =
                    config->session_timeout_compose / (60 * 60);
                unsigned long mins =
                    (config->session_timeout_compose / 60) % 60;

                if ((hours > 1) && (mins > 1))
                    bprintf(b, "%lu hours and %lu minutes", hours, mins);
                else if ((hours > 1) && (mins == 1))
                    bprintf(b, "%lu hours and 1 minute", hours);
                else if (hours > 1)
                    bprintf(b, "%lu hours", hours);
                else if (hours == 1)
                    bprintf(b, "1 hour");
                else if (mins > 1)
                    bprintf(b, "%lu minutes", mins);
                else if (mins == 1)
                    bprintf(b, "1 minute");     /* BOFH admin at work! */

                s += strlen("__TIMEOUT__");
                continue;
            }
            if (!strncmp(s, "__CONTACT__", strlen("__CONTACT__"))) {
                if (config->contact_email)
                    html_quote_string(b, config->contact_email);
                else
                    bputs(b, "No contact address defined");

                s += strlen("__CONTACT__");
                continue;
            }
        }
        bputc(b, c);
        s++;
    }

    /* Form body */
    html_form_start(session, b, "POST", "welcome");

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr><td>&nbsp;</td></tr>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_checkbox(b, "Show this screen next time you log in",
                             "enable_welcome", T);
    bputs(b, "</tr>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_proceed", "Proceed");
    bprintf(b, "</tr></table>" CRLF);

    /* End of form */
    html_form_end(b);
    html_end(b);

    response_html(request, 200);
}

void cmd_welcome(struct session *session)
{
    struct config *config = session->config;
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct request *request = session->request;
    char *string;
    unsigned long msgno;

    if ((request->method != POST) && (prefs->use_welcome)) {
        if (config->welcome_path &&
            (string = read_welcome(request->pool, config->welcome_path)))
            generate_welcome_page(session, string);
        else
            session_redirect(session, request, "list");

        return;
    }

    if (request->method == POST) {
        request_decode_post(request);
        if (!assoc_lookup(request->form, "enable_welcome")) {
            options->prefs->use_welcome = NIL;
            options->prefs_save->use_welcome = NIL;
            session->options->save = T;
        }
    }

    msgno = stream_find_unread(session, session->stream);

    if (msgno > 0) {
        session->current = msgno;
        session->last_displayed = msgno;
        session_message(session, "First unread message: %lu", msgno);
    } else
        session_message(session, "No unread messages in inbox");

    session_redirect(session, request, "list");
}
