/* $Cambridge: hermes/src/prayer/prayer/html.c,v 1.4 2004/10/27 14:58:15 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* Set of routines for generating chunks of HTML markup. */

/* html_start() **********************************************************
 *
 * HTML header
 ************************************************************************/

void html_start(struct session *session, struct buffer *b, char *title)
{
    struct config *config = session->config;
    struct config_theme *theme = session->theme_main;
    FILE *file;
    int c;

    bputs(b,
          ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\""
           " \"http://www.w3.org/TR/REC-html401/loose.dtd\">" CRLF));
    bputs(b, "<html>" CRLF);
    bputs(b, "<head>" CRLF);
    if (config->login_service_name)
        bprintf(b,
                "<title>%s Webmail service: %s</title>" CRLF,
                config->login_service_name, title);
    else
        bprintf(b, "<title>Webmail service: %s</title>" CRLF, title);
    bputs(b, "<meta name=\"robots\" content=\"none\">" CRLF);
    bputs(b, ("<meta http-equiv=\"Content-Type\" "
              "content=\"text/html; charset=ISO-8859-1\">" CRLF));
    bputs(b, "</head>" CRLF);

    bputs(b, "<body");
    if (theme->bgcolor)
        bprintf(b, " bgcolor=\"%s\"", theme->bgcolor);

    if (theme->fgcolor)
        bprintf(b, " text=\"%s\"", theme->fgcolor);

    if (theme->fgcolor_link) {
        bprintf(b, " alink=\"%s\"", theme->fgcolor_link);
        bprintf(b, " vlink=\"%s\"", theme->fgcolor_link);
    }
    bputs(b, ">" CRLF);

    if (config->session_banner_path &&
        ((file=fopen(config->session_banner_path, "r")) != NULL)) {
        /* Config provides template for session headline banner */
        while ((c=fgetc(file)) != EOF) {
            if (c=='\n')
                bputs(b, ""CRLF);
            else
                bputc(b, c);
        }
        fclose(file);
    }
}

/* html_help_start() *****************************************************
 *
 * HTML header for help text: different background colour.
 ************************************************************************/

void
html_help_start(struct session *session, struct buffer *b, char *title)
{
    struct config *config = session->config;
    struct config_theme *theme = session->theme_help;

    bputs(b,
          ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\""
           " \"http://www.w3.org/TR/REC-html401/loose.dtd\">" CRLF));
    bputs(b, "<html>" CRLF);
    bputs(b, "<head>" CRLF);
    if (config->login_service_name)
        bprintf(b, "<title>%s Webmail Service: %s</title>" CRLF,
                config->login_service_name, title);
    else
        bprintf(b, "<title>Webmail Service: %s</title>" CRLF, title);

    bputs(b, "<meta name=\"robots\" content=\"none\">" CRLF);
    bputs(b, ("<meta http-equiv=\"Content-Type\" "
              "content=\"text/html; charset=ISO-8859-1\">" CRLF));
    bputs(b, "</head>" CRLF);

    bputs(b, "<body");
    if (theme->bgcolor)
        bprintf(b, " bgcolor=\"%s\"", theme->bgcolor);

    if (theme->fgcolor)
        bprintf(b, " text=\"%s\"", theme->fgcolor);

    if (theme->fgcolor_link) {
        bprintf(b, " alink=\"%s\"", theme->fgcolor_link);
        bprintf(b, " vlink=\"%s\"", theme->fgcolor_link);
    }
    bputs(b, ">" CRLF);
}

/* html_end() ************************************************************
 *
 * End of HTML body
 ************************************************************************/

void html_end(struct buffer *b)
{
    bputs(b, "</body></html>" CRLF);
}

/* html_banner_start() ***************************************************
 *
 * Start of icon banner.
 ************************************************************************/

void html_banner_start(struct buffer *b, BOOL full)
{
    if (full)
        bputs(b, "<table width=\"100%\">" CRLF "<tr>" CRLF);
    else
        bputs(b, "<table>" CRLF "<tr>" CRLF);
}

/* html_banner_end() *****************************************************
 *
 * End of icon banner.
 ************************************************************************/

void html_banner_end(struct buffer *b)
{
    bputs(b, "</tr></table>" CRLF);
}

/* ====================================================================== */

/* html_session_bputs() **************************************************
 *
 * Print session URL, adding URL prefix (if required) otherwise
 * translating '/' characters in body to '@' for short URL scheme.
 *   session:
 *         b: Target buffer
 *    prefix: Static text to add before URL expansion
 *    suffix: Static text to add after  URL expansion
 *         s: String to print
 ************************************************************************/

void
html_session_bputs(struct session *session, struct buffer *b,
                   char *prefix, char *suffix, char *s)
{
    if (prefix)
        bputs(b, prefix);

    if (session->use_short) {
        bputs(b, session->url_prefix_short);
        bputc(b, '@');
        buffer_puts_translate(b, s);
    } else {
        bputs(b, session->url_prefix_session);
        bputc(b, '/');
        bputs(b, s);
    }

    if (suffix)
        bputs(b, suffix);
}

/* html_session_bprintf() **************************************************
 *
 * Print session URL, adding URL prefix (if required) otherwise
 * translating '/' characters in body to '@' for short URL scheme.
 *   session:
 *         b: Target buffer
 *    prefix: Static text to add before URL expansion
 *    suffix: Static text to add after  URL expansion
 *       fmt: Format string, followed by printf arguments.
 ************************************************************************/

void
html_session_bprintf(struct session *session, struct buffer *b,
                     char *prefix, char *suffix, char *fmt, ...)
{
    va_list ap;

    if (prefix)
        bputs(b, prefix);

    if (session->use_short) {
        bputs(b, session->url_prefix_short);
        bputc(b, '@');
        va_start(ap, fmt);
        buffer_vaprintf_translate(b, fmt, ap);
        va_end(ap);
    } else {
        bputs(b, session->url_prefix_session);
        bputc(b, '/');
        va_start(ap, fmt);
        buffer_vaprintf(b, fmt, ap);
        va_end(ap);
    }

    if (suffix)
        bputs(b, suffix);
}

/* html_icon() ***********************************************************
 *
 * Print icon URL.
 *   session:
 *         b: Target buffer
 *      icon: Icon filename
 *       alt: Alt text to use.
 ************************************************************************/

void
html_icon(struct session *session, struct buffer *b, char *icon, char *alt)
{
    struct user_agent *user_agent = session->user_agent;

    if (!(alt && alt[0]))
        alt = "";

    if (user_agent->use_icons)
        bprintf(b, "<img src=\"%s/%s\" border=0 alt=\"%s\">",
                session->url_prefix_icons, icon, alt);
    else
        bputs(b, alt);
}

/* ====================================================================== */

/* html_textlink_session() ***********************************************
 *
 * Add simple icon with message session link as a non-table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void
html_textlink_session(struct session *session,
                      struct buffer *b, char *cmd, char *text)
{
    html_session_bputs(session, b, "<a href=\"", "\">" CRLF, cmd);
    html_quote_string(b, text);
    bputs(b, "</a>" CRLF);
}

/* html_button_msg() *****************************************************
 *
 * Add simple icon with message session link as a non-table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *   msgno: Msgno for command
 *          (does UID lookup so that command can check that correct message
 *           involved, and search for it if message has moved).
 *     img: Icon filename
 *     alt: Alt text (if any)
 ************************************************************************/

void
html_button_msg(struct session *s,
                struct buffer *b,
                char *cmd, unsigned long msgno, char *img, char *alt)
{
    html_session_bprintf(s, b, "<a href=\"", "\">", "%s/%lu/%lu",
                         cmd, msgno, ml_uid(s, s->stream, msgno));
    html_icon(s, b, img, alt);
    bputs(b, "</a>" CRLF);
}

/* html_textlink_mailbox() ***********************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 * mailbox: Mailbox    (added to cmd as "%s/%s")
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void html_textlink_mailbox(struct session *s, struct buffer *b, char *cmd, char *mailbox,       /* Must be canonical! */
                           char *text)
{
    html_session_bprintf(s, b, "<a href=\"", "\">", "%s/%s", cmd, mailbox);
    html_quote_string(b, text);
    bprintf(b, "</a>" CRLF);
}

/* html_textlink_msg() ***************************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *   msgno: Msgno for command
 *          (does UID lookup so that command can check that correct message
 *           involved, and search for it if message has moved).
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void
html_textlink_msg(struct session *s,
                  struct buffer *b,
                  char *cmd, unsigned long msgno, char *text)
{
    html_session_bprintf(s, b, "<a href=\"", "\">" CRLF, "%s/%lu/%lu",
                         cmd, msgno, ml_uid(s, s->stream, msgno));
    html_quote_string(b, text);
    bprintf(b, "</a>" CRLF);
}

/* ====================================================================== */

/* Table elements: simple buttons and links */

/* html_button_table() ***************************************************
 *
 * Add simple icon with (non-session) link as a table element.
 *      s: Session
 *      b: Buffer
 *    img: Icon filename
 *    alt: Alt text
 *   link: Target URL
 ************************************************************************/

void
html_button_table(struct session *s, struct buffer *b,
                  char *img, char *alt, char *link)
{
    bprintf(b, "<td><a href=\"%s\">" CRLF, link);
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>" CRLF);
}

/* html_button_inactive_table() ******************************************
 *
 * Add inactive icon without target as a table element.
 *      s: Session
 *      b: Buffer
 *    img: Icon filename
 *    alt: Alt text
 ************************************************************************/

void
html_button_inactive_table(struct session *s, struct buffer *b,
                           char *img, char *alt)
{
    bputs(b, "<td>" CRLF);
    html_icon(s, b, img, alt);
    bputs(b, "</td>" CRLF);
}

/* html_button_session_table() *******************************************
 *
 * Add simple icon with session link as a table element.
 *      s: Session
 *      b: Buffer
 *    cmd: Command
 *    img: Icon filename
 *    alt: Alt text
 ************************************************************************/

void
html_button_session_table(struct session *s,
                          struct buffer *b, char *cmd, char *img,
                          char *alt)
{
    html_session_bputs(s, b, "<td><a href=\"", "\">" CRLF, cmd);
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>");
}

/* html_button_mailbox_table() *******************************************
 *
 * Add simple icon with mailbox session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 * mailbox: Added to end of cmd as "%s/%s"
 *     img: Icon filename
 *     alt: Alt text
 ************************************************************************/

void html_button_mailbox_table(struct session *s, struct buffer *b, char *cmd, char *mailbox,   /* Must be canon! */
                               char *img, char *alt)
{
    html_session_bprintf(s, b, "<td><a href=\"", "\">" CRLF,
                         "%s/%s", cmd, mailbox);
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>");
}

/* html_button_msg_table() ************************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *   msgno: Msgno for command
 *          (does UID lookup so that command can check that correct message
 *           involved, and search for it if message has moved).
 *     img: Icon filename
 *     alt: Alt text (if any)
 ************************************************************************/

void
html_button_msg_table(struct session *s,
                      struct buffer *b,
                      char *cmd, unsigned long msgno, char *img, char *alt)
{
    html_session_bprintf(s, b, "<td><a href=\"", "\">", "%s/%lu/%lu",
                         cmd, msgno, ml_uid(s, s->stream, msgno));
    html_icon(s, b, img, alt);
    bputs(b, "</a></td>" CRLF);
}

/* html_textlink_session_table() ******************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void
html_textlink_session_table(struct session *s,
                            struct buffer *b, char *cmd, char *text)
{
    html_session_bputs(s, b, "<td nowrap><a href=\"", "\">" CRLF, cmd);
    html_quote_string(b, text);
    bprintf(b, "</a></td>" CRLF);
}

/* html_textlink_mailbox_table() *****************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 * mailbox: Mailbox    (added to cmd as "%s/%s")
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void html_textlink_mailbox_table(struct session *s, struct buffer *b, char *cmd, char *mailbox, /* Must be canonical! */
                                 char *text)
{
    html_session_bprintf(s, b, "<td nowrap><a href=\"", "\">",
                         "%s/%s", cmd, mailbox);
    html_quote_string(b, text);
    bprintf(b, "</a></td>" CRLF);
}

/* html_textlink_msg_table() *********************************************
 *
 * Add simple icon with message session link as a table element.
 *       s: Session
 *       b: Buffer
 *     cmd: Target URL
 *   msgno: Msgno for command
 *          (does UID lookup so that command can check that correct message
 *           involved, and search for it if message has moved).
 *    text: Text for link (quoted for escape significant characters).
 ************************************************************************/

void
html_textlink_msg_table(struct session *s,
                        struct buffer *b,
                        char *cmd, unsigned long msgno, char *text)
{
    html_session_bprintf(s, b,
                         "<td nowrap><a href=\"", "\">" CRLF, "%s/%lu/%lu",
                         cmd, msgno, ml_uid(s, s->stream, msgno));
    html_quote_string(b, text);
    bprintf(b, "</a></td>" CRLF);
}

/* ====================================================================== */

/* Form items for use outside tables */

/* html_form_start() *****************************************************
 *
 * Markup for start of HTML form
 *       s: Session
 *       b: Buffer
 *  method: "GET" or "POST"
 *     cmd: Command to run when form submitted.
 ************************************************************************/

void
html_form_start(struct session *s, struct buffer *b, char *method,
                char *cmd)
{
    bprintf(b, "<form method=\"%s\" accept-charset=\"ISO-8859-1\"",
            method);
    html_session_bputs(s, b, " action=\"", "\">" CRLF, cmd);
}

/* html_form_end() *******************************************************
 *
 * Markup for end of HTML form
 ************************************************************************/

void html_form_end(struct buffer *b)
{
    bprintf(b, "</form>" CRLF);
}

/* html_form_submit() *****************************************************
 *
 * Markup for end of HTML submit tag
 *      b: Target buffer
 *   name: Submit name
 *  value: Submit value
 ************************************************************************/

void html_form_submit(struct buffer *b, char *name, char *value)
{
    bprintf(b, "<input type=\"submit\" name=\"%s\" value=\"%s\">" CRLF,
            name, value);
}

/* html_form_hidden() ****************************************************
 *
 * Markup for end of HTML hidden tag
 *      b: Target buffer
 *   name: Submit name
 *  value: Submit value
 ************************************************************************/

void html_form_hidden(struct buffer *b, char *name, char *value)
{
    bprintf(b, "<input type=\"hidden\" name=\"%s\" value=\"%s\">" CRLF,
            name, value);
}

/* html_form_field() *****************************************************
 *
 * HTML form "input" field
 *       b: Target buffer
 *    name: Input name
 *   value: Default value
 *    size: Size of input field
 ************************************************************************/

void
html_form_field(struct buffer *b,
                char *desc, char *name, char *value, unsigned long size)
{
    bprintf(b, "%s<input name=\"%s\" value=\"", desc, name);
    if (value)
        html_quote_string(b, value);
    bprintf(b, "\" size=\"%lu\">" CRLF, size);
}

/* html_form_checkbox() **************************************************
 *
 * Checkbox form item
 *       b: Target buffer
 *    desc: Text description of checkbox
 *    name: Name of checkbox item
 * enabled: Initially enabled?
 ************************************************************************/

void
html_form_checkbox(struct buffer *b, char *desc, char *name, BOOL enabled)
{
    bprintf(b, "<input type=\"checkbox\" name=\"%s\" value=\"1\"%s>",
            name, (enabled) ? "checked" : "");
    bputc(b, ' ');
    html_quote_string(b, desc);
    bputs(b, "" CRLF);
}

/* html_form_textarea() **************************************************
 *
 * Textare form item
 * session:
 *       b: Target buffer
 *    name: Name of object
 *   value: Initial/Default value
 *   width: Width of textbox
 *  height: Height of textbox.
 ************************************************************************/

void
html_form_textarea(struct session *session,
                   struct buffer *b, char *name, char *value,
                   unsigned long width, unsigned long height)
{
    struct request *request = session->request;
    struct user_agent *user_agent = request->user_agent;

    if (user_agent->is_netscape4)
        bprintf(b, ("<textarea name=\"%s\" "
                    "cols=\"%lu\" rows=\"%lu\" wrap=\"virtual\">" CRLF),
                name, width, height);
    else
        bprintf(b, "<textarea name=\"%s\" cols=\"%lu\" rows=\"%lu\">" CRLF,
                name, width, height);

    if (value)
        html_quote_string(b, value);
    bprintf(b, "</textarea>" CRLF);
}

/* ====================================================================== */

/* Form items for tables */

/* html_form_table_submit() **********************************************
 *
 * Markup for end of HTML submit tag
 *      b: Target buffer
 *   name: Submit name
 *  value: Submit value
 ************************************************************************/

void html_form_table_submit(struct buffer *b, char *name, char *value)
{
    bprintf(b,
            "<td><input type=\"submit\" name=\"%s\" value=\"%s\"></td>"
            CRLF, name, value);
}

/* html_form_table_hidden() **********************************************
 *
 * Markup for end of HTML hidden tag
 *      b: Target buffer
 *   name: Submit name
 *  value: Submit value
 ************************************************************************/

void html_form_table_hidden(struct buffer *b, char *name, char *value)
{
    bprintf(b,
            "<td><input type=\"hidden\" name=\"%s\" value=\"%s\"></td>"
            CRLF, name, value);
}

/* html_form_table_field() ***********************************************
 *
 * HTML form "input" field
 *       b: Target buffer
 *    name: Input name
 *   value: Default value
 *    size: Size of input field
 ************************************************************************/

void
html_form_table_field(struct buffer *b,
                      char *desc, char *name, char *value,
                      unsigned long size)
{
    if (desc && desc[0])
        bprintf(b, "<td>%s</td>", desc);

    bprintf(b, "<td><input name=\"%s\" value=\"", name);
    if (value)
        html_quote_string(b, value);
    bprintf(b, "\" size=\"%lu\"></td>" CRLF, size);
}

/* html_form_table_checkbox() ********************************************
 *
 * Checkbox form item
 *       b: Target buffer
 *    desc: Text description of checkbox
 *    name: Name of checkbox item
 * enabled: Initially enabled?
 ************************************************************************/

void
html_form_table_checkbox(struct buffer *b, char *desc, char *name,
                         BOOL enabled)
{
    bprintf(b,
            "<td><input type=\"checkbox\" name=\"%s\" value=\"1\"%s></td>",
            name, (enabled) ? "checked" : "");
    bputs(b, "<td>");
    bputs(b, desc);
    bputs(b, "</td>" CRLF);
}


/* html_form_table_textarea() ********************************************
 *
 * Textare form item
 *       b: Target buffer
 *    name: Name of object
 *   value: Initial/Default value
 *   width: Width of textbox
 *  height: Height of textbox.
 ************************************************************************/

void
html_form_table_textarea(struct buffer *b, char *name, char *value,
                         unsigned long width, unsigned long height)
{
    bprintf(b, "<td><textarea name=\"%s\" cols=\"%lu\" rows=\"%lu\">" CRLF,
            name, width, height);

    bprintf(b, "%s</textarea></td>" CRLF, ((value) ? value : ""));
}


/* ====================================================================== */

/* html_session_message() ************************************************
 *
 * Print status message recorded by session_message()
 *   session:
 *         b: Target buffer
 ************************************************************************/

void html_session_message(struct session *session, struct buffer *b)
{
    struct config *config = session->config;
    struct config_theme *theme = session->theme_main;
    struct memblock *m = session->message;
    char *t;

    if ((memblock_size(m) > 0) && (t = memblock_data(m)) && t[0]) {
        bputs(b, "<table width=\"100%\">" CRLF);
        if (theme->bgcolor_status && theme->bgcolor_status[0])
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_status);
        else
            bputs(b, "<tr>");
        bputs(b, "<td align=\"left\"><b>Status:</b> " CRLF);
        html_quote_string(b, t);
        bputs(b, "");
        bputs(b, "</td>" CRLF);
    } else {
        bputs(b, "<table width=\"100%\">" CRLF);
        if (theme->bgcolor_status_none && theme->bgcolor_status_none[0])
            bprintf(b, "<tr bgcolor=\"%s\">", theme->bgcolor_status_none);
        else
            bputs(b, "<tr bgcolor=\"#ccffcc\">");
        bputs(b, "<td nowrap>" CRLF);

        bputs(b, "<b>Reminder:</b> Logout when you have finished." CRLF);
        bputs(b, "</td>" CRLF);
    }

    bputs(b, "<td align=\"right\">");
    if (config->login_service_name)
        bprintf(b, "User currently logged into %s: %s",
                config->login_service_name, session->username);
    else
        bprintf(b, "User currently logged in: %s", session->username);
    bputs(b, "</td>" CRLF);
    bputs(b, "</tr></table>" CRLF);
    session_message_clear(session);
}

/* ====================================================================== */

/* html_banner_toolbar_start() *******************************************
 *
 * Static utility function for html_banner_toolbar()
 *************************************************************************/

static void
html_banner_toolbar_start(struct session *session,
                          struct buffer *b, char *node, BOOL is_form)
{
    struct request *request = session->request;
    struct banner *banner = banner_create(request->pool);

    banner_reset(banner, request->pool, NIL);
    banner_use_submit(banner, is_form);

    if (!strcmp(node, "display"))
        banner_add(banner, "Message", "display-inactive.gif", NIL);
    else
        banner_add(banner, "Message", "display.gif", "display");

    if (!strcmp(node, "list"))
        banner_add(banner, "Mailbox", "mailbox-inactive.gif", NIL);
    else
        banner_add(banner, "Mailbox", "mailbox.gif", "list");

    if (!strcmp(node, "folders"))
        banner_add(banner, "Folders", "folders-inactive.gif", NIL);
    else
        banner_add(banner, "Folders", "folders.gif", "folders");

    if (!strcmp(node, "compose"))
        banner_add(banner, "Compose", "compose-inactive.gif", NIL);
    else
        banner_add(banner, "Compose", "compose.gif", "compose");

    if (!strcmp(node, "abook_list"))
        banner_add(banner, "Addressbook", "addressbook-inactive.gif", NIL);
    else
        banner_add(banner, "Addressbook", "addressbook.gif", "abook_list");

    if (!strcmp(node, "manage"))
        banner_add(banner, "Manage", "manage-inactive.gif", NIL);
    else
        banner_add(banner, "Manage", "manage.gif", "manage");

    banner_print(banner, b, session);
}

/* html_banner_toolbar_favourites() ***************************************
 *
 * Static utility function for html_banner_toolbar()
 *************************************************************************/

static void
html_banner_toolbar_favourites(struct session *session,
                               struct buffer *b, BOOL save)
{
    struct request *request = session->request;
    struct options *options = session->options;
    struct favourite_list *fl = options->favourite_list;
    struct list_item *li;

    if (save)
        bputs(b, ("<select name =\"command\" size=1>" CRLF
                  "<option value=\"copy\" selected>"
                  "Copy marked to:"
                  "</option>" CRLF
                  "<option value=\"change\">"
                  "Change to:" "</option>" CRLF "</select>" CRLF));
    else
        bputs(b, "Change to:" CRLF);

    bputs(b, "<select name=\"folder\" size=1>" CRLF);

    if (fl->preferred && !strcmp(fl->preferred, "inbox"))
        bputs(b, "<option value=\"inbox\" selected>INBOX</option>" CRLF);
    else
        bputs(b, "<option value=\"inbox\">INBOX</option>" CRLF);

    for (li = fl->list->head; li; li = li->next) {
        if (fl->preferred && !strcmp(li->name, fl->preferred)) {
            bprintf(b, "<option value=\"%s\" selected>",
                    string_canon_encode(request->pool, li->name));
        } else {
            bprintf(b, "<option value=\"%s\">",
                    string_canon_encode(request->pool, li->name));
        }

        html_quote_string(b, li->name);
        bputs(b, "</option>" CRLF);
    }
    bputs(b, "</select>" CRLF);
    bputs(b, ("<input type=\"submit\" name=\"sub_folder_dialogue\" "
              "value=\"Go\">" CRLF));
}

/* html_banner_toolbar_folders() ******************************************
 *
 * Static utility function for html_banner_toolbar()
 *************************************************************************/

static void
html_banner_toolbar_folders(struct session *session,
                            struct buffer *b, BOOL save)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct config *config = session->config;
    struct request *request = session->request;
    struct dirlist *dl = dircache_fetch(session, session->cwd);
    struct dirlist *dl2 = dl;
    unsigned long count;
    char *prefix = NIL;

    if (config->change_max_folders > 0) {
        count = 0;
        for (dl2 = dl; dl2; dl2 = dl2->next) {
            if (prefs->suppress_dotfiles && (dl->name[0] == '.'))
                continue;
            count++;
        }

        if (count > config->change_max_folders) {
            bprintf(b,
                    "Change to: <strong>Disabled</strong> (%d folders)"
                    CRLF, count);
            return;
        }
    }

    if (save)
        bputs(b, ("<select name =\"command\" size=1>" CRLF
                  "<option value=\"copy\" selected>"
                  "Copy marked to:"
                  "</option>" CRLF
                  "<option value=\"change\">"
                  "Change to:" "</option>" CRLF "</select>" CRLF));
    else
        bputs(b, "Change to:" CRLF);

    bputs(b, "<select name=\"folder\" size=1>" CRLF);

    bputs(b, "<option value=\"inbox\">INBOX</option>" CRLF);

    if (session->cwd && session->cwd[0])
        prefix = string_canon_encode(request->pool,
                                     pool_strcat(request->pool,
                                                 session->cwd, "/"));


    for (; dl; dl = dl->next) {
        if (prefs->suppress_dotfiles && (dl->name[0] == '.'))
            continue;

        if (dl->isdir)
            continue;

        if (prefix) {
            bprintf(b, "<option value=\"%s%s\">",
                    prefix, string_canon_encode(request->pool, dl->name));
            html_quote_string(b, session->cwd);
            html_quote_string(b, "/");
            html_quote_string(b, dl->name);
            bputs(b, "</option>" CRLF);
        } else if (strcasecmp(dl->name, "INBOX") != 0) {
            bprintf(b, "<option value=\"%s\">",
                    string_canon_encode(request->pool, dl->name));
            html_quote_string(b, dl->name);
            bputs(b, "</option>" CRLF);
        }
    }
    bputs(b, "</select>" CRLF);
    bputs(b, ("<input type=\"submit\" name=\"sub_folder_dialogue\" "
              "value=\"Go\">" CRLF));
}

/* html_banner_toolbar_end() **********************************************
 *
 * Static utility function for html_banner_toolbar()
 *************************************************************************/

static void
html_banner_toolbar_end(struct session *session,
                        struct buffer *b,
                        char *node, char *back, BOOL is_form)
{
    struct request *request = session->request;
    struct banner *banner = banner_create(request->pool);

    banner_reset(banner, request->pool, NIL);
    banner_use_submit(banner, is_form);

    if (back)
        banner_add(banner, "Back", "back.gif", back);
    else if (is_form)
        banner_add(banner, "Help", "help.gif", "help");
    else
        banner_add(banner, "Help", "help.gif",
                   pool_strcat(request->pool, "help/", node));

    if (is_form)
        banner_add(banner, "Logout", "logout.gif", "logout");
    else
        banner_add(banner, "Logout", "logout.gif",
                   pool_strcat(request->pool, "logout/", node));
    banner_print(banner, b, session);
}

/* html_banner_toolbar_help() *********************************************
 *
 * Print common toolbar
 *   session: Global state
 *         b: Target buffer
 *      node: Node for help text and logout references
 *      back: Back link for help page
 ************************************************************************/

void
html_banner_toolbar_help(struct session *session,
                         struct buffer *b, char *node, char *back)
{
    struct options *options = session->options;
    struct favourite_list *fl = options->favourite_list;

    html_form_start(session, b, "POST", "change");

    bputs(b, "<table width=\"100%\"><tr><td>" CRLF);
    html_banner_toolbar_start(session, b, node, NIL);
    bputs(b, "</td><td>" CRLF);

    if (list_length(fl->list) > 0L)
        html_banner_toolbar_favourites(session, b, NIL);
    else
        html_banner_toolbar_folders(session, b, NIL);

    bputs(b, "</td><td align=\"right\">" CRLF);
    html_banner_toolbar_end(session, b, node, back, NIL);
    bputs(b, "</td></tr></table>" CRLF);

    html_form_end(b);
    html_session_message(session, b);
}

/* html_banner_toolbar() *************************************************
 *
 * Print common toolbar
 *   session: Global state
 *         b: Target buffer
 *      node: Node for help text and logout references
 *   is_form: Banner is part of form => need form submit items.
 ************************************************************************/

void
html_banner_toolbar(struct session *session,
                    struct buffer *b, char *node, int bitmask)
{
    struct options *options = session->options;
    struct prefs *prefs = options->prefs;
    struct favourite_list *fl = options->favourite_list;
    BOOL use_save = NIL;
    BOOL use_callback = NIL;
    BOOL use_callback_all = NIL;

    if (prefs->use_mark_persist && (bitmask & HTML_BANNER_CALLBACK_SAVE)) {
        unsigned long marked = msgmap_marked_count(session->zm);

        use_save = (marked > 0) ? T : NIL;
        use_callback = T;
    }

    if (bitmask & HTML_BANNER_CALLBACK_ALL) {
        use_callback = T;
        use_callback_all = T;
    } else
        html_form_start(session, b, "POST",
                        (use_callback) ? node : "change");

    bputs(b, "<table width=\"100%\"><tr><td>" CRLF);
    html_banner_toolbar_start(session, b, node, use_callback_all);
    bputs(b, "</td><td>" CRLF);

    if (list_length(fl->list) > 0L)
        html_banner_toolbar_favourites(session, b, use_save);
    else
        html_banner_toolbar_folders(session, b, use_save);

    bputs(b, "</td><td align=\"right\">" CRLF);
    html_banner_toolbar_end(session, b, node, NIL, use_callback_all);
    bputs(b, "</td></tr></table>" CRLF);

    if (!use_callback_all)
        html_form_end(b);

    if (!(bitmask & HTML_BANNER_SUPPRESS_STATUS))
        html_session_message(session, b);
}

/* ====================================================================== */

/* html_quote_char() ****************************************************
 *
 * Print character replacing significant HTML characters with eqivalent
 * escape sequences.
 * translation stuff.
 *     b: Buffer
 *     c: Character to print
 ***********************************************************************/

void html_quote_char(struct buffer *b, unsigned char c)
{
    if (c > 127) {
        bprintf(b, "&#%lu;", (unsigned long) c);
    } else
        switch (c) {
        case '"':
            bputs(b, "&quot;");
            break;
        case '&':
            bputs(b, "&amp;");
            break;
        case '<':
            bputs(b, "&lt;");
            break;
        case '>':
            bputs(b, "&gt;");
            break;
        default:
            bputc(b, c);
        }
}

/* html_quote() *********************************************************
 *
 * Print string replacing significant HTML characters with eqivalent
 * escape sequences.
 * translation stuff.
 *     b: Buffer
 *     t: String to print
 ***********************************************************************/

void html_quote_string(struct buffer *b, char *t)
{
    unsigned char *s = (unsigned char *) t;
    unsigned char c;

    if (!s)
        bputs(b, "(nil)");
    else
        while ((c = *s++))
            html_quote_char(b, c);
}

/* ====================================================================== */

/* html_quote_linewrap() ************************************************
 *
 * Print string replacing significant HTML characters with eqivalent
 * escape sequences and wraping long lines.
 * translation stuff.
 *           b: Buffer
 *           t: String to print
 * line_length: Line length to wrap at
 *      indent: Indent wrapped lines (used for wrapping address lists)
 ***********************************************************************/

void
html_quote_linewrap(struct buffer *b, char *t,
                    unsigned long line_length, BOOL indent)
{
    unsigned char *s = (unsigned char *) t;
    unsigned char c;
    unsigned long count;
    unsigned long offset = 0L;

    if (!s)
        bputs(b, "(nil)");
    else
        while ((c = *s++)) {
            switch (c) {
            case '\015':
                if (*s == '\012')
                    s++;
            case '\012':
                /* Convert CR, LF and CRLF into CRLF */
                bputs(b, "" CRLF);
                offset = 0;
                break;
            case '\t':
                count = 8 - (offset % 8);
                if ((offset + count) > line_length) {
                    /* Fold long line */
                    if (indent) {
                        bputs(b, CRLF "    ");
                        offset = 4;
                    } else {
                        bputs(b, "" CRLF);
                        offset = 0;
                    }
                    while ((*s == ' ') || (*s == '\t'))
                        s++;
                } else {
                    offset += count;
                    while (count-- > 0)
                        bputc(b, ' ');
                }
                break;
            case ' ':
                /* Should include next word */
                if (offset >= line_length) {
                    /* Fold long line */
                    if (indent) {
                        bputs(b, CRLF "    ");
                        offset = 4;
                    } else {
                        bputs(b, "" CRLF);
                        offset = 0;
                    }
                    while ((*s == ' ') || (*s == '\t'))
                        s++;
                } else {
                    bputc(b, ' ');
                    offset++;
                }
                break;
            default:
                html_quote_char(b, c);
                offset++;
                break;
            }
        }
}
