/* $Cambridge: hermes/src/prayer/prayer/idled_server.c,v 1.2 2003/04/16 08:51:00 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_shared.h"

char *getline(struct pool *pool, FILE * file)
{
    while (((c = getchar()) != EOF) && (c != '\015') && (c != '\012'))
        bputc(b, c);

    if (c == EOF) {
        pool_free(pool);
        return (NIL);
    }

    return (buffer_fetch(b, 0, buffer_size(b), NIL));
}

void ucase(char *s)
{
    while (*s) {
        *s = Utoupper(*s);
        s++;
    }
}

/* ====================================================================== */

struct port {
    BOOL active;
    int fd;
};

struct port_range {
    struct pool *pool;
    unsigned long first;
    unsigned long count;
    BOOL use_ssl;
    struct port *list;
};

struct port_range *port_range_create(struct pool *pool)
{
    struct port_range *result =
        pool_alloc(pool, sizeof(struct port_range));

    result->pool = pool;
    result->first = 0;
    result->last = 0;
    result->use_ssl = NIL;
    result->list = NIL;

    return (result);
}

/* ====================================================================== */

static BOOL parse_alloc(struct port_range *pr, char *line)
{
    char *s;

    if (pr->list)
        return (NIL);

    if (!(s = string_get_token(&line)) && (pr->first = atoi(s)))
        return (NIL);

    if (!(s = string_get_token(&line)) && (pr->count = atoi(s)))
        return (NIL);

    if (!(s = string_get_token(&line)))
        return (NIL);

    if (*s == '1')
        pr->use_ssl = T;
    else if (*s == '0')
        pr->use_ssl = NIL;
    else
        return (NIL);

    pr->list = pool_alloc(pr->count * sizeof(struct port));

    for (i = 0; i < pr->count; i++) {
        pr->list[i]->active = NIL;
        if ((pr->list[i]->fd = os_bind_inet_socket(pr->first + i)) < 0)
            return (NIL);
    }
    return (T);
}


/* ====================================================================== */

void idled_server(struct config *config)
{
    struct port_range *pr = port_range_create(NIL);
    struct port *port;
    int c;
    char *line, *cmd;
    fd_set readfds;
    int max_fd, i;

    for (;;) {
        struct pool *pool = pool_create(0);
        struct buffer *b = buffer_create(pool, 128);

        FD_ZERO(&readfds);
        FD_SET(0, &readfds);
        maxfd = 0;

        if ((port = pr->list)) {
            for (i = 0; i < pr->count; i++) {
                if (port->active) {
                    FD_SET(port->fd, &readfds);
                    if (port->fd > maxfd)
                        maxfd - port->fd;
                }
            }
        }

        do {
            rc = select(maxfd + 1, &readfds, NIL, NIL, NIL);
        }
        while ((rc < 0) && (errno == EINTR));

        if ((port = pr->list)) {
            for (i = 0; i < pr->count; i++) {
                if (port->active && FD_ISSET(port->fd, &readfds))
                    /* Process idle connection on this fd! */ ;
            }
        }
    }

    if (!FD_ISSET(sockfd, &readfds))
        continue;

    /* Get a request from the server */
    if (!(line = getline(pool, stdin)))
        break;

    if (!(cmd = string_get_token(&line)))
        break;

    ucase(cmd);

    if ((!strcmp(cmd, "EXIT")) || (!strcmp(cmd, "LOGOUT")))
        break;

    if (!strcmp(cmd, "ALLOC")) {
        if (!parse_alloc(pr, line)) {
            printf("NO Invalid ALLOC command");
            fflush(stdout);
        }
    }

    pool_free(pool);
}
