/* $Cambridge: hermes/src/prayer/prayer/session_exchange.c,v 1.1.1.1 2003/04/15 13:00:03 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

/* session_exchange() ***************************************************
 *
 * Manage single HTTP exchange with user agent (can be initiated by
 * proxy frontend or direct connection to session server).
 ***********************************************************************/

BOOL session_exchange(struct session * session)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct user_agent *user_agent = request->user_agent;
    char *sequence;

    /* Do something useful with HTTP/0.9 */
    if (request->major == 0) {
        response_0_9_error(request);
        return (NIL);
    }

    /* Check for valid host if one supplied */
    if (request->url_host) {
        BOOL rc = NIL;

        if (!strcasecmp(request->url_host, "localhost"))
            rc = T;
        else if (config->hostname &&
                 !strcasecmp(request->url_host, config->hostname))
            rc = T;
        else if (config->hostname_service &&
                 !strcasecmp(request->url_host, config->hostname_service))
            rc = T;

        if (!rc) {
            response_error(request, 403);       /* Forbidden */
            return (NIL);
        }
    }

    /* Check IMAP streams before processing each session request
     * (reopens streams if required + ping every once in a while
     * Have to do this here rather than in parent routines so that we
     * have a HTTP connection back to client for restart page... */
    if (!session_streams_check(session)) {
        session_redirect(session, request, "restart");
        return (T);
    }

    request_parse_argv(request);
    request_parse_charset(request);

    if (session->use_gzip) {
        request->allow_gzip = T;
        request_parse_encoding(request);
    } else
        request->allow_gzip = NIL;

    /* XXX Everything apart from /session redundant if no direct connect! */

    /* Deal with icons */
    if (config->icon_dir &&
        (request->argc == 2) && !strcmp(request->argv[0], "icons")) {
        char *filename = pool_strcat3(request->pool, config->icon_dir, "/",
                                      request->argv[1]);

        if ((request->method == GET) || (request->method == HEAD)) {
            return (response_file(request, request->method,
                                  filename, config->icon_expire_timeout,
                                  assoc_lookup(request->hdrs,
                                               "if-modified-since")));
        }
        response_error(request, 501);   /* Not implemented */
        return (NIL);
    }

    /* Deal with static pages */
    if (config->static_dir &&
        (request->argc == 2) && !strcmp(request->argv[0], "static")) {
        char *filename
            = pool_strcat3(request->pool, config->static_dir, "/",
                           request->argv[1]);

        if ((request->method == GET) || (request->method == HEAD)) {
            return (response_file(request, request->method,
                                  filename, config->icon_expire_timeout,
                                  assoc_lookup(request->hdrs,
                                               "if-modified-since")));
        }
        response_error(request, 501);   /* Not implemented */
        return (NIL);
    }

    /* Redirect URL */
    if ((request->argc == 2) && !strcmp(request->argv[0], "redirect")) {
        char *url = string_url_decode(request->argv[1]);

        response_redirect(request, url);
        return (T);
    }

    /* Cope with "/robots.txt" */
    if ((request->argc == 1)
        && (!strcasecmp(request->argv[0], "robots.txt"))) {
        struct buffer *b = request->write_buffer;

        bputs(b, "User-agent: *" CRLF);
        bputs(b, "Disallow: /session" CRLF);
        bputs(b, "Disallow: /login" CRLF);
        bputs(b, "Disallow: /icons" CRLF);
        response_text(request, 200);
        return (NIL);
    }

    if (request->argc < 3) {
        response_error(request, 404);
        return (NIL);
    }

    if (request->argc == 5) {
        /* XXX Following all redundant if direct enable and use_short disabled */
        /* Send an icon */
        if (config->icon_dir &&
            (request->argc == 5) && !strcmp(request->argv[3], "icons")) {
            char *filename
                = pool_strcat3(request->pool, config->icon_dir, "/",
                               request->argv[4]);

            if ((request->method == GET) || (request->method == HEAD))
                return (response_file(request, request->method,
                                      filename,
                                      config->icon_expire_timeout,
                                      assoc_lookup(request->hdrs,
                                                   "if-modified-since")));

            response_error(request, 501);       /* Not implemented */
            return (NIL);
        }

        /* Static pages */
        if (config->static_dir &&
            (request->argc == 5) && !strcmp(request->argv[3], "icons")) {
            char *filename
                = pool_strcat3(request->pool, config->icon_dir, "/",
                               request->argv[4]);

            if ((request->method == GET) || (request->method == HEAD))
                return (response_file(request, request->method,
                                      filename,
                                      config->icon_expire_timeout,
                                      assoc_lookup(request->hdrs,
                                                   "if-modified-since")));

            response_error(request, 501);       /* Not implemented */
            return (NIL);
        }

        /* Redirect URL */
        if ((request->argc == 5) && !strcmp(request->argv[3], "redirect")) {
            char *url = string_url_decode(request->argv[4]);

            response_redirect(request, url);
            return (T);
        }
    }

    /* Replace empty argv[2] with "CRSid=session" value if cookie exists */
    if (request->argv[2][0] == '\0') {
        char *s = assoc_lookup(request->hdrs, "cookie");
        char *t;
        char *u;
        char *key;

        if (config->http_cookie_use_port)
            key = pool_printf(request->pool, "%s:%lu",
                              request->argv[1], session->session_port);
        else
            key = request->argv[1];


        /* Need cookie parsing library? */
        while (s && *s && (t = strchr(s, '='))) {
            *t++ = '\0';
            if ((u = strchr(t, ';')))
                *u++ = '\0';

            if (!strcmp(string_trim_whitespace(s), key)) {
                request->argv[2] = string_trim_whitespace(t);
                break;
            }
            s = u;
        }
    }

    /* Decode username argument */
    string_url_decode(request->argv[1]);

    /* Check that username and sessionID correspond to this session... */
    if ((strcmp(request->argv[1], session->username) != 0) ||
        (strcmp(request->argv[2], session->sessionid) != 0)) {
        struct buffer *b = request->write_buffer;

        html_common_start(config, b, "Session disconnected");
        bputs(b, "<h3>" CRLF);
        bputs(b, "Couldn't connect to session. Maybe it timed out?" CRLF);
        bprintf(b,
                "<p>Click <a href=\"%s/login/%s\">here</a> to login again</p>"
                CRLF, session->url_prefix, request->argv[1]);
        bputs(b, "</h3>" CRLF);
        html_common_end(b);

        /* Do we need to kill session cookie? Next login should kill anyway */
        response_html(request, 404);
        return (NIL);
    }

    /* argv: /session/username/session/sequence... */

    if ((request->argc < 5) || (strcmp(request->argv[0], "session") != 0)) {
        response_error(request, 404);   /* Not found */
        return (NIL);
    }

    sequence = request->argv[3];
    request->argv = &request->argv[4];
    request->argc -= 4;

    session->sequence_okay = T;

    if (strcmp(sequence, "NOSEQ") != 0) {
        BOOL error = NIL;

        if (session->sequence != session_sequence(sequence))
            session->sequence_okay = NIL;

        session_bump_sequence(session);
        session_update_sequence(session);

        if (user_agent->use_substitution) {
            /* Check that the sequence number is correct:
             *   Browser back button doesn't work with page substituton enabled
             *   Spell checker and browser buttons don't get on at all well
             *    (flag forall methods would be sensible)
             */
            if (!session->sequence_okay)
                error = T;
        } else {
            /* Browser back button works fine without page substition
             * apart from folder changes.
             */

            if (session->sequence_last_change > session_sequence(sequence))
                error = T;
        }

        if (error) {
            char *last_cmd = session_last_cmd(session);

            if (last_cmd && last_cmd[0]) {
                session_message(session,
                                ("Browser history disabled as unsafe: "
                                 "use Webmail navigation icons and links"));
                session_redirect(session, request, last_cmd);
                return (T);
            }

            session_log(session, ("[session_exchange] "
                                  "Internal error: last command not recorded"));
            session_redirect(session, request, "restart");
            return (T);
        }
    }

    /* Reopen local domains every once in a while */
    session_config_local_domain_ping(session->config);

    session_log(session, "[session_exchange] %s", request->argv[0]);

    /* Look for defined session method */
    if (cmd_dispatch(session, request->argv[0]))
        return (request->persist);

    if ((request->method == GET) || (request->method == HEAD))
        response_error(request, 404);   /* Not found */
    else
        response_error(request, 501);   /* Not implemented */

    return (NIL);
}
